
open util/steps[Snapshot]

// Snapshot definition
    sig Snapshot extends BaseSnapshot {
        Example_v : one A
    }

/***************************** STATE SPACE ************************************/
    abstract sig SystemState extends StateLabel {}
    one sig Example extends SystemState {}

/*************************** TRANSITIONS SPACE ********************************/
    one sig Example_a1_to_a2 extends TransitionLabel {}
    one sig Example_a2_to_a3 extends TransitionLabel {}

    // Transition Example_a1_to_a2
    pred pre_Example_a1_to_a2[s:Snapshot] {
        Example in s.conf
        {
            (s.Example_v) = a1
        }
    }

    pred pos_Example_a1_to_a2[s, s':Snapshot] {
        s'.conf = s.conf - Example + {
            Example
        }
        {
            (s'.Example_v) = a2
        }
    }

    pred Example_a1_to_a2[s, s': Snapshot] {
        pre_Example_a1_to_a2[s]
        pos_Example_a1_to_a2[s, s']
        semantics_Example_a1_to_a2[s, s']
    }
    pred semantics_Example_a1_to_a2[s, s': Snapshot] {
        s'.taken = Example_a1_to_a2
    }
    // Transition Example_a2_to_a3
    pred pre_Example_a2_to_a3[s:Snapshot] {
        Example in s.conf
        {
            (s.Example_v) = a2
        }
    }

    pred pos_Example_a2_to_a3[s, s':Snapshot] {
        s'.conf = s.conf - Example + {
            Example
        }
        {
            (s'.Example_v) = a2
            (s'.Example_v) = a3
        }
    }

    pred Example_a2_to_a3[s, s': Snapshot] {
        pre_Example_a2_to_a3[s]
        pos_Example_a2_to_a3[s, s']
        semantics_Example_a2_to_a3[s, s']
    }
    pred semantics_Example_a2_to_a3[s, s': Snapshot] {
        s'.taken = Example_a2_to_a3
    }
/****************************** INITIAL CONDITIONS ****************************/
    pred init[s: Snapshot] {
        s.conf = {
            Example
        }
        no s.taken
        // Model specific constraints
        (s.Example_v) = a1
    }

/**************************** SIGNIFICANCE AXIOMS *****************************/
    /** Uncomment the operationsAxiom to find a significant scope. */
    pred significance {
        steps/reachabilityAxiom
        // operationsAxiom
    }
    run significance for 3 Snapshot, 0 EventLabel
        expect 1

    pred operationsAxiom {
        some s, s': Snapshot | Example_a1_to_a2[s, s']
        some s, s': Snapshot | Example_a2_to_a3[s, s']
    }

/***************************** MODEL DEFINITION *******************************/
    pred operation[s, s': Snapshot] {
        Example_a1_to_a2[s, s'] or
        Example_a2_to_a3[s, s']
    }

    pred small_step[s, s': Snapshot] {
        operation[s, s']
    }

    pred equals[s, s': Snapshot] {
        s'.conf = s.conf
        s'.taken = s.taken
        // Model specific declarations
        s'.Example_v = s.Example_v
    }

    fact {
        all s: Snapshot | s in initial iff init[s]
        all s, s': Snapshot | s->s' in nextStep iff small_step[s, s']
        all s, s': Snapshot | equals[s, s'] => s = s'
        significance
    }



    abstract sig A {}
    
    one sig a1, a2, a3 extends A {}
    
    pred eventually_a3  {
        ctl_mc[ef[{ s: Snapshot | {
            (s.Example_v) = a3
        }
        }]]
    }
    
    run eventually_a3 
        for 3 Snapshot, 0 EventLabel
    

