------------------------------ MODULE deadlock ------------------------------

EXTENDS Integers, Sequences, TLC

VARIABLES P, RA, RB, ct
vars == << P, RA, RB, ct >>

trains == 0..7
LA == 7
LB == 7

TypeOK ==
    /\ P \in [trains -> 0..6]
    /\ RA \in 0..8
    /\ RB \in 0..8
    /\ ct \in trains

T ==  \* train missions
    ( 0 :> (0:>1  @@ 1:>9  @@ 2:>10 @@ 3:>13 @@ 4:>15 @@ 5:>20 @@ 6:>23) @@
      1 :> (0:>3  @@ 1:>9  @@ 2:>10 @@ 3:>13 @@ 4:>15 @@ 5:>20 @@ 6:>24) @@
      2 :> (0:>5  @@ 1:>27 @@ 2:>11 @@ 3:>13 @@ 4:>16 @@ 5:>20 @@ 6:>25) @@
      3 :> (0:>7  @@ 1:>27 @@ 2:>11 @@ 3:>13 @@ 4:>16 @@ 5:>20 @@ 6:>26) @@
      4 :> (0:>23 @@ 1:>22 @@ 2:>17 @@ 3:>18 @@ 4:>11 @@ 5:>9  @@ 6:>2)  @@
      5 :> (0:>24 @@ 1:>22 @@ 2:>17 @@ 3:>18 @@ 4:>11 @@ 5:>9  @@ 6:>4)  @@
      6 :> (0:>25 @@ 1:>22 @@ 2:>17 @@ 3:>18 @@ 4:>12 @@ 5:>27 @@ 6:>6)  @@
      7 :> (0:>26 @@ 1:>22 @@ 2:>17 @@ 3:>18 @@ 4:>12 @@ 5:>27 @@ 6:>8) )

A ==  \* region A train constraints
    ( 0 :> (0:>0 @@ 1:>0 @@ 2:>0 @@ 3:>1  @@ 4:>0  @@ 5:>-1 @@ 6:>0) @@
      1 :> (0:>0 @@ 1:>0 @@ 2:>0 @@ 3:>1  @@ 4:>0  @@ 5:>-1 @@ 6:>0) @@
      2 :> (0:>0 @@ 1:>0 @@ 2:>1 @@ 3:>-1 @@ 4:>0  @@ 5:>1  @@ 6:>0) @@
      3 :> (0:>0 @@ 1:>0 @@ 2:>1 @@ 3:>-1 @@ 4:>0  @@ 5:>0  @@ 6:>0) @@
      4 :> (0:>0 @@ 1:>1 @@ 2:>0 @@ 3:>0  @@ 4:>-1 @@ 5:>0  @@ 6:>0) @@
      5 :> (0:>0 @@ 1:>1 @@ 2:>0 @@ 3:>0  @@ 4:>-1 @@ 5:>0  @@ 6:>0) @@
      6 :> (0:>0 @@ 1:>0 @@ 2:>0 @@ 3:>-1 @@ 4:>0  @@ 5:>0  @@ 6:>0) @@
      7 :> (0:>0 @@ 1:>1 @@ 2:>0 @@ 3:>-1 @@ 4:>0  @@ 5:>0  @@ 6:>0) )

B ==  \* region B train constraints
    ( 0 :> (0:>0 @@ 1:>0 @@ 2:>0 @@ 3:>1  @@ 4:>0  @@ 5:>-1 @@ 6:>0) @@
      1 :> (0:>0 @@ 1:>0 @@ 2:>0 @@ 3:>1  @@ 4:>0  @@ 5:>-1 @@ 6:>0) @@
      2 :> (0:>0 @@ 1:>0 @@ 2:>1 @@ 3:>-1 @@ 4:>0  @@ 5:>0  @@ 6:>0) @@
      3 :> (0:>0 @@ 1:>0 @@ 2:>1 @@ 3:>-1 @@ 4:>0  @@ 5:>1  @@ 6:>0) @@
      4 :> (0:>0 @@ 1:>1 @@ 2:>0 @@ 3:>0  @@ 4:>-1 @@ 5:>0  @@ 6:>0) @@
      5 :> (0:>0 @@ 1:>1 @@ 2:>0 @@ 3:>0  @@ 4:>-1 @@ 5:>0  @@ 6:>0) @@
      6 :> (0:>0 @@ 1:>1 @@ 2:>0 @@ 3:>-1 @@ 4:>0  @@ 5:>0  @@ 6:>0) @@
      7 :> (0:>0 @@ 1:>0 @@ 2:>0 @@ 3:>-1 @@ 4:>0  @@ 5:>0  @@ 6:>0) )

Init ==
    /\ P = [t \in trains |-> 0]
    /\ RA = 1
    /\ RB = 1
    /\ ct \in trains

move_train ==
    /\ /\ P[ct] < 6
       /\ \A t \in trains:
           t # ct => T[ct][P[ct] + 1] # T[t][P[t]]
       /\ RA + A[ct][P[ct] + 1] <= LA
       /\ RB + B[ct][P[ct] + 1] <= LB
    /\ P' = [P EXCEPT ![ct] = P[ct] + 1]
    /\ RA' = RA + A[ct][P'[ct]]
    /\ RB' = RB + B[ct][P'[ct]]
    /\ UNCHANGED ct

choose_train ==
    /\ IF ct < 7
          THEN ct' = ct + 1
          ELSE ct' = 0
    /\ UNCHANGED << P, RA, RB >>

Next ==
    \/ move_train
    \/ choose_train

Spec ==
    /\ Init /\ [][Next]_vars
    /\ SF_vars(move_train)
    /\ WF_vars(choose_train)

ASSERT_LTL == []<>(\A t \in trains: P[t] = 6)

=============================================================================
\* Modification History
\* Last modified Fri Jan 24 21:34:52 EST 2020 by bandali
\* Created Wed Jan 22 01:12:38 EST 2020 by bandali
