
open util/steps[Snapshot]
open util/ordering[Snapshot]
open util/boolean

// Snapshot definition
    sig Snapshot extends BaseSnapshot {
        stable: one Bool,
        events: set EventLabel,
    }

/***************************** STATE SPACE ************************************/
    abstract sig SystemState extends StateLabel {}
    abstract sig DigitalWatch extends SystemState {}
    abstract sig DigitalWatch_Light extends DigitalWatch {}
    one sig DigitalWatch_Light_Off extends DigitalWatch_Light {}
    one sig DigitalWatch_Light_On extends DigitalWatch_Light {}
    abstract sig DigitalWatch_Main extends DigitalWatch {}
    abstract sig DigitalWatch_Main_Displays extends DigitalWatch_Main {}
    one sig DigitalWatch_Main_Displays_Time extends DigitalWatch_Main_Displays {}
    one sig DigitalWatch_Main_Displays_Date extends DigitalWatch_Main_Displays {}
    one sig DigitalWatch_Main_Displays_Wait extends DigitalWatch_Main_Displays {}
    one sig DigitalWatch_Main_Displays_Update extends DigitalWatch_Main_Displays {}
    one sig DigitalWatch_Main_Displays_Alarm1 extends DigitalWatch_Main_Displays {}
    one sig DigitalWatch_Main_Displays_Alarm2 extends DigitalWatch_Main_Displays {}
    one sig DigitalWatch_Main_Displays_Chime extends DigitalWatch_Main_Displays {}
    one sig DigitalWatch_Main_Displays_StopWatch extends DigitalWatch_Main_Displays {}
    one sig DigitalWatch_Main_Alarms_Beep extends DigitalWatch_Main {}

/***************************** EVENTS SPACE ***********************************/
    one sig DigitalWatch_press_a extends EnvironmentEvent {}
    one sig DigitalWatch_release_a extends EnvironmentEvent {}
    one sig DigitalWatch_press_b extends EnvironmentEvent {}
    one sig DigitalWatch_release_b extends EnvironmentEvent {}
    one sig DigitalWatch_press_c extends EnvironmentEvent {}
    one sig DigitalWatch_release_c extends EnvironmentEvent {}
    one sig DigitalWatch_press_d extends EnvironmentEvent {}
    one sig DigitalWatch_release_d extends EnvironmentEvent {}
    one sig DigitalWatch_Main_Displays_Date_waited_2_min extends EnvironmentEvent {}
    one sig DigitalWatch_Main_Displays_Wait_waited_2_sec extends EnvironmentEvent {}

/*************************** TRANSITIONS SPACE ********************************/
    one sig DigitalWatch_Light_Off_light_on extends TransitionLabel {}
    one sig DigitalWatch_Light_On_light_off extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_Time_show_date extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_Time_try_update extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_Time_go2alarm1 extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_Date_show_time extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_Date_return_to_time extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_Wait_show_time extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_Wait_show_update extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_Update_show_time extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_Alarm1_go2alarm2 extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_Alarm2_go2chime extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_Chime_go2Stopwatch extends TransitionLabel {}
    one sig DigitalWatch_Main_Displays_StopWatch_go2Time extends TransitionLabel {}

    // Transition DigitalWatch_Light_Off_light_on
    pred pre_DigitalWatch_Light_Off_light_on[s:Snapshot] {
        DigitalWatch_Light_Off in s.conf
        s.stable = True => {
            DigitalWatch_press_b in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_press_b in s.events
        }
    }

    pred pos_DigitalWatch_Light_Off_light_on[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Light_Off + {
            DigitalWatch_Light_On
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Light_Off_light_on] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Light_Off_light_on[s, s': Snapshot] {
        pre_DigitalWatch_Light_Off_light_on[s]
        pos_DigitalWatch_Light_Off_light_on[s, s']
        semantics_DigitalWatch_Light_Off_light_on[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Light_Off_light_on[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Light_Off in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Light_On_light_off + 
                DigitalWatch_Light_Off_light_on
            }
            DigitalWatch_press_b in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Light_On_light_off + 
                DigitalWatch_Light_Off_light_on
            }
            DigitalWatch_press_b in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Light_Off_light_on[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Light_Off_light_on
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Light_Off_light_on
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Light_On_light_off + 
                DigitalWatch_Light_Off_light_on
            }
        }
    }
    // Transition DigitalWatch_Light_On_light_off
    pred pre_DigitalWatch_Light_On_light_off[s:Snapshot] {
        DigitalWatch_Light_On in s.conf
        s.stable = True => {
            DigitalWatch_release_b in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_release_b in s.events
        }
    }

    pred pos_DigitalWatch_Light_On_light_off[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Light_On + {
            DigitalWatch_Light_Off
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Light_On_light_off] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Light_On_light_off[s, s': Snapshot] {
        pre_DigitalWatch_Light_On_light_off[s]
        pos_DigitalWatch_Light_On_light_off[s, s']
        semantics_DigitalWatch_Light_On_light_off[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Light_On_light_off[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Light_On in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Light_On_light_off + 
                DigitalWatch_Light_Off_light_on
            }
            DigitalWatch_release_b in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Light_On_light_off + 
                DigitalWatch_Light_Off_light_on
            }
            DigitalWatch_release_b in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Light_On_light_off[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Light_On_light_off
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Light_On_light_off
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Light_On_light_off + 
                DigitalWatch_Light_Off_light_on
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_Time_show_date
    pred pre_DigitalWatch_Main_Displays_Time_show_date[s:Snapshot] {
        DigitalWatch_Main_Displays_Time in s.conf
        s.stable = True => {
            DigitalWatch_press_d in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_press_d in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_Time_show_date[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_Time + {
            DigitalWatch_Main_Displays_Date
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_Time_show_date] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_Time_show_date[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_Time_show_date[s]
        pos_DigitalWatch_Main_Displays_Time_show_date[s, s']
        semantics_DigitalWatch_Main_Displays_Time_show_date[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_Time_show_date[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_Time in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_d in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_d in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_Time_show_date[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_Time_show_date
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_Time_show_date
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_Time_try_update
    pred pre_DigitalWatch_Main_Displays_Time_try_update[s:Snapshot] {
        DigitalWatch_Main_Displays_Time in s.conf
        s.stable = True => {
            DigitalWatch_press_c in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_press_c in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_Time_try_update[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_Time + {
            DigitalWatch_Main_Displays_Wait
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_Time_try_update] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_Time_try_update[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_Time_try_update[s]
        pos_DigitalWatch_Main_Displays_Time_try_update[s, s']
        semantics_DigitalWatch_Main_Displays_Time_try_update[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_Time_try_update[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_Time in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_c in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_c in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_Time_try_update[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_Time_try_update
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_Time_try_update
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_Time_go2alarm1
    pred pre_DigitalWatch_Main_Displays_Time_go2alarm1[s:Snapshot] {
        DigitalWatch_Main_Displays_Time in s.conf
        s.stable = True => {
            DigitalWatch_press_a in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_press_a in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_Time_go2alarm1[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_Time + {
            DigitalWatch_Main_Displays_Alarm1
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_Time_go2alarm1] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_Time_go2alarm1[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_Time_go2alarm1[s]
        pos_DigitalWatch_Main_Displays_Time_go2alarm1[s, s']
        semantics_DigitalWatch_Main_Displays_Time_go2alarm1[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_Time_go2alarm1[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_Time in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_a in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_a in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_Time_go2alarm1[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_Time_go2alarm1
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_Time_go2alarm1
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_Date_show_time
    pred pre_DigitalWatch_Main_Displays_Date_show_time[s:Snapshot] {
        DigitalWatch_Main_Displays_Date in s.conf
        s.stable = True => {
            DigitalWatch_press_d in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_press_d in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_Date_show_time[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_Date + {
            DigitalWatch_Main_Displays_Time
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_Date_show_time] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_Date_show_time[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_Date_show_time[s]
        pos_DigitalWatch_Main_Displays_Date_show_time[s, s']
        semantics_DigitalWatch_Main_Displays_Date_show_time[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_Date_show_time[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_Date in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_d in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_d in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_Date_show_time[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_Date_show_time
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_Date_show_time
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_Date_return_to_time
    pred pre_DigitalWatch_Main_Displays_Date_return_to_time[s:Snapshot] {
        DigitalWatch_Main_Displays_Date in s.conf
        s.stable = True => {
            DigitalWatch_Main_Displays_Date_waited_2_min in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_Main_Displays_Date_waited_2_min in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_Date_return_to_time[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_Date + {
            DigitalWatch_Main_Displays_Time
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_Date_return_to_time] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_Date_return_to_time[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_Date_return_to_time[s]
        pos_DigitalWatch_Main_Displays_Date_return_to_time[s, s']
        semantics_DigitalWatch_Main_Displays_Date_return_to_time[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_Date_return_to_time[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_Date in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Date_return_to_time + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update
            }
            DigitalWatch_Main_Displays_Date_waited_2_min in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Date_return_to_time + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update
            }
            DigitalWatch_Main_Displays_Date_waited_2_min in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_Date_return_to_time[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_Date_return_to_time
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_Date_return_to_time
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Date_return_to_time + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_Wait_show_time
    pred pre_DigitalWatch_Main_Displays_Wait_show_time[s:Snapshot] {
        DigitalWatch_Main_Displays_Wait in s.conf
        s.stable = True => {
            DigitalWatch_release_c in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_release_c in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_Wait_show_time[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_Wait + {
            DigitalWatch_Main_Displays_Time
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_Wait_show_time] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_Wait_show_time[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_Wait_show_time[s]
        pos_DigitalWatch_Main_Displays_Wait_show_time[s, s']
        semantics_DigitalWatch_Main_Displays_Wait_show_time[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_Wait_show_time[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_Wait in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_release_c in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_release_c in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_Wait_show_time[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_Wait_show_time
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_Wait_show_time
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_Wait_show_update
    pred pre_DigitalWatch_Main_Displays_Wait_show_update[s:Snapshot] {
        DigitalWatch_Main_Displays_Wait in s.conf
        s.stable = True => {
            DigitalWatch_Main_Displays_Wait_waited_2_sec in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_Main_Displays_Wait_waited_2_sec in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_Wait_show_update[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_Wait + {
            DigitalWatch_Main_Displays_Update
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_Wait_show_update] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_Wait_show_update[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_Wait_show_update[s]
        pos_DigitalWatch_Main_Displays_Wait_show_update[s, s']
        semantics_DigitalWatch_Main_Displays_Wait_show_update[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_Wait_show_update[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_Wait in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_Main_Displays_Wait_waited_2_sec in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_Main_Displays_Wait_waited_2_sec in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_Wait_show_update[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_Wait_show_update
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_Wait_show_update
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_Update_show_time
    pred pre_DigitalWatch_Main_Displays_Update_show_time[s:Snapshot] {
        DigitalWatch_Main_Displays_Update in s.conf
        s.stable = True => {
            DigitalWatch_press_b in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_press_b in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_Update_show_time[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_Update + {
            DigitalWatch_Main_Displays_Time
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_Update_show_time] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_Update_show_time[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_Update_show_time[s]
        pos_DigitalWatch_Main_Displays_Update_show_time[s, s']
        semantics_DigitalWatch_Main_Displays_Update_show_time[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_Update_show_time[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_Update in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_b in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_b in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_Update_show_time[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_Update_show_time
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_Update_show_time
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_Alarm1_go2alarm2
    pred pre_DigitalWatch_Main_Displays_Alarm1_go2alarm2[s:Snapshot] {
        DigitalWatch_Main_Displays_Alarm1 in s.conf
        s.stable = True => {
            DigitalWatch_press_a in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_press_a in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_Alarm1_go2alarm2[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_Alarm1 + {
            DigitalWatch_Main_Displays_Alarm2
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_Alarm1_go2alarm2] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_Alarm1_go2alarm2[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_Alarm1_go2alarm2[s]
        pos_DigitalWatch_Main_Displays_Alarm1_go2alarm2[s, s']
        semantics_DigitalWatch_Main_Displays_Alarm1_go2alarm2[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_Alarm1_go2alarm2[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_Alarm1 in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_a in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_a in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_Alarm1_go2alarm2[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_Alarm1_go2alarm2
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_Alarm1_go2alarm2
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_Alarm2_go2chime
    pred pre_DigitalWatch_Main_Displays_Alarm2_go2chime[s:Snapshot] {
        DigitalWatch_Main_Displays_Alarm2 in s.conf
        s.stable = True => {
            DigitalWatch_press_a in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_press_a in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_Alarm2_go2chime[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_Alarm2 + {
            DigitalWatch_Main_Displays_Chime
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_Alarm2_go2chime] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_Alarm2_go2chime[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_Alarm2_go2chime[s]
        pos_DigitalWatch_Main_Displays_Alarm2_go2chime[s, s']
        semantics_DigitalWatch_Main_Displays_Alarm2_go2chime[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_Alarm2_go2chime[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_Alarm2 in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_a in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_a in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_Alarm2_go2chime[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_Alarm2_go2chime
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_Alarm2_go2chime
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_Chime_go2Stopwatch
    pred pre_DigitalWatch_Main_Displays_Chime_go2Stopwatch[s:Snapshot] {
        DigitalWatch_Main_Displays_Chime in s.conf
        s.stable = True => {
            DigitalWatch_press_a in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_press_a in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_Chime_go2Stopwatch[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_Chime + {
            DigitalWatch_Main_Displays_StopWatch
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_Chime_go2Stopwatch] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_Chime_go2Stopwatch[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_Chime_go2Stopwatch[s]
        pos_DigitalWatch_Main_Displays_Chime_go2Stopwatch[s, s']
        semantics_DigitalWatch_Main_Displays_Chime_go2Stopwatch[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_Chime_go2Stopwatch[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_Chime in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_a in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_a in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_Chime_go2Stopwatch[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_Chime_go2Stopwatch
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_Chime_go2Stopwatch
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
        }
    }
    // Transition DigitalWatch_Main_Displays_StopWatch_go2Time
    pred pre_DigitalWatch_Main_Displays_StopWatch_go2Time[s:Snapshot] {
        DigitalWatch_Main_Displays_StopWatch in s.conf
        s.stable = True => {
            DigitalWatch_press_a in (s.events & EnvironmentEvent)
        } else {
            DigitalWatch_press_a in s.events
        }
    }

    pred pos_DigitalWatch_Main_Displays_StopWatch_go2Time[s, s':Snapshot] {
        s'.conf = s.conf - DigitalWatch_Main_Displays_StopWatch + {
            DigitalWatch_Main_Displays_Time
        }
    
        testIfNextStable[s, s', {none}, DigitalWatch_Main_Displays_StopWatch_go2Time] => {
            s'.stable = True
            s.stable = True => {
                no ((s'.events & InternalEvent) )
            } else {
                no ((s'.events & InternalEvent) - { (InternalEvent & s.events)})
            }
        } else {
            s'.stable = False
            s.stable = True => {
                s'.events & InternalEvent = {none}
                s'.events & EnvironmentEvent = s.events & EnvironmentEvent
            } else {
                s'.events = s.events + {none}
            }
        }
    }

    pred DigitalWatch_Main_Displays_StopWatch_go2Time[s, s': Snapshot] {
        pre_DigitalWatch_Main_Displays_StopWatch_go2Time[s]
        pos_DigitalWatch_Main_Displays_StopWatch_go2Time[s, s']
        semantics_DigitalWatch_Main_Displays_StopWatch_go2Time[s, s']
    }

    pred enabledAfterStep_DigitalWatch_Main_Displays_StopWatch_go2Time[_s, s: Snapshot, t: TransitionLabel, genEvents: set InternalEvent] {
        // Preconditions
        DigitalWatch_Main_Displays_StopWatch in s.conf
        _s.stable = True => {
            no t & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_a in {(_s.events & EnvironmentEvent)  + genEvents}
        } else {
            no {_s.taken + t} & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
            DigitalWatch_press_a in {_s.events  + genEvents}
        }
    }
    pred semantics_DigitalWatch_Main_Displays_StopWatch_go2Time[s, s': Snapshot] {
        (s.stable = True) => {
            // SINGLE semantics
            s'.taken = DigitalWatch_Main_Displays_StopWatch_go2Time
        } else {
            // SINGLE semantics
            s'.taken = s.taken + DigitalWatch_Main_Displays_StopWatch_go2Time
            // Bigstep "TAKE_ONE" semantics
            no s.taken & {
                DigitalWatch_Main_Displays_StopWatch_go2Time + 
                DigitalWatch_Main_Displays_Alarm2_go2chime + 
                DigitalWatch_Main_Displays_Date_show_time + 
                DigitalWatch_Main_Displays_Wait_show_time + 
                DigitalWatch_Main_Displays_Update_show_time + 
                DigitalWatch_Main_Displays_Alarm1_go2alarm2 + 
                DigitalWatch_Main_Displays_Time_show_date + 
                DigitalWatch_Main_Displays_Time_go2alarm1 + 
                DigitalWatch_Main_Displays_Chime_go2Stopwatch + 
                DigitalWatch_Main_Displays_Wait_show_update + 
                DigitalWatch_Main_Displays_Time_try_update + 
                DigitalWatch_Main_Displays_Date_return_to_time
            }
        }
    }
/****************************** INITIAL CONDITIONS ****************************/
    pred init[s: Snapshot] {
        s.conf = {
            DigitalWatch_Light_Off + 
            DigitalWatch_Main_Displays + 
            DigitalWatch_Main_Displays_Time
        }
        no s.taken
        s.stable = True
        no s.events & InternalEvent
    }


/***************************** MODEL DEFINITION *******************************/
    pred operation[s, s': Snapshot] {
        DigitalWatch_Light_Off_light_on[s, s'] or
        DigitalWatch_Light_On_light_off[s, s'] or
        DigitalWatch_Main_Displays_Time_show_date[s, s'] or
        DigitalWatch_Main_Displays_Time_try_update[s, s'] or
        DigitalWatch_Main_Displays_Time_go2alarm1[s, s'] or
        DigitalWatch_Main_Displays_Date_show_time[s, s'] or
        DigitalWatch_Main_Displays_Date_return_to_time[s, s'] or
        DigitalWatch_Main_Displays_Wait_show_time[s, s'] or
        DigitalWatch_Main_Displays_Wait_show_update[s, s'] or
        DigitalWatch_Main_Displays_Update_show_time[s, s'] or
        DigitalWatch_Main_Displays_Alarm1_go2alarm2[s, s'] or
        DigitalWatch_Main_Displays_Alarm2_go2chime[s, s'] or
        DigitalWatch_Main_Displays_Chime_go2Stopwatch[s, s'] or
        DigitalWatch_Main_Displays_StopWatch_go2Time[s, s']
    }

    pred small_step[s, s': Snapshot] {
        operation[s, s']
    }

    pred testIfNextStable[s, s': Snapshot, genEvents: set InternalEvent, t:TransitionLabel] {
        !enabledAfterStep_DigitalWatch_Light_Off_light_on[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Light_On_light_off[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_Time_show_date[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_Time_try_update[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_Time_go2alarm1[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_Date_show_time[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_Date_return_to_time[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_Wait_show_time[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_Wait_show_update[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_Update_show_time[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_Alarm1_go2alarm2[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_Alarm2_go2chime[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_Chime_go2Stopwatch[s, s', t, genEvents]
        !enabledAfterStep_DigitalWatch_Main_Displays_StopWatch_go2Time[s, s', t, genEvents]
    }

    pred isEnabled[s:Snapshot] {
        pre_DigitalWatch_Light_Off_light_on[s]or
        pre_DigitalWatch_Light_On_light_off[s]or
        pre_DigitalWatch_Main_Displays_Time_show_date[s]or
        pre_DigitalWatch_Main_Displays_Time_try_update[s]or
        pre_DigitalWatch_Main_Displays_Time_go2alarm1[s]or
        pre_DigitalWatch_Main_Displays_Date_show_time[s]or
        pre_DigitalWatch_Main_Displays_Date_return_to_time[s]or
        pre_DigitalWatch_Main_Displays_Wait_show_time[s]or
        pre_DigitalWatch_Main_Displays_Wait_show_update[s]or
        pre_DigitalWatch_Main_Displays_Update_show_time[s]or
        pre_DigitalWatch_Main_Displays_Alarm1_go2alarm2[s]or
        pre_DigitalWatch_Main_Displays_Alarm2_go2chime[s]or
        pre_DigitalWatch_Main_Displays_Chime_go2Stopwatch[s]or
        pre_DigitalWatch_Main_Displays_StopWatch_go2Time[s]
    }

    pred equals[s, s': Snapshot] {
        s'.conf = s.conf
        s'.events = s.events
        s'.taken = s.taken
    }

    fact {
        all s: Snapshot | s in initial iff init[s]
        all s, s': Snapshot | s->s' in nextStep iff small_step[s, s']
        all s, s': Snapshot | equals[s, s'] => s = s'
        all s: Snapshot | (isEnabled[s] && no s': Snapshot | small_step[s, s']) => s.stable = False
        all s: Snapshot | s.stable = False => some s.nextStep
        path
    }

    pred path {
        all s:Snapshot, s': s.next | operation[s, s']
        init[first]
    }
    run path for 5 Snapshot, 10 EventLabel
        expect 1



    
    // Significant model
    run significance for 10 Snapshot, exactly 10 EventLabel
    
    // *********************** Eventually time ************************//
        pred eventually_time {
            ctl_mc[
                ag[
                    imp_[{ s : Snapshot | DigitalWatch_press_a in s.events},
                        ef[ex[{s: Snapshot | DigitalWatch_Main_Displays_Time in s.conf }]]
                    ]
                ]
            ]
        }
        run eventually_time for 10 Snapshot, exactly 10 EventLabel
    
        assert check_eventually_time {
            eventually_time
        }
        check check_eventually_time for 10 Snapshot, exactly 10 EventLabel expect 0
    

