asm ehealth

import StandardLibrary

signature:
	// domains
	enum domain RuleId = {ADD_PATIENT
						| ADD_MEDICATION
						| ADD_INTERACTION
						| ADD_PRESCRIPTION
						| REMOVE_INTERACTION
						| REMOVE_PRESCRIPTION}
	enum domain Patients = {PP1 | PP2 | PP3}
	enum domain Medications = {MM1 | MM2 | MM3}

	// functions
	controlled patients: Patients -> Boolean
	controlled medications: Medications -> Boolean
	controlled interactions: Prod(Medications, Medications) -> Boolean
	controlled prescriptions: Prod(Patients, Medications) -> Boolean

definitions:

	rule r_add_patient($p in Patients) =
		if (not patients($p)) then
			patients($p) := true
		endif

	rule r_add_medication($m in Medications) =
		if (not medications($m)) then
			medications($m) := true
		endif

	rule r_add_interaction($m1 in Medications, $m2 in Medications) =
		if (medications($m1) and medications($m2)
				and $m1 != $m2
				and not (exist $p in Patients with (prescriptions($p, $m1)
												or  prescriptions($p, $m2)))) then
			par
				interactions($m1, $m2) := true
				interactions($m2, $m1) := true
			endpar
		endif

	rule r_add_prescription($p in Patients, $m in Medications) =
		if (patients($p) and medications($m)
				and not prescriptions($p, $m)
				and not (exist $m0 in Medications with (prescriptions($p, $m0)
													and interactions($m, $m0)))) then
			prescriptions($p, $m) := true
		endif

	rule r_remove_interaction($m1 in Medications, $m2 in Medications) =
		if (medications($m1) and medications($m2) and interactions($m1, $m2)) then
			par
				interactions($m1, $m2) := false
				interactions($m2, $m1) := false
			endpar
		endif

	rule r_remove_prescription($p in Patients, $m in Medications) =
		if (patients($p) and medications($m) and prescriptions($p, $m)) then
			prescriptions($p, $m) := false
		endif

	invariant over interactions:
		(forall $m1 in Medications, $m2 in Medications
			with (medications($m1) and medications($m2) and interactions($m1, $m2))
			implies interactions($m2, $m1))

	invariant over interactions:
		(forall $m in Medications with medications($m) implies not interactions($m, $m))

	invariant over prescriptions:
		(forall $m1 in Medications, $m2 in Medications, $p in Patients
			with (medications($m1) and medications($m2) and patients($p)
					and interactions($m1, $m2))
			implies not (prescriptions($p, $m1) and prescriptions($p, $m2)))

	main rule r_Main =
		choose $p in Patients, $m1 in Medications, $m2 in Medications, $rule in RuleId with true do
			switch($rule)
				case ADD_PATIENT:
					r_add_patient[$p]
				case ADD_MEDICATION:
					r_add_medication[$m1]
				case ADD_INTERACTION:
					r_add_interaction[$m1, $m2]
				case ADD_PRESCRIPTION:
					r_add_prescription[$p, $m1]
				case REMOVE_INTERACTION:
					r_remove_interaction[$m1, $m2]
				case REMOVE_PRESCRIPTION:
					r_remove_prescription[$p, $m1]
			endswitch

default init s0:
	function patients($p in Patients) = false
	function medications($m in Medications) = false
	function interactions($m1 in Medications, $m2 in Medications) = false
	function prescriptions($p in Patients, $m in Medications) = false
