-------------------------- MODULE ehealth_pluscal --------------------------

EXTENDS Naturals, Sequences, TLC

CONSTANTS
  Patients,   \* set of all patients in the universe
  Medications \* set of all medications in the universe

(* --algorithm ehealth_pluscal
variables patients = {},
          medications = {},
          interactions = {},
          prescriptions = {};

procedure add_patient(p) begin
  if
    /\ p \in Patients
    /\ p \notin patients
  then
    patients := patients \union {p}
  end if
end procedure

procedure add_medication(m) begin
  if
    /\ m \in Medications
    /\ m \notin medications
  then
    medications := medications \union {m}
  end if
end procedure

procedure add_interaction(m1, m2) begin
  if
    /\ m1 \in medications
    /\ m2 \in medications
    /\ m1 /= m2
    /\ <<m1, m2>> \notin interactions
    /\ \A ps \in patients: ~ (<<p_, m1>> \in prescriptions
                           /\ <<p_, m2>> \in prescriptions)
  then
    interactions := interactions \union {<<m1, m2>>, <<m2, m1>>}
  end if
end procedure

procedure add_prescription(p, m) begin
  if
    /\ p \in patients
    /\ m \in medications
    /\ <<p, m>> \notin prescriptions
    /\ \A x \in medications: <<p, x>> \in prescriptions =>
        <<x, m>> \notin interactions
  then
    prescriptions := prescriptions \union {<<p, m>>}
  end if
end procedure

procedure remove_interaction(m1, m2) begin
  if
    /\ m1 \in medications
    /\ m2 \in medications
    /\ <<m1, m2>> \in interactions
  then
    interactions := interactions \ {<<m1, m2>>, <<m2, m1>>}
  end if
end procedure

procedure remove_prescription(p, m) begin
  if
    /\ p \in patients
    /\ m \in medications
    /\ <<p, m>> \in prescriptions
  then
    prescriptions := prescriptions \ {<<p, m>>}
  end if
end procedure

begin
  with _p \in Patients do
    call add_patient(_p);
  end with;
  with _m \in Medications do
    call add_medication(_m);
  end with;
  with _m1 \in medications do
    with _m2 \in medications do
      either
        call add_interaction(_m1, _m2);
      or
        call remove_interaction(_m1, _m2);
      end either
    end with
  end with;
  with _p \in patients do
    with _m \in medications do
      either
        call add_prescription(_p, _m);
      or
        call remove_prescription(_p, _m);
      end either
    end with
  end with;
end algorithm; *)

\* BEGIN TRANSLATION
\* Parameter p of procedure add_patient at line 15 col 23 changed to p_
\* Parameter m of procedure add_medication at line 24 col 26 changed to m_
\* Parameter m1 of procedure add_interaction at line 33 col 27 changed to m1_
\* Parameter m2 of procedure add_interaction at line 33 col 31 changed to m2_
\* Parameter p of procedure add_prescription at line 46 col 28 changed to p_a
\* Parameter m of procedure add_prescription at line 46 col 31 changed to m_a
CONSTANT defaultInitValue
VARIABLES patients, medications, interactions, prescriptions, pc, stack, p_, 
          m_, m1_, m2_, p_a, m_a, m1, m2, p, m

vars == << patients, medications, interactions, prescriptions, pc, stack, p_, 
           m_, m1_, m2_, p_a, m_a, m1, m2, p, m >>

Init == (* Global variables *)
        /\ patients = {}
        /\ medications = {}
        /\ interactions = {}
        /\ prescriptions = {}
        (* Procedure add_patient *)
        /\ p_ = defaultInitValue
        (* Procedure add_medication *)
        /\ m_ = defaultInitValue
        (* Procedure add_interaction *)
        /\ m1_ = defaultInitValue
        /\ m2_ = defaultInitValue
        (* Procedure add_prescription *)
        /\ p_a = defaultInitValue
        /\ m_a = defaultInitValue
        (* Procedure remove_interaction *)
        /\ m1 = defaultInitValue
        /\ m2 = defaultInitValue
        (* Procedure remove_prescription *)
        /\ p = defaultInitValue
        /\ m = defaultInitValue
        /\ stack = << >>
        /\ pc = "Lbl_7"

Lbl_1 == /\ pc = "Lbl_1"
         /\ IF /\ p_ \in Patients
               /\ p_ \notin patients
               THEN /\ patients' = (patients \union {p_})
               ELSE /\ TRUE
                    /\ UNCHANGED patients
         /\ pc' = "Error"
         /\ UNCHANGED << medications, interactions, prescriptions, stack, p_, 
                         m_, m1_, m2_, p_a, m_a, m1, m2, p, m >>

add_patient == Lbl_1

Lbl_2 == /\ pc = "Lbl_2"
         /\ IF /\ m_ \in Medications
               /\ m_ \notin medications
               THEN /\ medications' = (medications \union {m_})
               ELSE /\ TRUE
                    /\ UNCHANGED medications
         /\ pc' = "Error"
         /\ UNCHANGED << patients, interactions, prescriptions, stack, p_, m_, 
                         m1_, m2_, p_a, m_a, m1, m2, p, m >>

add_medication == Lbl_2

Lbl_3 == /\ pc = "Lbl_3"
         /\ IF /\ m1_ \in medications
               /\ m2_ \in medications
               /\ m1_ /= m2_
               /\ <<m1_, m2_>> \notin interactions
               /\ \A ps \in patients: ~ (<<p_, m1_>> \in prescriptions
                                      /\ <<p_, m2_>> \in prescriptions)
               THEN /\ interactions' = (interactions \union {<<m1_, m2_>>, <<m2_, m1_>>})
               ELSE /\ TRUE
                    /\ UNCHANGED interactions
         /\ pc' = "Error"
         /\ UNCHANGED << patients, medications, prescriptions, stack, p_, m_, 
                         m1_, m2_, p_a, m_a, m1, m2, p, m >>

add_interaction == Lbl_3

Lbl_4 == /\ pc = "Lbl_4"
         /\ IF /\ p_a \in patients
               /\ m_a \in medications
               /\ <<p_a, m_a>> \notin prescriptions
               /\ \A x \in medications: <<p_a, x>> \in prescriptions =>
                   <<x, m_a>> \notin interactions
               THEN /\ prescriptions' = (prescriptions \union {<<p_a, m_a>>})
               ELSE /\ TRUE
                    /\ UNCHANGED prescriptions
         /\ pc' = "Error"
         /\ UNCHANGED << patients, medications, interactions, stack, p_, m_, 
                         m1_, m2_, p_a, m_a, m1, m2, p, m >>

add_prescription == Lbl_4

Lbl_5 == /\ pc = "Lbl_5"
         /\ IF /\ m1 \in medications
               /\ m2 \in medications
               /\ <<m1, m2>> \in interactions
               THEN /\ interactions' = interactions \ {<<m1, m2>>, <<m2, m1>>}
               ELSE /\ TRUE
                    /\ UNCHANGED interactions
         /\ pc' = "Error"
         /\ UNCHANGED << patients, medications, prescriptions, stack, p_, m_, 
                         m1_, m2_, p_a, m_a, m1, m2, p, m >>

remove_interaction == Lbl_5

Lbl_6 == /\ pc = "Lbl_6"
         /\ IF /\ p \in patients
               /\ m \in medications
               /\ <<p, m>> \in prescriptions
               THEN /\ prescriptions' = prescriptions \ {<<p, m>>}
               ELSE /\ TRUE
                    /\ UNCHANGED prescriptions
         /\ pc' = "Error"
         /\ UNCHANGED << patients, medications, interactions, stack, p_, m_, 
                         m1_, m2_, p_a, m_a, m1, m2, p, m >>

remove_prescription == Lbl_6

Lbl_7 == /\ pc = "Lbl_7"
         /\ \E _p \in Patients:
              /\ /\ p_' = _p
                 /\ stack' = << [ procedure |->  "add_patient",
                                  pc        |->  "Lbl_8",
                                  p_        |->  p_ ] >>
                              \o stack
              /\ pc' = "Lbl_1"
         /\ UNCHANGED << patients, medications, interactions, prescriptions, 
                         m_, m1_, m2_, p_a, m_a, m1, m2, p, m >>

Lbl_8 == /\ pc = "Lbl_8"
         /\ \E _m \in Medications:
              /\ /\ m_' = _m
                 /\ stack' = << [ procedure |->  "add_medication",
                                  pc        |->  "Lbl_9",
                                  m_        |->  m_ ] >>
                              \o stack
              /\ pc' = "Lbl_2"
         /\ UNCHANGED << patients, medications, interactions, prescriptions, 
                         p_, m1_, m2_, p_a, m_a, m1, m2, p, m >>

Lbl_9 == /\ pc = "Lbl_9"
         /\ \E _m1 \in medications:
              \E _m2 \in medications:
                \/ /\ /\ m1_' = _m1
                      /\ m2_' = _m2
                      /\ stack' = << [ procedure |->  "add_interaction",
                                       pc        |->  "Lbl_10",
                                       m1_       |->  m1_,
                                       m2_       |->  m2_ ] >>
                                   \o stack
                   /\ pc' = "Lbl_3"
                   /\ UNCHANGED <<m1, m2>>
                \/ /\ /\ m1' = _m1
                      /\ m2' = _m2
                      /\ stack' = << [ procedure |->  "remove_interaction",
                                       pc        |->  "Lbl_10",
                                       m1        |->  m1,
                                       m2        |->  m2 ] >>
                                   \o stack
                   /\ pc' = "Lbl_5"
                   /\ UNCHANGED <<m1_, m2_>>
         /\ UNCHANGED << patients, medications, interactions, prescriptions, 
                         p_, m_, p_a, m_a, p, m >>

Lbl_10 == /\ pc = "Lbl_10"
          /\ \E _p \in patients:
               \E _m \in medications:
                 \/ /\ /\ m_a' = _m
                       /\ p_a' = _p
                       /\ stack' = << [ procedure |->  "add_prescription",
                                        pc        |->  "Done",
                                        p_a       |->  p_a,
                                        m_a       |->  m_a ] >>
                                    \o stack
                    /\ pc' = "Lbl_4"
                    /\ UNCHANGED <<p, m>>
                 \/ /\ /\ m' = _m
                       /\ p' = _p
                       /\ stack' = << [ procedure |->  "remove_prescription",
                                        pc        |->  "Done",
                                        p         |->  p,
                                        m         |->  m ] >>
                                    \o stack
                    /\ pc' = "Lbl_6"
                    /\ UNCHANGED <<p_a, m_a>>
          /\ UNCHANGED << patients, medications, interactions, prescriptions, 
                          p_, m_, m1_, m2_, m1, m2 >>

(* Allow infinite stuttering to prevent deadlock on termination. *)
Terminating == pc = "Done" /\ UNCHANGED vars

Next == add_patient \/ add_medication \/ add_interaction
           \/ add_prescription \/ remove_interaction \/ remove_prescription
           \/ Lbl_7 \/ Lbl_8 \/ Lbl_9 \/ Lbl_10
           \/ Terminating

Spec == Init /\ [][Next]_vars

Termination == <>(pc = "Done")

\* END TRANSLATION

TypeOK ==
    /\ patients \subseteq Patients
    /\ medications \subseteq Medications
    /\ interactions \subseteq medications \X medications \* dangerous interactions
    /\ prescriptions \subseteq patients \X medications

\* Invariants

Symmetry == \A i_m1, i_m2 \in medications:
    <<i_m1, i_m2>> \in interactions <=>
    <<i_m2, i_m1>> \in interactions

\* A medication does not interact with itself
Irreflexive == \A i_m \in medications:
    <<i_m, i_m>> \notin interactions

\* All prescriptions are safe
\* i.e. do not have dangerous interactions
SafePrescriptions ==
    \A i_m1, i_m2 \in medications, i_p \in patients:
        <<i_m1, i_m2>> \in interactions =>
            ~ (<<i_p, i_m1>> \in prescriptions
            /\ <<i_p, i_m2>> \in prescriptions)


=============================================================================
\* Modification History
\* Last modified Fri Oct 04 01:58:04 EDT 2019 by bandali
\* Created Fri Oct 04 01:03:50 EDT 2019 by bandali
