------------------------ MODULE landing_gear_pluscal ------------------------
EXTENDS Integers, Sequences, TLC

GEAR_STATE == { "retracted", "extended", "gear_moving" }
DOOR_STATE == { "open", "closed", "door_moving" }
HANDLE_STATE == { "up", "down" }

(* --algorithm landing_gear_pluscal
variables gear = "extended",
          door = "closed",
          handle = "down";

define
TypeOK ==
    /\ gear \in GEAR_STATE
    /\ door \in DOOR_STATE \* invd in R1GearDoor
    /\ handle \in HANDLE_STATE \* inv in R2GearDoor

\* in R1GearDoor
safety_inv1 == gear = "gear_moving" => door = "open"
end define

process env_start_retracting = "env_start_retracting"
begin
    env_start_retracting:
        when
            /\ gear = "extended" \/ gear = "gear_moving"
            /\ door = "open"
            /\ handle = "up";
        gear := "gear_moving";
        goto env_start_retracting
end process

process env_retract_gear = "env_retract_gear"
begin
    env_retract_gear:
        when
            /\ gear = "gear_moving"
            /\ door = "open"
            /\ handle = "up";
        gear := "retracted";
        goto env_retract_gear
end process

process env_start_extending = "env_start_extending"
begin
    env_start_extending:
        when
            /\ gear = "retracted" \/ gear = "gear_moving"
            /\ door = "open"
            /\ handle = "down";
        gear := "gear_moving";
        goto env_start_extending
end process

process env_extend_gear = "env_extend_gear"
begin
    env_extend_gear:
        when
            /\ gear = "gear_moving"
            /\ door = "open"
            /\ handle = "down";
        gear := "extended";
        goto env_extend_gear
end process

process env_start_open_door = "env_start_open_door"
begin
    env_start_open_door:
        when
            /\ door = "closed" \/ door = "door_moving"
            /\ gear # "gear_moving"
            /\ \/ handle = "down" /\ gear = "retracted"
               \/ handle = "up" /\ gear = "extended";
        door := "door_moving";
        goto env_start_open_door;
end process

process env_open_door = "env_open_door"
begin
    env_open_door:
        when
            /\ door = "door_moving"
            /\ gear # "gear_moving"
            /\ \/ handle = "down" /\ gear = "retracted"
               \/ handle = "up" /\ gear = "extended";
        door := "open";
        goto env_open_door;
end process

process env_start_close_door = "env_start_close_door"
begin
    env_start_close_door:
        when
            /\ door = "open" \/ door = "door_moving"
            /\ gear # "gear_moving"
            /\ \/ handle = "down" /\ gear = "extended"
               \/ handle = "up" /\ (gear = "retracted" \/ gear = "extended");
        door := "door_moving";
        goto env_start_close_door;
end process

process env_close_door = "env_close_door"
begin
    env_close_door:
        when
            /\ door = "door_moving"
            /\ gear # "gear_moving"
            /\ \/ handle = "down" /\ gear = "extended"
               \/ handle = "up" /\ (gear = "retracted" \/ gear = "extended");
        door := "closed";
        goto env_close_door;
end process

process toggle_handle_up = "toggle_handle_up"
begin
    toggle_handle_up:
        when handle = "down";
        handle := "up";
        goto toggle_handle_up
end process

process toggle_handle_down = "toggle_handle_down"
begin
    toggle_handle_down:
        when handle = "up";
        handle := "down";
        goto toggle_handle_down
end process

end algorithm *)
\* BEGIN TRANSLATION
\* Label env_start_retracting of process env_start_retracting at line 26 col 9 changed to env_start_retracting_
\* Label env_retract_gear of process env_retract_gear at line 37 col 9 changed to env_retract_gear_
\* Label env_start_extending of process env_start_extending at line 48 col 9 changed to env_start_extending_
\* Label env_extend_gear of process env_extend_gear at line 59 col 9 changed to env_extend_gear_
\* Label env_start_open_door of process env_start_open_door at line 70 col 9 changed to env_start_open_door_
\* Label env_open_door of process env_open_door at line 82 col 9 changed to env_open_door_
\* Label env_start_close_door of process env_start_close_door at line 94 col 9 changed to env_start_close_door_
\* Label env_close_door of process env_close_door at line 106 col 9 changed to env_close_door_
\* Label toggle_handle_up of process toggle_handle_up at line 118 col 9 changed to toggle_handle_up_
\* Label toggle_handle_down of process toggle_handle_down at line 126 col 9 changed to toggle_handle_down_
VARIABLES gear, door, handle, pc

(* define statement *)
TypeOK ==
    /\ gear \in GEAR_STATE
    /\ door \in DOOR_STATE
    /\ handle \in HANDLE_STATE


safety_inv1 == gear = "gear_moving" => door = "open"


vars == << gear, door, handle, pc >>

ProcSet == {"env_start_retracting"} \cup {"env_retract_gear"} \cup {"env_start_extending"} \cup {"env_extend_gear"} \cup {"env_start_open_door"} \cup {"env_open_door"} \cup {"env_start_close_door"} \cup {"env_close_door"} \cup {"toggle_handle_up"} \cup {"toggle_handle_down"}

Init == (* Global variables *)
        /\ gear = "extended"
        /\ door = "closed"
        /\ handle = "down"
        /\ pc = [self \in ProcSet |-> CASE self = "env_start_retracting" -> "env_start_retracting_"
                                        [] self = "env_retract_gear" -> "env_retract_gear_"
                                        [] self = "env_start_extending" -> "env_start_extending_"
                                        [] self = "env_extend_gear" -> "env_extend_gear_"
                                        [] self = "env_start_open_door" -> "env_start_open_door_"
                                        [] self = "env_open_door" -> "env_open_door_"
                                        [] self = "env_start_close_door" -> "env_start_close_door_"
                                        [] self = "env_close_door" -> "env_close_door_"
                                        [] self = "toggle_handle_up" -> "toggle_handle_up_"
                                        [] self = "toggle_handle_down" -> "toggle_handle_down_"]

env_start_retracting_ == /\ pc["env_start_retracting"] = "env_start_retracting_"
                         /\ /\ gear = "extended" \/ gear = "gear_moving"
                            /\ door = "open"
                            /\ handle = "up"
                         /\ gear' = "gear_moving"
                         /\ pc' = [pc EXCEPT !["env_start_retracting"] = "env_start_retracting_"]
                         /\ UNCHANGED << door, handle >>

env_start_retracting == env_start_retracting_

env_retract_gear_ == /\ pc["env_retract_gear"] = "env_retract_gear_"
                     /\ /\ gear = "gear_moving"
                        /\ door = "open"
                        /\ handle = "up"
                     /\ gear' = "retracted"
                     /\ pc' = [pc EXCEPT !["env_retract_gear"] = "env_retract_gear_"]
                     /\ UNCHANGED << door, handle >>

env_retract_gear == env_retract_gear_

env_start_extending_ == /\ pc["env_start_extending"] = "env_start_extending_"
                        /\ /\ gear = "retracted" \/ gear = "gear_moving"
                           /\ door = "open"
                           /\ handle = "down"
                        /\ gear' = "gear_moving"
                        /\ pc' = [pc EXCEPT !["env_start_extending"] = "env_start_extending_"]
                        /\ UNCHANGED << door, handle >>

env_start_extending == env_start_extending_

env_extend_gear_ == /\ pc["env_extend_gear"] = "env_extend_gear_"
                    /\ /\ gear = "gear_moving"
                       /\ door = "open"
                       /\ handle = "down"
                    /\ gear' = "extended"
                    /\ pc' = [pc EXCEPT !["env_extend_gear"] = "env_extend_gear_"]
                    /\ UNCHANGED << door, handle >>

env_extend_gear == env_extend_gear_

env_start_open_door_ == /\ pc["env_start_open_door"] = "env_start_open_door_"
                        /\ /\ door = "closed" \/ door = "door_moving"
                           /\ gear # "gear_moving"
                           /\ \/ handle = "down" /\ gear = "retracted"
                              \/ handle = "up" /\ gear = "extended"
                        /\ door' = "door_moving"
                        /\ pc' = [pc EXCEPT !["env_start_open_door"] = "env_start_open_door_"]
                        /\ UNCHANGED << gear, handle >>

env_start_open_door == env_start_open_door_

env_open_door_ == /\ pc["env_open_door"] = "env_open_door_"
                  /\ /\ door = "door_moving"
                     /\ gear # "gear_moving"
                     /\ \/ handle = "down" /\ gear = "retracted"
                        \/ handle = "up" /\ gear = "extended"
                  /\ door' = "open"
                  /\ pc' = [pc EXCEPT !["env_open_door"] = "env_open_door_"]
                  /\ UNCHANGED << gear, handle >>

env_open_door == env_open_door_

env_start_close_door_ == /\ pc["env_start_close_door"] = "env_start_close_door_"
                         /\ /\ door = "open" \/ door = "door_moving"
                            /\ gear # "gear_moving"
                            /\ \/ handle = "down" /\ gear = "extended"
                               \/ handle = "up" /\ (gear = "retracted" \/ gear = "extended")
                         /\ door' = "door_moving"
                         /\ pc' = [pc EXCEPT !["env_start_close_door"] = "env_start_close_door_"]
                         /\ UNCHANGED << gear, handle >>

env_start_close_door == env_start_close_door_

env_close_door_ == /\ pc["env_close_door"] = "env_close_door_"
                   /\ /\ door = "door_moving"
                      /\ gear # "gear_moving"
                      /\ \/ handle = "down" /\ gear = "extended"
                         \/ handle = "up" /\ (gear = "retracted" \/ gear = "extended")
                   /\ door' = "closed"
                   /\ pc' = [pc EXCEPT !["env_close_door"] = "env_close_door_"]
                   /\ UNCHANGED << gear, handle >>

env_close_door == env_close_door_

toggle_handle_up_ == /\ pc["toggle_handle_up"] = "toggle_handle_up_"
                     /\ handle = "down"
                     /\ handle' = "up"
                     /\ pc' = [pc EXCEPT !["toggle_handle_up"] = "toggle_handle_up_"]
                     /\ UNCHANGED << gear, door >>

toggle_handle_up == toggle_handle_up_

toggle_handle_down_ == /\ pc["toggle_handle_down"] = "toggle_handle_down_"
                       /\ handle = "up"
                       /\ handle' = "down"
                       /\ pc' = [pc EXCEPT !["toggle_handle_down"] = "toggle_handle_down_"]
                       /\ UNCHANGED << gear, door >>

toggle_handle_down == toggle_handle_down_

(* Allow infinite stuttering to prevent deadlock on termination. *)
Terminating == /\ \A self \in ProcSet: pc[self] = "Done"
               /\ UNCHANGED vars

Next == env_start_retracting \/ env_retract_gear \/ env_start_extending
           \/ env_extend_gear \/ env_start_open_door \/ env_open_door
           \/ env_start_close_door \/ env_close_door \/ toggle_handle_up
           \/ toggle_handle_down
           \/ Terminating

Spec == Init /\ [][Next]_vars

Termination == <>(\A self \in ProcSet: pc[self] = "Done")

\* END TRANSLATION

=============================================================================
\* Modification History
\* Last modified Thu Apr 16 21:01:31 EDT 2020 by bandali
\* Created Thu Apr 16 18:20:00 EDT 2020 by bandali
