------------------------ MODULE landing_gear_pluscal ------------------------
EXTENDS Sequences, TLC, utils

GEAR_STATE == { "retracted", "extended", "gear_moving" }
DOOR_STATE == { "open", "closed", "door_moving" }
HANDLE_STATE == { "up", "down" }
POSITION == { "front", "left", "right" }

(* --algorithm landing_gear_pluscal
variables gears = [p \in POSITION |-> "extended"],
          doors = [p \in POSITION |-> "closed"],
          handle = "down";

define
TypeOK ==
    /\ gears \in [POSITION -> GEAR_STATE] \* inv1 in R3GearDoorHandle
    /\ doors \in [POSITION -> DOOR_STATE] \* inv2 in R3GearDoorHandle
    /\ handle \in HANDLE_STATE \* inv in R2GearDoor
end define

process env_start_retracting_first = "env_start_retracting_first"
variable gr \in POSITION
begin
    env_start_retracting_first:
        when
            /\ Range(doors) = {"open"}
            /\ gears[gr] = "extended"
            /\ handle = "up";
        gears[gr] := "gear_moving";
        goto env_start_retracting_first
end process

process env_retract_gear_skip = "env_retract_gear_skip"
variable gr \in POSITION
begin
    env_retract_gear_skip:
        when
            /\ Range(doors) = {"open"}
            /\ Range(DomSub({gr}, gears)) # {"retracted"}
            /\ handle = "up"
            /\ gears[gr] = "gear_moving";
        gears[gr] := "retracted";
        goto env_retract_gear_skip
end process

process env_retract_gear_last = "env_retract_gear_last"
variable gr \in POSITION
begin
    env_retract_gear_last:
        when
            /\ Range(doors) = {"open"}
            /\ Range(DomSub({gr}, gears)) = {"retracted"}
            /\ handle = "up"
            /\ gears[gr] = "gear_moving";
        gears[gr] := "retracted";
        goto env_retract_gear_last
end process

process env_start_extending = "env_start_extending"
variable gr \in POSITION
begin
    env_start_extending:
        when
            /\ Range(doors) = {"open"}
            /\ handle = "down"
            /\ gears[gr] = "retracted";
        gears[gr] := "gear_moving";
        goto env_start_extending
end process

process env_extend_gear_last = "env_extend_gear_last"
variable gr \in POSITION
begin
    env_extend_gear_last:
        when
            /\ Range(doors) = {"open"}
            /\ handle = "down"
            /\ Range(DomSub({gr}, gears)) = {"extended"}
            /\ gears[gr] = "gear_moving";
        gears[gr] := "extended";
        goto env_extend_gear_last
end process

process env_extend_gear_skip = "env_extend_gear_skip"
variable gr \in POSITION
begin
    env_extend_gear_skip:
        when
            /\ Range(doors) = {"open"}
            /\ handle = "down"
            /\ Range(DomSub({gr}, gears)) # {"extended"}
            /\ gears[gr] = "gear_moving";
        gears[gr] := "extended";
        goto env_extend_gear_skip
end process

process env_start_open_door = "env_start_open_door"
variable gr \in POSITION
begin
    env_start_open_door:
        when
            /\ doors[gr] = "closed"
            /\ gears[gr] # "gear_moving"
            /\ "gear_moving" \notin Range(gears)
            /\ \/ handle = "down" /\ Range(gears) = {"retracted"}
               \/ handle = "up" /\ Range(gears) = {"extended"};
        doors[gr] := "door_moving";
        goto env_start_open_door;
end process

process env_open_door_last = "env_open_door_last"
variable gr \in POSITION
begin
    env_open_door_last:
        when
            /\ doors[gr] = "door_moving"
            /\ gears[gr] # "gear_moving"
            /\ "gears_moving" \notin Range(gears)
            /\ Range(DomSub({gr}, doors)) = {"open"}
            /\ \/ handle = "down" /\ Range(gears) = {"retracted"}
               \/ handle = "up" /\ Range(gears) = {"extended"};
        doors[gr] := "open";
        goto env_open_door_last;
end process

process env_open_door_skip = "env_open_door_skip"
variable gr \in POSITION
begin
    env_open_door_skip:
        when
            /\ doors[gr] = "door_moving"
            /\ gears[gr] # "gear_moving"
            /\ "gears_moving" \notin Range(gears)
            /\ Range(DomSub({gr}, doors)) # {"open"}
            /\ \/ handle = "down" /\ Range(gears) = {"retracted"}
               \/ handle = "up" /\ Range(gears) = {"extended"};
        doors[gr] := "open";
        goto env_open_door_skip;
end process

process env_start_close_door = "env_start_close_door"
variable gr \in POSITION
begin
    env_start_close_door:
        when
            /\ doors[gr] = "open"
            /\ gears[gr] # "gear_moving"
            /\ \/ handle = "up" /\ (Range(gears) = {"retracted"} \/ Range(gears) = {"extended"})
               \/ handle = "down" /\ Range(gears) = {"extended"};
        doors[gr] := "door_moving";
        goto env_start_close_door;
end process

process env_close_door = "env_close_door"
variable gr \in POSITION
begin
    env_close_door:
        when
            /\ doors[gr] = "door_moving"
            /\ gears[gr] # "gear_moving"
            /\ "gear_moving" \notin Range(gears)
            /\ Range(DomSub({gr}, doors)) = {"closed"}
            /\ \/ handle = "up" /\ (Range(gears) = {"retracted"} \/ Range(gears) = {"extended"})
               \/ handle = "down" /\ Range(gears) = {"extended"};
        doors[gr] := "closed";
        goto env_close_door;
end process

process env_close_door_skip = "env_close_door_skip"
variable gr \in POSITION
begin
    env_close_door_skip:
        when
            /\ doors[gr] = "door_moving"
            /\ gears[gr] # "gear_moving"
            /\ "gear_moving" \notin Range(gears)
            /\ Range(DomSub({gr}, doors)) # {"closed"}
            /\ \/ handle = "up" /\ (Range(gears) = {"retracted"} \/ Range(gears) = {"extended"})
               \/ handle = "down" /\ Range(gears) = {"extended"};
        doors[gr] := "closed";
        goto env_close_door_skip;
end process

process toggle_handle_up = "toggle_handle_up"
begin
    toggle_handle_up:
        when handle = "down";
        handle := "up";
        goto toggle_handle_up
end process

process toggle_handle_down = "toggle_handle_down"
begin
    toggle_handle_down:
        when handle = "up";
        handle := "down";
        goto toggle_handle_down
end process

end algorithm *)
\* BEGIN TRANSLATION
\* Label env_start_retracting_first of process env_start_retracting_first at line 25 col 9 changed to env_start_retracting_first_
\* Label env_retract_gear_skip of process env_retract_gear_skip at line 37 col 9 changed to env_retract_gear_skip_
\* Label env_retract_gear_last of process env_retract_gear_last at line 50 col 9 changed to env_retract_gear_last_
\* Label env_start_extending of process env_start_extending at line 63 col 9 changed to env_start_extending_
\* Label env_extend_gear_last of process env_extend_gear_last at line 75 col 9 changed to env_extend_gear_last_
\* Label env_extend_gear_skip of process env_extend_gear_skip at line 88 col 9 changed to env_extend_gear_skip_
\* Label env_start_open_door of process env_start_open_door at line 101 col 9 changed to env_start_open_door_
\* Label env_open_door_last of process env_open_door_last at line 115 col 9 changed to env_open_door_last_
\* Label env_open_door_skip of process env_open_door_skip at line 130 col 9 changed to env_open_door_skip_
\* Label env_start_close_door of process env_start_close_door at line 145 col 9 changed to env_start_close_door_
\* Label env_close_door of process env_close_door at line 158 col 9 changed to env_close_door_
\* Label env_close_door_skip of process env_close_door_skip at line 173 col 9 changed to env_close_door_skip_
\* Label toggle_handle_up of process toggle_handle_up at line 187 col 9 changed to toggle_handle_up_
\* Label toggle_handle_down of process toggle_handle_down at line 195 col 9 changed to toggle_handle_down_
\* Process variable gr of process env_start_retracting_first at line 22 col 10 changed to gr_
\* Process variable gr of process env_retract_gear_skip at line 34 col 10 changed to gr_e
\* Process variable gr of process env_retract_gear_last at line 47 col 10 changed to gr_en
\* Process variable gr of process env_start_extending at line 60 col 10 changed to gr_env
\* Process variable gr of process env_extend_gear_last at line 72 col 10 changed to gr_env_
\* Process variable gr of process env_extend_gear_skip at line 85 col 10 changed to gr_env_e
\* Process variable gr of process env_start_open_door at line 98 col 10 changed to gr_env_s
\* Process variable gr of process env_open_door_last at line 112 col 10 changed to gr_env_o
\* Process variable gr of process env_open_door_skip at line 127 col 10 changed to gr_env_op
\* Process variable gr of process env_start_close_door at line 142 col 10 changed to gr_env_st
\* Process variable gr of process env_close_door at line 155 col 10 changed to gr_env_c
VARIABLES gears, doors, handle, pc

(* define statement *)
TypeOK ==
    /\ gears \in [POSITION -> GEAR_STATE]
    /\ doors \in [POSITION -> DOOR_STATE]
    /\ handle \in HANDLE_STATE

VARIABLES gr_, gr_e, gr_en, gr_env, gr_env_, gr_env_e, gr_env_s, gr_env_o, 
          gr_env_op, gr_env_st, gr_env_c, gr

vars == << gears, doors, handle, pc, gr_, gr_e, gr_en, gr_env, gr_env_, 
           gr_env_e, gr_env_s, gr_env_o, gr_env_op, gr_env_st, gr_env_c, gr
        >>

ProcSet == {"env_start_retracting_first"} \cup {"env_retract_gear_skip"} \cup {"env_retract_gear_last"} \cup {"env_start_extending"} \cup {"env_extend_gear_last"} \cup {"env_extend_gear_skip"} \cup {"env_start_open_door"} \cup {"env_open_door_last"} \cup {"env_open_door_skip"} \cup {"env_start_close_door"} \cup {"env_close_door"} \cup {"env_close_door_skip"} \cup {"toggle_handle_up"} \cup {"toggle_handle_down"}

Init == (* Global variables *)
        /\ gears = [p \in POSITION |-> "extended"]
        /\ doors = [p \in POSITION |-> "closed"]
        /\ handle = "down"
        (* Process env_start_retracting_first *)
        /\ gr_ \in POSITION
        (* Process env_retract_gear_skip *)
        /\ gr_e \in POSITION
        (* Process env_retract_gear_last *)
        /\ gr_en \in POSITION
        (* Process env_start_extending *)
        /\ gr_env \in POSITION
        (* Process env_extend_gear_last *)
        /\ gr_env_ \in POSITION
        (* Process env_extend_gear_skip *)
        /\ gr_env_e \in POSITION
        (* Process env_start_open_door *)
        /\ gr_env_s \in POSITION
        (* Process env_open_door_last *)
        /\ gr_env_o \in POSITION
        (* Process env_open_door_skip *)
        /\ gr_env_op \in POSITION
        (* Process env_start_close_door *)
        /\ gr_env_st \in POSITION
        (* Process env_close_door *)
        /\ gr_env_c \in POSITION
        (* Process env_close_door_skip *)
        /\ gr \in POSITION
        /\ pc = [self \in ProcSet |-> CASE self = "env_start_retracting_first" -> "env_start_retracting_first_"
                                        [] self = "env_retract_gear_skip" -> "env_retract_gear_skip_"
                                        [] self = "env_retract_gear_last" -> "env_retract_gear_last_"
                                        [] self = "env_start_extending" -> "env_start_extending_"
                                        [] self = "env_extend_gear_last" -> "env_extend_gear_last_"
                                        [] self = "env_extend_gear_skip" -> "env_extend_gear_skip_"
                                        [] self = "env_start_open_door" -> "env_start_open_door_"
                                        [] self = "env_open_door_last" -> "env_open_door_last_"
                                        [] self = "env_open_door_skip" -> "env_open_door_skip_"
                                        [] self = "env_start_close_door" -> "env_start_close_door_"
                                        [] self = "env_close_door" -> "env_close_door_"
                                        [] self = "env_close_door_skip" -> "env_close_door_skip_"
                                        [] self = "toggle_handle_up" -> "toggle_handle_up_"
                                        [] self = "toggle_handle_down" -> "toggle_handle_down_"]

env_start_retracting_first_ == /\ pc["env_start_retracting_first"] = "env_start_retracting_first_"
                               /\ /\ Range(doors) = {"open"}
                                  /\ gears[gr_] = "extended"
                                  /\ handle = "up"
                               /\ gears' = [gears EXCEPT ![gr_] = "gear_moving"]
                               /\ pc' = [pc EXCEPT !["env_start_retracting_first"] = "env_start_retracting_first_"]
                               /\ UNCHANGED << doors, handle, gr_, gr_e, gr_en, 
                                               gr_env, gr_env_, gr_env_e, 
                                               gr_env_s, gr_env_o, gr_env_op, 
                                               gr_env_st, gr_env_c, gr >>

env_start_retracting_first == env_start_retracting_first_

env_retract_gear_skip_ == /\ pc["env_retract_gear_skip"] = "env_retract_gear_skip_"
                          /\ /\ Range(doors) = {"open"}
                             /\ Range(DomSub({gr_e}, gears)) # {"retracted"}
                             /\ handle = "up"
                             /\ gears[gr_e] = "gear_moving"
                          /\ gears' = [gears EXCEPT ![gr_e] = "retracted"]
                          /\ pc' = [pc EXCEPT !["env_retract_gear_skip"] = "env_retract_gear_skip_"]
                          /\ UNCHANGED << doors, handle, gr_, gr_e, gr_en, 
                                          gr_env, gr_env_, gr_env_e, gr_env_s, 
                                          gr_env_o, gr_env_op, gr_env_st, 
                                          gr_env_c, gr >>

env_retract_gear_skip == env_retract_gear_skip_

env_retract_gear_last_ == /\ pc["env_retract_gear_last"] = "env_retract_gear_last_"
                          /\ /\ Range(doors) = {"open"}
                             /\ Range(DomSub({gr_en}, gears)) = {"retracted"}
                             /\ handle = "up"
                             /\ gears[gr_en] = "gear_moving"
                          /\ gears' = [gears EXCEPT ![gr_en] = "retracted"]
                          /\ pc' = [pc EXCEPT !["env_retract_gear_last"] = "env_retract_gear_last_"]
                          /\ UNCHANGED << doors, handle, gr_, gr_e, gr_en, 
                                          gr_env, gr_env_, gr_env_e, gr_env_s, 
                                          gr_env_o, gr_env_op, gr_env_st, 
                                          gr_env_c, gr >>

env_retract_gear_last == env_retract_gear_last_

env_start_extending_ == /\ pc["env_start_extending"] = "env_start_extending_"
                        /\ /\ Range(doors) = {"open"}
                           /\ handle = "down"
                           /\ gears[gr_env] = "retracted"
                        /\ gears' = [gears EXCEPT ![gr_env] = "gear_moving"]
                        /\ pc' = [pc EXCEPT !["env_start_extending"] = "env_start_extending_"]
                        /\ UNCHANGED << doors, handle, gr_, gr_e, gr_en, 
                                        gr_env, gr_env_, gr_env_e, gr_env_s, 
                                        gr_env_o, gr_env_op, gr_env_st, 
                                        gr_env_c, gr >>

env_start_extending == env_start_extending_

env_extend_gear_last_ == /\ pc["env_extend_gear_last"] = "env_extend_gear_last_"
                         /\ /\ Range(doors) = {"open"}
                            /\ handle = "down"
                            /\ Range(DomSub({gr_env_}, gears)) = {"extended"}
                            /\ gears[gr_env_] = "gear_moving"
                         /\ gears' = [gears EXCEPT ![gr_env_] = "extended"]
                         /\ pc' = [pc EXCEPT !["env_extend_gear_last"] = "env_extend_gear_last_"]
                         /\ UNCHANGED << doors, handle, gr_, gr_e, gr_en, 
                                         gr_env, gr_env_, gr_env_e, gr_env_s, 
                                         gr_env_o, gr_env_op, gr_env_st, 
                                         gr_env_c, gr >>

env_extend_gear_last == env_extend_gear_last_

env_extend_gear_skip_ == /\ pc["env_extend_gear_skip"] = "env_extend_gear_skip_"
                         /\ /\ Range(doors) = {"open"}
                            /\ handle = "down"
                            /\ Range(DomSub({gr_env_e}, gears)) # {"extended"}
                            /\ gears[gr_env_e] = "gear_moving"
                         /\ gears' = [gears EXCEPT ![gr_env_e] = "extended"]
                         /\ pc' = [pc EXCEPT !["env_extend_gear_skip"] = "env_extend_gear_skip_"]
                         /\ UNCHANGED << doors, handle, gr_, gr_e, gr_en, 
                                         gr_env, gr_env_, gr_env_e, gr_env_s, 
                                         gr_env_o, gr_env_op, gr_env_st, 
                                         gr_env_c, gr >>

env_extend_gear_skip == env_extend_gear_skip_

env_start_open_door_ == /\ pc["env_start_open_door"] = "env_start_open_door_"
                        /\ /\ doors[gr_env_s] = "closed"
                           /\ gears[gr_env_s] # "gear_moving"
                           /\ "gear_moving" \notin Range(gears)
                           /\ \/ handle = "down" /\ Range(gears) = {"retracted"}
                              \/ handle = "up" /\ Range(gears) = {"extended"}
                        /\ doors' = [doors EXCEPT ![gr_env_s] = "door_moving"]
                        /\ pc' = [pc EXCEPT !["env_start_open_door"] = "env_start_open_door_"]
                        /\ UNCHANGED << gears, handle, gr_, gr_e, gr_en, 
                                        gr_env, gr_env_, gr_env_e, gr_env_s, 
                                        gr_env_o, gr_env_op, gr_env_st, 
                                        gr_env_c, gr >>

env_start_open_door == env_start_open_door_

env_open_door_last_ == /\ pc["env_open_door_last"] = "env_open_door_last_"
                       /\ /\ doors[gr_env_o] = "door_moving"
                          /\ gears[gr_env_o] # "gear_moving"
                          /\ "gears_moving" \notin Range(gears)
                          /\ Range(DomSub({gr_env_o}, doors)) = {"open"}
                          /\ \/ handle = "down" /\ Range(gears) = {"retracted"}
                             \/ handle = "up" /\ Range(gears) = {"extended"}
                       /\ doors' = [doors EXCEPT ![gr_env_o] = "open"]
                       /\ pc' = [pc EXCEPT !["env_open_door_last"] = "env_open_door_last_"]
                       /\ UNCHANGED << gears, handle, gr_, gr_e, gr_en, gr_env, 
                                       gr_env_, gr_env_e, gr_env_s, gr_env_o, 
                                       gr_env_op, gr_env_st, gr_env_c, gr >>

env_open_door_last == env_open_door_last_

env_open_door_skip_ == /\ pc["env_open_door_skip"] = "env_open_door_skip_"
                       /\ /\ doors[gr_env_op] = "door_moving"
                          /\ gears[gr_env_op] # "gear_moving"
                          /\ "gears_moving" \notin Range(gears)
                          /\ Range(DomSub({gr_env_op}, doors)) # {"open"}
                          /\ \/ handle = "down" /\ Range(gears) = {"retracted"}
                             \/ handle = "up" /\ Range(gears) = {"extended"}
                       /\ doors' = [doors EXCEPT ![gr_env_op] = "open"]
                       /\ pc' = [pc EXCEPT !["env_open_door_skip"] = "env_open_door_skip_"]
                       /\ UNCHANGED << gears, handle, gr_, gr_e, gr_en, gr_env, 
                                       gr_env_, gr_env_e, gr_env_s, gr_env_o, 
                                       gr_env_op, gr_env_st, gr_env_c, gr >>

env_open_door_skip == env_open_door_skip_

env_start_close_door_ == /\ pc["env_start_close_door"] = "env_start_close_door_"
                         /\ /\ doors[gr_env_st] = "open"
                            /\ gears[gr_env_st] # "gear_moving"
                            /\ \/ handle = "up" /\ (Range(gears) = {"retracted"} \/ Range(gears) = {"extended"})
                               \/ handle = "down" /\ Range(gears) = {"extended"}
                         /\ doors' = [doors EXCEPT ![gr_env_st] = "door_moving"]
                         /\ pc' = [pc EXCEPT !["env_start_close_door"] = "env_start_close_door_"]
                         /\ UNCHANGED << gears, handle, gr_, gr_e, gr_en, 
                                         gr_env, gr_env_, gr_env_e, gr_env_s, 
                                         gr_env_o, gr_env_op, gr_env_st, 
                                         gr_env_c, gr >>

env_start_close_door == env_start_close_door_

env_close_door_ == /\ pc["env_close_door"] = "env_close_door_"
                   /\ /\ doors[gr_env_c] = "door_moving"
                      /\ gears[gr_env_c] # "gear_moving"
                      /\ "gear_moving" \notin Range(gears)
                      /\ Range(DomSub({gr_env_c}, doors)) = {"closed"}
                      /\ \/ handle = "up" /\ (Range(gears) = {"retracted"} \/ Range(gears) = {"extended"})
                         \/ handle = "down" /\ Range(gears) = {"extended"}
                   /\ doors' = [doors EXCEPT ![gr_env_c] = "closed"]
                   /\ pc' = [pc EXCEPT !["env_close_door"] = "env_close_door_"]
                   /\ UNCHANGED << gears, handle, gr_, gr_e, gr_en, gr_env, 
                                   gr_env_, gr_env_e, gr_env_s, gr_env_o, 
                                   gr_env_op, gr_env_st, gr_env_c, gr >>

env_close_door == env_close_door_

env_close_door_skip_ == /\ pc["env_close_door_skip"] = "env_close_door_skip_"
                        /\ /\ doors[gr] = "door_moving"
                           /\ gears[gr] # "gear_moving"
                           /\ "gear_moving" \notin Range(gears)
                           /\ Range(DomSub({gr}, doors)) # {"closed"}
                           /\ \/ handle = "up" /\ (Range(gears) = {"retracted"} \/ Range(gears) = {"extended"})
                              \/ handle = "down" /\ Range(gears) = {"extended"}
                        /\ doors' = [doors EXCEPT ![gr] = "closed"]
                        /\ pc' = [pc EXCEPT !["env_close_door_skip"] = "env_close_door_skip_"]
                        /\ UNCHANGED << gears, handle, gr_, gr_e, gr_en, 
                                        gr_env, gr_env_, gr_env_e, gr_env_s, 
                                        gr_env_o, gr_env_op, gr_env_st, 
                                        gr_env_c, gr >>

env_close_door_skip == env_close_door_skip_

toggle_handle_up_ == /\ pc["toggle_handle_up"] = "toggle_handle_up_"
                     /\ handle = "down"
                     /\ handle' = "up"
                     /\ pc' = [pc EXCEPT !["toggle_handle_up"] = "toggle_handle_up_"]
                     /\ UNCHANGED << gears, doors, gr_, gr_e, gr_en, gr_env, 
                                     gr_env_, gr_env_e, gr_env_s, gr_env_o, 
                                     gr_env_op, gr_env_st, gr_env_c, gr >>

toggle_handle_up == toggle_handle_up_

toggle_handle_down_ == /\ pc["toggle_handle_down"] = "toggle_handle_down_"
                       /\ handle = "up"
                       /\ handle' = "down"
                       /\ pc' = [pc EXCEPT !["toggle_handle_down"] = "toggle_handle_down_"]
                       /\ UNCHANGED << gears, doors, gr_, gr_e, gr_en, gr_env, 
                                       gr_env_, gr_env_e, gr_env_s, gr_env_o, 
                                       gr_env_op, gr_env_st, gr_env_c, gr >>

toggle_handle_down == toggle_handle_down_

(* Allow infinite stuttering to prevent deadlock on termination. *)
Terminating == /\ \A self \in ProcSet: pc[self] = "Done"
               /\ UNCHANGED vars

Next == env_start_retracting_first \/ env_retract_gear_skip
           \/ env_retract_gear_last \/ env_start_extending
           \/ env_extend_gear_last \/ env_extend_gear_skip \/ env_start_open_door
           \/ env_open_door_last \/ env_open_door_skip \/ env_start_close_door
           \/ env_close_door \/ env_close_door_skip \/ toggle_handle_up
           \/ toggle_handle_down
           \/ Terminating

Spec == Init /\ [][Next]_vars

Termination == <>(\A self \in ProcSet: pc[self] = "Done")

\* END TRANSLATION

=============================================================================
\* Modification History
\* Last modified Thu Apr 16 23:25:19 EDT 2020 by bandali
\* Created Thu Apr 16 18:20:00 EDT 2020 by bandali
