open util/sequniv 

open util/steps[Snapshot]

// Snapshot definition
    sig Snapshot extends BaseSnapshot {
        Library_loans : (s.Library_books) one -> one ((s.Library_members)),
        Library_in_m : lone MemberID,
        Library_reservations : (s.Library_books) one -> one (seq (s.Library_members)),
        Library_in_b : lone BookID,
        Library_members : set MemberID,
        Library_books : set BookID
    }

/***************************** STATE SPACE ************************************/
    abstract sig SystemState extends StateLabel {}
    one sig Library extends SystemState {}

/*************************** TRANSITIONS SPACE ********************************/
    one sig Library_Acquire extends TransitionLabel {}
    one sig Library_Cancel extends TransitionLabel {}
    one sig Library_Discard extends TransitionLabel {}
    one sig Library_Join extends TransitionLabel {}
    one sig Library_Leave extends TransitionLabel {}
    one sig Library_Lend extends TransitionLabel {}
    one sig Library_Renew extends TransitionLabel {}
    one sig Library_Reserve extends TransitionLabel {}
    one sig Library_Return extends TransitionLabel {}
    one sig Library_Take extends TransitionLabel {}

    // Transition Library_Acquire
    pred pre_Library_Acquire[s:Snapshot] {
        Library in s.conf
        !(((s.Library_in_b) in (s.Library_books)))
    }

    pred pos_Library_Acquire[s, s':Snapshot] {
        s'.conf = s.conf - Library + {
            Library
        }
        s'.Library_loans = s.Library_loans
        s'.Library_reservations = s.Library_reservations
        s'.Library_members = s.Library_members
        (s'.Library_books) = (s.Library_books) + (s.Library_in_b)
    }

    pred Library_Acquire[s, s': Snapshot] {
        pre_Library_Acquire[s]
        pos_Library_Acquire[s, s']
        semantics_Library_Acquire[s, s']
    }
    pred semantics_Library_Acquire[s, s': Snapshot] {
        s'.taken = Library_Acquire
    }
    // Transition Library_Cancel
    pred pre_Library_Cancel[s:Snapshot] {
        Library in s.conf
        {
            (s.Library_in_m) in (s.Library_members)
            (s.Library_in_b) in (s.Library_books)
            one (Int -> (s.Library_in_m) & (s.Library_reservations)[ (s.Library_in_b)])
        }
    }

    pred pos_Library_Cancel[s, s':Snapshot] {
        s'.conf = s.conf - Library + {
            Library
        }
        s'.Library_loans = s.Library_loans
        s'.Library_members = s.Library_members
        s'.Library_books = s.Library_books
        (s'.Library_reservations)[ (s.Library_in_b)] = delete[ ((s.Library_reservations)[ (s.Library_in_b)]), (idxOf[ ((s.Library_reservations)[ (s.Library_in_b)]), (s.Library_in_m)])]
    }

    pred Library_Cancel[s, s': Snapshot] {
        pre_Library_Cancel[s]
        pos_Library_Cancel[s, s']
        semantics_Library_Cancel[s, s']
    }
    pred semantics_Library_Cancel[s, s': Snapshot] {
        s'.taken = Library_Cancel
    }
    // Transition Library_Discard
    pred pre_Library_Discard[s:Snapshot] {
        Library in s.conf
        {
            (s.Library_in_b) in (s.Library_books)
            no (s.Library_loans)[ (s.Library_in_b)]
            no (s.Library_reservations)[ (s.Library_in_b)]
        }
    }

    pred pos_Library_Discard[s, s':Snapshot] {
        s'.conf = s.conf - Library + {
            Library
        }
        s'.Library_loans = s.Library_loans
        s'.Library_reservations = s.Library_reservations
        s'.Library_members = s.Library_members
        (s'.Library_books) = (s.Library_books) - (s.Library_in_b)
    }

    pred Library_Discard[s, s': Snapshot] {
        pre_Library_Discard[s]
        pos_Library_Discard[s, s']
        semantics_Library_Discard[s, s']
    }
    pred semantics_Library_Discard[s, s': Snapshot] {
        s'.taken = Library_Discard
    }
    // Transition Library_Join
    pred pre_Library_Join[s:Snapshot] {
        Library in s.conf
        !(((s.Library_in_m) in (s.Library_members)))
    }

    pred pos_Library_Join[s, s':Snapshot] {
        s'.conf = s.conf - Library + {
            Library
        }
        s'.Library_loans = s.Library_loans
        s'.Library_reservations = s.Library_reservations
        s'.Library_books = s.Library_books
        (s'.Library_members) = (s.Library_members) + (s.Library_in_m)
    }

    pred Library_Join[s, s': Snapshot] {
        pre_Library_Join[s]
        pos_Library_Join[s, s']
        semantics_Library_Join[s, s']
    }
    pred semantics_Library_Join[s, s': Snapshot] {
        s'.taken = Library_Join
    }
    // Transition Library_Leave
    pred pre_Library_Leave[s:Snapshot] {
        Library in s.conf
        {
            (s.Library_in_m) in (s.Library_members)
            no (s.Library_loans).(s.Library_in_m)
            no (Int -> (s.Library_in_m) & (s.Library_reservations)[ (s.Library_in_b)])
        }
    }

    pred pos_Library_Leave[s, s':Snapshot] {
        s'.conf = s.conf - Library + {
            Library
        }
        s'.Library_loans = s.Library_loans
        s'.Library_reservations = s.Library_reservations
        s'.Library_books = s.Library_books
        (s'.Library_members) = (s.Library_members) - (s.Library_in_m)
    }

    pred Library_Leave[s, s': Snapshot] {
        pre_Library_Leave[s]
        pos_Library_Leave[s, s']
        semantics_Library_Leave[s, s']
    }
    pred semantics_Library_Leave[s, s': Snapshot] {
        s'.taken = Library_Leave
    }
    // Transition Library_Lend
    pred pre_Library_Lend[s:Snapshot] {
        Library in s.conf
        {
            (s.Library_in_m) in (s.Library_members)
            (s.Library_in_b) in (s.Library_books)
            all m : (s.Library_members)
             | no (s.Library_loans).m & (s.Library_in_b)
            no (s.Library_reservations)[ (s.Library_in_b)]
            #((s.Library_loans).(s.Library_in_m)) < Const.maxNbLoans
        }
    }

    pred pos_Library_Lend[s, s':Snapshot] {
        s'.conf = s.conf - Library + {
            Library
        }
        s'.Library_reservations = s.Library_reservations
        s'.Library_members = s.Library_members
        s'.Library_books = s.Library_books
        (s'.Library_loans) = (s.Library_loans) + (s.Library_in_b) -> (s.Library_in_m)
    }

    pred Library_Lend[s, s': Snapshot] {
        pre_Library_Lend[s]
        pos_Library_Lend[s, s']
        semantics_Library_Lend[s, s']
    }
    pred semantics_Library_Lend[s, s': Snapshot] {
        s'.taken = Library_Lend
    }
    // Transition Library_Renew
    pred pre_Library_Renew[s:Snapshot] {
        Library in s.conf
        {
            (s.Library_in_m) in (s.Library_members)
            (s.Library_in_b) in (s.Library_books)
            (s.Library_in_b) -> (s.Library_in_m) in (s.Library_loans)
            no (s.Library_reservations)[ (s.Library_in_b)]
        }
    }

    pred pos_Library_Renew[s, s':Snapshot] {
        s'.conf = s.conf - Library + {
            Library
        }
        s'.Library_loans = s.Library_loans
        s'.Library_reservations = s.Library_reservations
        s'.Library_members = s.Library_members
        s'.Library_books = s.Library_books
    }

    pred Library_Renew[s, s': Snapshot] {
        pre_Library_Renew[s]
        pos_Library_Renew[s, s']
        semantics_Library_Renew[s, s']
    }
    pred semantics_Library_Renew[s, s': Snapshot] {
        s'.taken = Library_Renew
    }
    // Transition Library_Reserve
    pred pre_Library_Reserve[s:Snapshot] {
        Library in s.conf
        {
            (s.Library_in_m) in (s.Library_members)
            (s.Library_in_b) in (s.Library_books)
            !(((s.Library_in_b) -> (s.Library_in_m) in (s.Library_loans)))
            no Int -> (s.Library_in_m) & (s.Library_reservations)[ (s.Library_in_b)]
            (some (s.Library_loans)[ (s.Library_in_b)] or (!(isEmpty[ ((s.Library_reservations)[ (s.Library_in_b)])])))
        }
    }

    pred pos_Library_Reserve[s, s':Snapshot] {
        s'.conf = s.conf - Library + {
            Library
        }
        s'.Library_loans = s.Library_loans
        s'.Library_members = s.Library_members
        s'.Library_books = s.Library_books
        (s'.Library_reservations)[ (s.Library_in_b)] = add[ ((s.Library_reservations)[ (s.Library_in_b)]), (s.Library_in_m)]
    }

    pred Library_Reserve[s, s': Snapshot] {
        pre_Library_Reserve[s]
        pos_Library_Reserve[s, s']
        semantics_Library_Reserve[s, s']
    }
    pred semantics_Library_Reserve[s, s': Snapshot] {
        s'.taken = Library_Reserve
    }
    // Transition Library_Return
    pred pre_Library_Return[s:Snapshot] {
        Library in s.conf
        {
            (s.Library_in_b) in (s.Library_books)
            some (s.Library_loans)[ (s.Library_in_b)]
        }
    }

    pred pos_Library_Return[s, s':Snapshot] {
        s'.conf = s.conf - Library + {
            Library
        }
        s'.Library_reservations = s.Library_reservations
        s'.Library_members = s.Library_members
        s'.Library_books = s.Library_books
        (s'.Library_loans) = (s.Library_loans) - (s.Library_in_b) -> ((s.Library_loans)[ (s.Library_in_b)])
    }

    pred Library_Return[s, s': Snapshot] {
        pre_Library_Return[s]
        pos_Library_Return[s, s']
        semantics_Library_Return[s, s']
    }
    pred semantics_Library_Return[s, s': Snapshot] {
        s'.taken = Library_Return
    }
    // Transition Library_Take
    pred pre_Library_Take[s:Snapshot] {
        Library in s.conf
        {
            (s.Library_in_m) in (s.Library_members)
            (s.Library_in_b) in (s.Library_books)
            no (s.Library_loans)[ (s.Library_in_b)]
            #((s.Library_loans).(s.Library_in_m)) < Const.maxNbLoans
            #((s.Library_reservations)[ (s.Library_in_b)]) > 0
            first[ ((s.Library_reservations)[ (s.Library_in_b)])] = (s.Library_in_m)
        }
    }

    pred pos_Library_Take[s, s':Snapshot] {
        s'.conf = s.conf - Library + {
            Library
        }
        s'.Library_members = s.Library_members
        s'.Library_books = s.Library_books
        {
            (s'.Library_loans) = (s.Library_loans) + (s.Library_in_b) -> (s.Library_in_m)
            (s'.Library_reservations)[ (s.Library_in_b)] = delete[ ((s.Library_reservations)[ (s.Library_in_b)]), (idxOf[ ((s.Library_reservations)[ (s.Library_in_b)]), (s.Library_in_m)])]
        }
    }

    pred Library_Take[s, s': Snapshot] {
        pre_Library_Take[s]
        pos_Library_Take[s, s']
        semantics_Library_Take[s, s']
    }
    pred semantics_Library_Take[s, s': Snapshot] {
        s'.taken = Library_Take
    }
/****************************** INITIAL CONDITIONS ****************************/
    pred init[s: Snapshot] {
        s.conf = {
            Library
        }
        no s.taken
        // Model specific constraints
        no (s.Library_members)
            no (s.Library_books)
            no (s.Library_loans)
            no (s.Library_reservations)
    }

/**************************** SIGNIFICANCE AXIOMS *****************************/
    /** Uncomment the operationsAxiom to find a significant scope. */
    pred significance {
        steps/reachabilityAxiom
        // operationsAxiom
    }
    run significance for 5 Snapshot, 0 EventLabel,
        3 MemberID, 3 BookID
        expect 1

    pred operationsAxiom {
        some s, s': Snapshot | Library_Acquire[s, s']
        some s, s': Snapshot | Library_Cancel[s, s']
        some s, s': Snapshot | Library_Discard[s, s']
        some s, s': Snapshot | Library_Join[s, s']
        some s, s': Snapshot | Library_Leave[s, s']
        some s, s': Snapshot | Library_Lend[s, s']
        some s, s': Snapshot | Library_Renew[s, s']
        some s, s': Snapshot | Library_Reserve[s, s']
        some s, s': Snapshot | Library_Return[s, s']
        some s, s': Snapshot | Library_Take[s, s']
    }

/***************************** MODEL DEFINITION *******************************/
    pred operation[s, s': Snapshot] {
        Library_Acquire[s, s'] or
        Library_Cancel[s, s'] or
        Library_Discard[s, s'] or
        Library_Join[s, s'] or
        Library_Leave[s, s'] or
        Library_Lend[s, s'] or
        Library_Renew[s, s'] or
        Library_Reserve[s, s'] or
        Library_Return[s, s'] or
        Library_Take[s, s']
    }

    pred small_step[s, s': Snapshot] {
        operation[s, s']
    }

    pred equals[s, s': Snapshot] {
        s'.conf = s.conf
        s'.taken = s.taken
        // Model specific declarations
        s'.Library_loans = s.Library_loans
        s'.Library_in_m = s.Library_in_m
        s'.Library_reservations = s.Library_reservations
        s'.Library_in_b = s.Library_in_b
        s'.Library_members = s.Library_members
        s'.Library_books = s.Library_books
    }

    fact {
        all s: Snapshot | s in initial iff init[s]
        all s, s': Snapshot | s->s' in nextStep iff small_step[s, s']
        all s, s': Snapshot | equals[s, s'] => s = s'
        significance
    }

/****************************** INVARIANTS ************************************/
    fact Library_maxNbLoans {
        all s: Snapshot | Library in s.conf => {
            Const.maxNbLoans = 7
        }
    }
    


    one sig Const  {
        maxNbLoans : Int
     }
    
    sig MemberID, BookID {}
    
    
    // run operation for 8 Snapshot, 0 EventLabel, 3 Patient, 4 Medication
    
    check assert_ltl_2 {
        all s: Snapshot | pre_Library_Acquire[s] => not(s.Library_in_b in s.Library_books)
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    check assert_ltl_3 {
        all s: Snapshot |
            (s.Library_in_b in s.Library_books and pre_Library_Discard[s]) =>
                (no (s.Library_in_b).(s.Library_loans) and
                 no (s.Library_in_b).(s.Library_reservations))
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    check assert_ltl_4 {
        all s: Snapshot | pre_Library_Lend[s] => s.Library_in_m in s.Library_members
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    check assert_ltl_5 {
        all s: Snapshot | pre_Library_Reserve[s] =>
            ((no (s.Library_reservations).(s.Library_in_m)) and
             not (s.Library_in_m in s.Library_loans[s.Library_in_b]) and
             ((some ((s.Library_in_b).(s.Library_loans))) or
              (no (s.Library_in_b).(s.Library_reservations))))
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    check assert_ltl_6 {
        all s: Snapshot | pre_Library_Reserve[s] =>
            not (s.Library_in_m in s.Library_loans[s.Library_in_b])
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    check assert_ltl_7 {
        all s: Snapshot | pre_Library_Reserve[s] =>
            no ((s.Library_reservations).(s.Library_in_m))
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    check assert_ltl_8 {
        all s: Snapshot | (some ((s.Library_in_b).(s.Library_reservations)) =>
            not(pre_Library_Lend[s]))
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    check assert_ltl_9 {
        all s: Snapshot | (some ((s.Library_in_b).(s.Library_reservations)) =>
            not(pre_Library_Renew[s]))
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    check assert_ltl_10 {
        all s: Snapshot | pre_Library_Take[s] =>
            ((some (s.Library_reservations[s.Library_in_b])) and
             first[(s.Library_reservations[s.Library_in_b])] = s.Library_in_m)
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    check assert_ltl_11 {
        all s: Snapshot {
            all b: s.Library_books {
                (some s.Library_loans[b]) =>
                    not(pre_Library_Take[s] and b = s.Library_in_b)
            }
            (pre_Library_Take[s] =>
                (no s.Library_loans[s.Library_in_b]))
        }
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    check assert_ltl_13 {
        all s: Snapshot | pre_Library_Leave[s] =>
            ((s.Library_in_m in s.Library_members) and
             (no ((s.Library_loans).(s.Library_in_m))) and
             (no ((s.Library_reservations).(s.Library_in_m))))
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    assert assert_ltl_14  {
        ctl_mc[ag[{  (ef[{ s: Snapshot | ((s.Library_in_m) !in (s.Library_members))}])}]]
    }
    
    check assert_ltl_14 
        for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    
    
    check assert_ltl_15 {
        all s: Snapshot, m: s.Library_members | #(s.Library_loans.m) <= Const.maxNbLoans
    } for 8 Snapshot, 0 EventLabel, 3 MemberID, 3 BookID expect 0
    

