asm musical_chairs

import StandardLibrary

signature:
	// domains
	enum domain RuleId = {WALK | SIT | ELIMINATELOSER | WIN}
	enum domain Player = {PP1 | PP2 | PP3}
	enum domain Chair = {CC1 | CC2}
	enum domain State = {START | WALKING | SITTING | WON}

	// functions
	monitored musicPlaying: Boolean

	controlled activePlayers: Powerset(Player)
	controlled activeChairs: Powerset(Chair)
	controlled occupiedChairs: Chair -> Player
	controlled numOccupied: Natural
	controlled state: State
	controlled chosenLoser: Player

definitions:
	// helper rules
	rule r_SitAllBut($l in Player) =
		forall $c in activeChairs with true do
			choose $p in activePlayers with $p != $l do
				if (isUndef(occupiedChairs($c))) then
					seq
						occupiedChairs($c) := $p
						numOccupied := numOccupied + 1n
					endseq
				endif

	rule r_StandAll =
		forall $c in activeChairs with true do
			occupiedChairs($c) := undef

	// rules
	rule r_Walk =
		if (state = START and musicPlaying and size(activePlayers) > 1) then
			state := WALKING
		endif

	rule r_Sit =
		choose $loser in activePlayers with true do
			if (state = WALKING and not musicPlaying) then
				seq
					chosenLoser := $loser
					numOccupied := 0n
					r_SitAllBut[$loser]
					state := SITTING
				endseq
			endif

	rule r_EliminateLoser =
		choose $c in activeChairs with true do
			if (state = SITTING and size(activePlayers) > 1
					and (size(activePlayers) - ntoi(numOccupied) = 1)) then
				seq
					r_StandAll[]
					activePlayers := excluding(activePlayers, chosenLoser)
					activeChairs := excluding(activeChairs, $c)
					chosenLoser := undef
					numOccupied := 0n
					if (size(activePlayers) > 1) then
						state := START
					else
						state := SITTING
					endif
				endseq
			endif

	rule r_Win =
		if (state = SITTING and size(activePlayers) = 1) then
			state := WON
		endif

	main rule r_Main =
		choose $rule in RuleId with true do
				switch($rule)
					case WALK:
						r_Walk[]
					case SIT:
						r_Sit[]
					case ELIMINATELOSER:
						r_EliminateLoser[]
					case WIN:
						r_Win[]
				endswitch

default init s0:
	function activePlayers = Player
	function activeChairs = Chair
	function occupiedChairs($c in Chair) = undef
	function numOccupied = 0n
	function state = START
	function chosenLoser = undef
