----------------------- MODULE musical_chairs_pluscal -----------------------

EXTENDS Integers, FiniteSets, TLC, Sequences

CONSTANTS CHAIRS, PLAYERS

STATE_ == { "Start", "Walking", "Sitting", "Won" }

\* Helper predicate for range of a function
Range(f) == {f[x]: x \in DOMAIN f}

(* --algorithm musical_chairs_pluscal
variables activePlayers = PLAYERS,
          activeChairs = CHAIRS,
          occupied = <<>>,
          musicPlaying = TRUE,
          state = "Start";

fair+ process Walk = "Walk"
begin
    walk:
        await
            /\ state = "Start"
            /\ musicPlaying
            /\ Cardinality(activePlayers) > 1;
        occupied := <<>>;
        state := "Walking";
        goto walk
end process

fair+ process Sit = "Sit"
\* variable loser = CHOOSE l \in activePlayers: TRUE;
begin
    sit:
        await
            /\ state = "Walking"
            /\ ~musicPlaying;
        occupied := CHOOSE o \in [activeChairs -> activePlayers]:
                        /\ (\A c \in activeChairs,
                              p1 \in activePlayers,
                              p2 \in activePlayers:
                               o[c] = p1 /\ o[c] = p2 => p1 = p2)
                        /\ (\A p \in Range(o),
                              c1 \in DOMAIN o,
                              c2 \in DOMAIN o:
                               o[c1] = p /\ o[c2] = p => c1 = c2);
        assert(\E p \in activePlayers: p \notin Range(occupied));
        state := "Sitting";
        goto sit
end process

fair+ process EliminateLoser = "EliminateLoser"
variable oldCard = Cardinality(activeChairs);
begin
    eliminateloser:
        await
            /\ state = "Sitting"
            /\ Cardinality(activePlayers) > 1
            /\ Cardinality(activePlayers) - Cardinality(Range(occupied)) = 1;
        activePlayers := Range(occupied);
        activeChairs := activeChairs \ {CHOOSE c \in activeChairs : TRUE};
        assert(Cardinality(activeChairs) = oldCard - 1);
        occupied := <<>>;
        oldCard := Cardinality(activeChairs);
        if Cardinality(activePlayers) = 1 then
            state := "Sitting";
            goto Done
        else
            state := "Start";
            goto eliminateloser
        end if
end process

fair+ process Win = "Win"
begin
    win:
        await
            /\ state = "Sitting"
            /\ Cardinality(activePlayers) = 1;
        state := "Won"
end process

fair process ChangeMusicPlaying = "ChangeMusicPlaying"
begin
    cmp:
        await state \in {"Start", "Walking"};
        musicPlaying := ~musicPlaying;
        goto cmp
end process

end algorithm; *)

\* BEGIN TRANSLATION
VARIABLES activePlayers, activeChairs, occupied, musicPlaying, state, pc, 
          oldCard

vars == << activePlayers, activeChairs, occupied, musicPlaying, state, pc, 
           oldCard >>

ProcSet == {"Walk"} \cup {"Sit"} \cup {"EliminateLoser"} \cup {"Win"} \cup {"ChangeMusicPlaying"}

Init == (* Global variables *)
        /\ activePlayers = PLAYERS
        /\ activeChairs = CHAIRS
        /\ occupied = <<>>
        /\ musicPlaying = TRUE
        /\ state = "Start"
        (* Process EliminateLoser *)
        /\ oldCard = Cardinality(activeChairs)
        /\ pc = [self \in ProcSet |-> CASE self = "Walk" -> "walk"
                                        [] self = "Sit" -> "sit"
                                        [] self = "EliminateLoser" -> "eliminateloser"
                                        [] self = "Win" -> "win"
                                        [] self = "ChangeMusicPlaying" -> "cmp"]

walk == /\ pc["Walk"] = "walk"
        /\ /\ state = "Start"
           /\ musicPlaying
           /\ Cardinality(activePlayers) > 1
        /\ occupied' = <<>>
        /\ state' = "Walking"
        /\ pc' = [pc EXCEPT !["Walk"] = "walk"]
        /\ UNCHANGED << activePlayers, activeChairs, musicPlaying, oldCard >>

Walk == walk

sit == /\ pc["Sit"] = "sit"
       /\ /\ state = "Walking"
          /\ ~musicPlaying
       /\ occupied' = (CHOOSE o \in [activeChairs -> activePlayers]:
                           /\ (\A c \in activeChairs,
                                 p1 \in activePlayers,
                                 p2 \in activePlayers:
                                  o[c] = p1 /\ o[c] = p2 => p1 = p2)
                           /\ (\A p \in Range(o),
                                 c1 \in DOMAIN o,
                                 c2 \in DOMAIN o:
                                  o[c1] = p /\ o[c2] = p => c1 = c2))
       /\ Assert((\E p \in activePlayers: p \notin Range(occupied')), 
                 "Failure of assertion at line 47, column 9.")
       /\ state' = "Sitting"
       /\ pc' = [pc EXCEPT !["Sit"] = "sit"]
       /\ UNCHANGED << activePlayers, activeChairs, musicPlaying, oldCard >>

Sit == sit

eliminateloser == /\ pc["EliminateLoser"] = "eliminateloser"
                  /\ /\ state = "Sitting"
                     /\ Cardinality(activePlayers) > 1
                     /\ Cardinality(activePlayers) - Cardinality(Range(occupied)) = 1
                  /\ activePlayers' = Range(occupied)
                  /\ activeChairs' = activeChairs \ {CHOOSE c \in activeChairs : TRUE}
                  /\ Assert((Cardinality(activeChairs') = oldCard - 1), 
                            "Failure of assertion at line 62, column 9.")
                  /\ occupied' = <<>>
                  /\ oldCard' = Cardinality(activeChairs')
                  /\ IF Cardinality(activePlayers') = 1
                        THEN /\ state' = "Sitting"
                             /\ pc' = [pc EXCEPT !["EliminateLoser"] = "Done"]
                        ELSE /\ state' = "Start"
                             /\ pc' = [pc EXCEPT !["EliminateLoser"] = "eliminateloser"]
                  /\ UNCHANGED musicPlaying

EliminateLoser == eliminateloser

win == /\ pc["Win"] = "win"
       /\ /\ state = "Sitting"
          /\ Cardinality(activePlayers) = 1
       /\ state' = "Won"
       /\ pc' = [pc EXCEPT !["Win"] = "Done"]
       /\ UNCHANGED << activePlayers, activeChairs, occupied, musicPlaying, 
                       oldCard >>

Win == win

cmp == /\ pc["ChangeMusicPlaying"] = "cmp"
       /\ state \in {"Start", "Walking"}
       /\ musicPlaying' = ~musicPlaying
       /\ pc' = [pc EXCEPT !["ChangeMusicPlaying"] = "cmp"]
       /\ UNCHANGED << activePlayers, activeChairs, occupied, state, oldCard >>

ChangeMusicPlaying == cmp

(* Allow infinite stuttering to prevent deadlock on termination. *)
Terminating == /\ \A self \in ProcSet: pc[self] = "Done"
               /\ UNCHANGED vars

Next == Walk \/ Sit \/ EliminateLoser \/ Win \/ ChangeMusicPlaying
           \/ Terminating

Spec == /\ Init /\ [][Next]_vars
        /\ SF_vars(Walk)
        /\ SF_vars(Sit)
        /\ SF_vars(EliminateLoser)
        /\ SF_vars(Win)
        /\ WF_vars(ChangeMusicPlaying)

Termination == <>(\A self \in ProcSet: pc[self] = "Done")

\* END TRANSLATION

\* Typing invariant
TypeOK ==
    /\ activePlayers  \subseteq PLAYERS
    /\ activeChairs   \subseteq CHAIRS
    /\ occupied \in [activeChairs -> activePlayers] \union {<<>>}
    /\ musicPlaying \in BOOLEAN  \* whether music is playing
    /\ state \in STATE_

\* Safety invariants

OneMorePlayerThanChairs ==
    Cardinality(activePlayers) = Cardinality(activeChairs) + 1

\* Temporal properties

ExistentialLiveness ==
    \E p \in PLAYERS: <> (activePlayers = {p})

FiniteLiveness == <> ENABLED Sit

InfiniteLiveness == <>[](Cardinality(activePlayers) = 1)

=============================================================================
\* Modification History
\* Last modified Mon Dec 23 01:12:03 EST 2019 by bandali
\* Created Sun Dec 22 02:24:10 EST 2019 by bandali
