package uw.watform.dash;

import javax.inject.Singleton

import org.eclipse.emf.ecore.EObject
import org.eclipse.xtext.naming.IQualifiedNameProvider
import org.eclipse.xtext.naming.QualifiedName

import com.google.inject.Inject
import uw.watform.dash.dash.State
import org.eclipse.emf.common.util.EList
import uw.watform.dash.dash.Signature
import org.eclipse.xtext.nodemodel.util.NodeModelUtils
import uw.watform.dash.dash.StateVar
import uw.watform.dash.dash.Declaration
import uw.watform.dash.dash.Element

/**
 * This class has convenient methods for workign with the custom namespacing of DASH
 * 
 * @author Jose Serna
 *
 */
@Singleton
class DashUtil {

    @Inject
    IQualifiedNameProvider qualifiedNameProvider;

    /**
     * Returns the qualified name of the new element to create
     * The qualified name is the name of the parent state plus the elementName
     *
     * @param o
     * @param elementName
     * @return The qualified name of the new object to create
     */
    def QualifiedName getQualifiedName(EObject o, String elementName) {
        var name = QualifiedName.create(elementName);
        var parent = o.eContainer()
        while (parent !== null) {
            if (parent instanceof State) {
                name = qualifiedNameProvider.getFullyQualifiedName(parent)
                if (name !== null)
                    name = name.append(elementName)
            }
            parent = parent.eContainer()
        }
        return name;
    }

    /**
     * Returns a list of names for objects declared in the same statement
     *
     * @param o
     * @return NameList if there are multiple declaration in the same statement
     *         or null
     */
    static def EList<Element> getNamesList(EObject o) {
        var EList<Element> l = null
        if (o instanceof Signature) {
            l = (o.getNames())
        }
        if (o instanceof StateVar) {
            l = (o).decl.getNames()
        }
        if (o instanceof Declaration) {
            l = (o).getNames()
        }
        return l
    }

    static def getTokenText(EObject object) {
        var  node = NodeModelUtils.getNode(object)
        if (node === null)
            return null
        NodeModelUtils.getTokenText(node)
    }

    /**
     * The reference name could either be a single name or a fully qualified name.
     * The last segment of a fully qualified name correspondes to the original name of the declaration
     */
    static def getReferenceName(EObject ref) {
        ref.getTokenText?.split("/").last
    }

    static def <T> void removeDuplicates(EList<T> list) {
        var objects = list.toSet
        list.clear
        list.addAll(objects)
    }
}