package uw.watform.dash.generator

import org.eclipse.xtext.generator.AbstractGenerator
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.xtext.generator.IFileSystemAccess2
import org.eclipse.xtext.generator.IGeneratorContext
import com.google.inject.Inject
import uw.watform.dash.transform.CoreDash
import org.eclipse.xtext.validation.IResourceValidator
import org.eclipse.xtext.validation.CheckMode
import org.eclipse.xtext.util.CancelIndicator
import org.eclipse.xtext.validation.Issue
import java.io.File

import uw.watform.dash.dash.Disjunction
import uw.watform.dash.dash.BiImplication
import uw.watform.dash.dash.Implication
import uw.watform.dash.dash.Conjunction
import uw.watform.dash.dash.Negation
import uw.watform.dash.dash.Comparison
import uw.watform.dash.dash.QuantOrMulti
import uw.watform.dash.dash.UnionOrDiff
import uw.watform.dash.dash.Cardinality
import uw.watform.dash.dash.Override
import uw.watform.dash.dash.Intersection
import uw.watform.dash.dash.ArrowProduct
import uw.watform.dash.dash.DomainRestriction
import uw.watform.dash.dash.RangeRestriction
import uw.watform.dash.dash.Box
import uw.watform.dash.dash.DotJoin
import uw.watform.dash.dash.Transpose
import uw.watform.dash.dash.BlockExpression
import uw.watform.dash.dash.Block
import uw.watform.dash.dash.BlockOrBar
import uw.watform.dash.dash.Quantification
import uw.watform.dash.dash.FLevelBlockOrBar
import uw.watform.dash.dash.Constant
import uw.watform.dash.dash.Ref
import uw.watform.dash.dash.Prime
import uw.watform.dash.dash.ElementRef
import uw.watform.dash.dash.Action
import uw.watform.dash.dash.Condition
import uw.watform.dash.dash.PredDecl
import uw.watform.dash.dash.Function
import uw.watform.dash.dash.FactDecl
import uw.watform.dash.dash.Int
import uw.watform.dash.dash.Declaration
import uw.watform.dash.dash.SigExt
import uw.watform.dash.dash.SigRef
import org.eclipse.emf.common.util.EList
import uw.watform.dash.dash.Import
import uw.watform.dash.dash.Signature
import org.eclipse.emf.ecore.EObject
import uw.watform.dash.DashUtil
import uw.watform.dash.dash.Transition
import uw.watform.dash.generator.elements.TransitionGeneratorElement
import uw.watform.dash.dash.State
import uw.watform.dash.generator.elements.StateGeneratorElement
import uw.watform.dash.dash.Event
import uw.watform.dash.generator.elements.EventGeneratorElement
import uw.watform.dash.dash.InvariantDecl
import uw.watform.dash.generator.elements.InvariantGeneratorElement
import java.util.Map
import uw.watform.dash.generator.elements.GeneratorElement
import org.eclipse.xtext.diagnostics.Severity
import uw.watform.dash.generator.elements.StateVarGeneratorElement
import uw.watform.dash.dash.StateVar
import uw.watform.dash.dash.Element
import uw.watform.dash.dash.ConstantRef
import uw.watform.dash.dash.SignatureRef
import uw.watform.dash.dash.ParensExpression
import uw.watform.dash.dash.ExitDecl
import uw.watform.dash.dash.EnterDecl
import uw.watform.dash.generator.elements.EnterDeclGeneratorElement
import uw.watform.dash.generator.elements.ExitDeclGeneratorElement
import uw.watform.dash.dash.CmdDecl
import uw.watform.dash.dash.Scope
import uw.watform.dash.dash.TypeScopeList
import uw.watform.dash.dash.TypeScope
import uw.watform.dash.dash.AssertDecl

class BaseGenerator extends AbstractGenerator{

    protected val EMPTY_BRACES = "{}"

    @Inject
    CoreDash dashTransform;

    @Inject
    protected IResourceValidator validator;

    @Inject
    protected Options options

    protected Map<EObject, GeneratorElement> generatorElements

    override doGenerate(Resource input, IFileSystemAccess2 fsa, IGeneratorContext context) {
        throw new UnsupportedOperationException("TODO: auto-generated method stub")
    }

    def void init(Resource input) {
        var result = input.transform
        this.generatorElements = result.getGeneratorElements();
    }

    protected def getGeneratedFileName(Resource resource){
        var file = new File(resource.URI.toFileString)
        val fileName = file.name.substring(0, file.name.lastIndexOf("."))
        fileName
    }

    def transform(Resource resource) {
        // Tansform model
        dashTransform.doTransform(resource);
    }

    def boolean validate(Resource resource) {
        var ret = true;
        // Validate the resource
        var list = validator.validate(resource, CheckMode.ALL, CancelIndicator.NullImpl);
        if (!list.isEmpty()) {
            for (Issue issue : list) {
                if (issue.severity == Severity.ERROR) {
                    System.err.println(issue)
                    ret = false
                 } else {
                    println(issue)
                }
            }
        }
        return ret
    }

    def compile(Declaration d){
        compileDeclaration(d, d.names)
    }

    def compileDeclaration(Declaration d, EList<Element> names) {
        var options = new ExpressionOptions()
        '''
        «IF d.private»private «ENDIF»«IF d.disjoint»disj «ENDIF»«names.compile» : «IF d.disjType» disj «ENDIF»«d.type.compile(options)»
        '''
    }

/******************************* Expressions **********************************/
    def dispatch CharSequence compile(Disjunction expr, ExpressionOptions options) {
        '''«expr.left.compile(options)» or «expr.right.compile(options)»'''
    }

    def dispatch CharSequence compile(BiImplication expr, ExpressionOptions options) {
        '''«expr.left.compile(options)» iff «expr.right.compile(options)»'''
    }

    def dispatch CharSequence compile(Implication expr, ExpressionOptions options) {
        '''«expr.left.compile(options)» => «expr.right.compile(options)»'''
    }

    def dispatch CharSequence compile(Conjunction expr, ExpressionOptions options) {
        '''«expr.left.compile(options)» and «expr.right.compile(options)»'''
    }

    def dispatch CharSequence compile(Negation expr, ExpressionOptions options) {
        ''''!(«expr.expression.compile(options)»)'''
    }

    def dispatch CharSequence compile(Comparison expr, ExpressionOptions options) {
        '''«expr.left.compile(options)» «IF expr.not»!«ENDIF»«expr.op» «expr.right.compile(options)»'''
    }

    def dispatch CharSequence compile(QuantOrMulti expr, ExpressionOptions options) {
        '''«expr.multi» «expr.expression.compile(options)»'''
    }

    def dispatch CharSequence compile(UnionOrDiff expr, ExpressionOptions options) {
        '''«expr.left.compile(options)» «expr.op» «expr.right.compile(options)»'''
    }

    def dispatch CharSequence compile(Cardinality expr, ExpressionOptions options) {
        '''#«expr.expression.compile(options)»'''
    }

    def dispatch CharSequence compile(Override expr, ExpressionOptions options) {
        '''«expr.left.compile(options)» ++ «expr.right.compile(options)»'''
    }

    def dispatch CharSequence compile(Intersection expr, ExpressionOptions options) {
        '''«expr.left.compile(options)» & «expr.right.compile(options)»'''
    }

    def  dispatch CharSequence compile(ArrowProduct expr, ExpressionOptions options) {
        '''«expr.left.compile(options)»«IF expr.leftMulti !== null» «expr.leftMulti»«ENDIF» -> «IF expr.rightMulti !== null»«expr.rightMulti» «ENDIF»«expr.right.compile(options)»'''
    }

    def dispatch CharSequence compile(DomainRestriction expr, ExpressionOptions options) {
        '''«expr.left.compile(options)» <: «expr.right.compile(options)»'''
    }

    def dispatch CharSequence compile(RangeRestriction expr, ExpressionOptions options) {
        '''«expr.left.compile(options)» :> «expr.right.compile(options)»'''
    }

    def dispatch CharSequence compile(Box expr, ExpressionOptions options) {
        '''«expr.left.compile(options)»[«FOR arg : expr.args SEPARATOR ","» «arg.compile(options)»«ENDFOR»]'''
    }

    def dispatch CharSequence compile(DotJoin expr, ExpressionOptions options) {
        '''«expr.left.compile(options)».«expr.right.compile(options)»'''
    }

    def dispatch CharSequence compile(Transpose expr, ExpressionOptions options) {
        '''~«expr.expression.compile(options)»'''
    }

    def dispatch CharSequence compile(BlockExpression expr, ExpressionOptions options) {
        '''{
            «FOR decl: expr.declarations SEPARATOR ","»
                «decl.compile(options)»
            «ENDFOR»
            «expr.body.compile(options)»
            }
        '''
    }

    def dispatch CharSequence compile(BlockOrBar expr, ExpressionOptions options) {
        '''| «expr.expr.compile(options)»'''
    }

    def dispatch CharSequence compile(Constant expr, ExpressionOptions options) {
        '''«expr.value»'''
    }

    def dispatch CharSequence compile(Ref expr, ExpressionOptions options) {
        '''@«expr.ref»'''
    }

    def dispatch CharSequence compile(Prime expr, ExpressionOptions options) {
        compilePrime(expr, options)
    }

     protected def compilePrime(Prime expr, ExpressionOptions options) {
        '''«expr.stateVar.name»' '''
    }

    def dispatch CharSequence compile(ElementRef expr, ExpressionOptions options) {
        compileRef(expr)
    }

    protected def compileRef(ElementRef expr) {
        var ret = ""
        var ref = expr.ref
        switch (ref) {
            Declaration, Signature, StateVar:
                ret = DashUtil.getReferenceName(expr)
            PredDecl:
                ret = ref.name
            Function:
                ret = ref.name
            Action, Condition:
                ret = ref.name
            FactDecl :
                ret = ref.name
            Element:
                ret = ref.name
        }
        ret
    }

    def dispatch CharSequence compile(Int expr, ExpressionOptions options) {
        '''«expr.value»'''
    }

    def dispatch compile(uw.watform.dash.dash.PrimitiveType expt, ExpressionOptions options) {
        '''PrimitiveType'''
    }

    def dispatch CharSequence compile(Quantification expr, ExpressionOptions options) {
        '''«expr.quantifier» «FOR decl: expr.declarations SEPARATOR ","»«decl.compile»«ENDFOR» «expr.body.compile(options)»'''
    }

    def dispatch CharSequence compile(FLevelBlockOrBar expr, ExpressionOptions options) {
        '''«IF expr.expression !== null»| «expr.expression.compile(options)»«ENDIF»'''
    }

    def dispatch CharSequence compile(ParensExpression expr, ExpressionOptions options) {
        '''(«expr.expression.compile(options)»)'''
    }

    def compile(BlockOrBar bob) {
        val options = new ExpressionOptions()
        '''«IF bob.expr !== null» | «bob.expr.compile(options)»«ENDIF»'''
    }

    def dispatch CharSequence compile(Block expr, ExpressionOptions options) {
        '''
        {
            «FOR e: expr.expressions»
                «e.compile(options)»
            «ENDFOR»
        }
        '''
    }

    def CharSequence compile(Block expr) {
        var options = new ExpressionOptions()
        expr.compile(options)
    }

    def compile(Import imp) {
        '''
        open «imp.importURI»«IF imp.names !== null»«imp.names.compileList("[", "]") »«ENDIF» «IF imp.alias !== null»as «imp.alias»«ENDIF»
        '''
    }

    def compile(Signature sig) {
        '''
        «FOR q: sig.qualifiers SEPARATOR ' ' AFTER ' '»«q»«ENDFOR»sig «sig.names.compile»«IF sig.extension !== null» «sig.extension.compile»«ENDIF» «IF sig.variables.empty»«EMPTY_BRACES»«ELSE» {
            «FOR v : sig.variables SEPARATOR ","»
            «v.compile»
            «ENDFOR»
         }«ENDIF»«IF sig.body !== null»«sig.body.compile»«ENDIF»
        '''
    }

    def compile(Function f) {
        var options = new ExpressionOptions()
        '''
        fun «f.name» «f.params.generateParams» : «f.getReturn.compile(options)» «f.body.compile»
        '''
    }

    def compile(PredDecl p) {
        '''
        pred «IF p.name !== null»«p.name» «ENDIF»«IF p.params !== null && !p.params.empty»«p.params.generateParams»«ENDIF» «p.body.compile»
        '''
    }

    def compile (AssertDecl a){
        '''
        assert «IF a.name !== null»«a.name» «ENDIF»«a.body.compile»
        '''
    }

    def  CharSequence compile(CmdDecl cmd) {
        var blockName = ""
        val cmdRef = cmd.decl
        if (cmdRef instanceof PredDecl) {
            blockName = cmdRef.name
        } else if (cmdRef instanceof AssertDecl){
            blockName = cmdRef.name
        }
        '''
        «IF cmd.name !== null»«cmd.name» :«ENDIF»«cmd.command» «blockName» «cmd.body?.compile»
            «cmd.scope?.compile»
        '''
    }

    def compile(Scope s) {
        '''
        «IF s.number > 0»
            for «s.number» «IF s.butScopes !== null»but «s.butScopes.compile»«ENDIF»«IF s.expect» expect «s.expectation»«ENDIF»
        «ELSE»
            for «s.typeScopes.compile»«IF s.expect» expect «s.expectation»«ENDIF»
        «ENDIF»
        '''
    }

    def compile(TypeScopeList tl){
        '''«FOR ts: tl.typeScopes SEPARATOR ", "»«ts.compile»«ENDFOR»'''
    }

    def compile(TypeScope ts) {
        '''«IF ts.exactly»exactly «ENDIF»«ts.number»«IF ts.element !== null» «ts.element»«ENDIF»'''
    }

    def generateParams(EList<Declaration> params) {
        '''[«FOR p : params SEPARATOR", "»«p.compile.toString.trim»«ENDFOR»]'''
    }

    def compile(FactDecl f) {
        '''
        fact «IF f.name !== null»«f.name» «ENDIF» «f.body.compile»
        '''
    }

    def compile(SigExt ext) {
        '''«IF ext.parent !== null»extends «ext.parent.compile»«ELSE»in «FOR n: ext.names SEPARATOR ','»«n.compile»«ENDFOR»«ENDIF»'''
    }

    def compile(SigRef ref) {
        var ret = ''''''
        switch ref {
            SignatureRef: ret = '''«ref.name.compile»'''
            ConstantRef: ret = '''«ref.value»'''
        }
        return ret
    }

    def compileList(EList<String> nameList) {
        nameList.compileList("", "")
    }

    def compile(EList<Element> nameList) {
        nameList.compile("", "")
    }

    def compile(Element e) {
        '''«e.name»'''
    }

    def compile(EList<Element> nameList, String before, String after) {
        '''«FOR name: nameList BEFORE before SEPARATOR ', ' AFTER after»«name.name»«ENDFOR»'''
    }

    def compileList(EList<String> nameList, String before, String after) {
        '''«FOR name: nameList BEFORE before SEPARATOR ', ' AFTER after»«name»«ENDFOR»'''
    }

    def generatorElement(EObject o) {
        generatorElements.get(o)
    }

    def generatorElement(ExitDecl d) {
        generatorElements.get(d) as ExitDeclGeneratorElement
    }

    def generatorElement(EnterDecl d) {
        generatorElements.get(d) as EnterDeclGeneratorElement
    }

    def generatorElement(InvariantDecl i) {
        generatorElements.get(i) as InvariantGeneratorElement
    }

    def generatorElement(Transition t) {
        generatorElements.get(t) as TransitionGeneratorElement
    }

    def generatorElement(State s) {
        generatorElements.get(s) as StateGeneratorElement
    }

    def generatorElement(Event e) {
        generatorElements.get(e) as EventGeneratorElement
    }

    def generatorElement(StateVar sv) {
        generatorElements.get(sv) as StateVarGeneratorElement
    }
}