package uw.watform.dash.generator

import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.xtext.generator.IFileSystemAccess2
import org.eclipse.xtext.generator.IGeneratorContext
import uw.watform.dash.dash.Module
import uw.watform.dash.dash.State
import uw.watform.dash.dash.Event
import uw.watform.dash.dash.Transition
import java.util.Collections
import java.util.Comparator
import uw.watform.dash.generator.elements.GeneratorElement
import java.util.List
import org.eclipse.xtext.EcoreUtil2
import org.eclipse.emf.ecore.EObject
import java.util.Set
import java.util.HashSet
import uw.watform.dash.dash.Prime
import uw.watform.dash.dash.ElementRef
import uw.watform.dash.dash.Action
import uw.watform.dash.dash.Condition
import uw.watform.dash.dash.InitDecl
import uw.watform.dash.dash.InvariantDecl
import uw.watform.dash.dash.Escape
import uw.watform.dash.dash.StateVar
import uw.watform.dash.generator.elements.StateGeneratorElement
import uw.watform.dash.dash.PredDecl
import uw.watform.dash.dash.Element
import uw.watform.dash.dash.CTLUnaryOperation
import uw.watform.dash.dash.CTLBinaryOperation
import uw.watform.dash.dash.DashConstant
import uw.watform.dash.dash.Conjunction
import uw.watform.dash.dash.Expr
import uw.watform.dash.dash.Disjunction
import uw.watform.dash.dash.Implication
import uw.watform.dash.dash.Negation
import uw.watform.dash.dash.AssertDecl
import uw.watform.dash.dash.EnterDecl
import uw.watform.dash.dash.ExitDecl
import java.util.ArrayList
import uw.watform.dash.dash.Signature
import uw.watform.dash.dash.FactDecl
import uw.watform.dash.dash.Function
import uw.watform.dash.dash.CmdDecl
import uw.watform.dash.generator.elements.Helper
import uw.watform.dash.dash.ParensExpression

class Dash2AlloyGenerator extends BaseGenerator {
    val EVENT_SIG = "EventLabel"
    val INTERNAL_EVENT_SIG = "InternalEvent"
    val ENVIRONMENT_EVENT_SIG = "EnvironmentEvent"
    val TRANSITION_SIG = "TransitionLabel"
    val BASE_SNAPSHOT_SIG = "BaseSnapshot"
    val SNAPSHOT_SIG = "Snapshot"
    val STATE_LABEL_SIG = "StateLabel"
    val SNAPSHOT_INPUT_VAR_PLACEHOLDER = "?input?"
    val DEFAULT_SIG_SCOPE = 3

    var Set<StateVar> snapshotVars

    var hasStateHierarchy = false
    var hasOrthogonality = false
    var hasEvents = false
    var hasEnterPredicates = false
    var hasExitPredicates = false

    override  doGenerate(Resource resource, IFileSystemAccess2 fsa, IGeneratorContext context) {
        if (!resource.validate) {
             return
        }

        init(resource)
        val fileName = getGeneratedFileName(resource)
        for (m : resource.contents.filter(Module)){
            fsa.generateFile(
                fileName + ".als",
                m.compile.toString.replace(SNAPSHOT_INPUT_VAR_PLACEHOLDER, ExpressionOptions.SNAPSHOT_VAR_NAME))
        }
    }

    override init(Resource resource) {
        super.init(resource)
        snapshotVars = new HashSet<StateVar>
    }

    def compile(Module module) {
        // Initialise
        var moduleStates = EcoreUtil2.getAllContentsOfType(module, State)
        hasStateHierarchy = moduleStates.length != module.paragraphs.filter(State).length
        hasOrthogonality = moduleStates.filter[s | s.concurrent].length > 1
        hasEvents = !EcoreUtil2.getAllContentsOfType(module, Event).empty
        hasEnterPredicates = !EcoreUtil2.getAllContentsOfType(module, EnterDecl).empty
        hasExitPredicates = !EcoreUtil2.getAllContentsOfType(module, ExitDecl).empty

        '''
        ««« Module declaration
        «IF module.module !== null»
            module «module.module.name» «module.module.names.compile("[", "]")»
        «ENDIF»
        ««« Imports
        «FOR imp : module.imports»
            «imp.compile»
        «ENDFOR»

        open util/steps[«SNAPSHOT_SIG»]
        «IF options.forcePath»open util/ordering[«SNAPSHOT_SIG»]«ENDIF»
        «IF hasOrthogonality»
            open util/boolean
        «ENDIF»

        «generateSnapshotDefinition(module)»

        «generateStateSection(module)»
        «IF hasEvents»

        «generateEventsSection(module)»
        «ENDIF»

        «generateTransitionSection(module)»
        «IF hasEnterPredicates»

           «generateEnterPredicates(module)»
        «ENDIF»
        «IF hasExitPredicates»

        «generateExitPredicates(module)»
        «ENDIF»
        «IF options.generateEnterExitFunctions»
        «generateEnterFunctions(module)»

        «generateInitFunctions(module)»

        «generateDefaultFunctions(module)»

        «generateExitFunctions(module)»
        «ENDIF»
        «generateInitialConditions(module)»

        «IF options.forcePath == false»
            «generateSignificanceAxioms(module)»
        «ENDIF»

        «generateModelDefinition(module)»

        «generateInvariants(module)»

        «IF options.checkReachability»
            «generateReachabilityProps(module)»
        «ENDIF»

            «generateAlloyParagraphs(module)»

        «IF !options.useFullNames»«generateNamesMapping()»«ENDIF»
        '''
    }

    def generateAlloyParagraphs(Module m) {
        var alloyParagraphs = m.paragraphs.reject[e | e instanceof State]
        var value = "not_implemented"
        val sb = new StringBuilder()
        for (p: alloyParagraphs) {
            switch p {
                Signature: value = p.compile.toString
                FactDecl: value = p.compile.toString
                PredDecl: value = p.compile.toString
                Function: value = p.compile.toString
                AssertDecl: value = p.compile.toString
                CmdDecl: value = p.compile.toString
                Escape: value = p.compile.toString
            }
            sb.append(value)
            sb.append(System.lineSeparator())
        }
        sb.toString
    }

    def CharSequence buildStateHierarchy(State s, String parentName) {
        var modifier = ""
        var children = s.stateItems.filter(State)
        if (children.length > 0) {
            modifier = "abstract"
        } else {
            modifier = "one"
        }
        '''
        «generateSig(modifier, s.signatureName, parentName)»
        «FOR substate: children»
            «buildStateHierarchy(substate, s.signatureName)»
        «ENDFOR»
        '''
    }

    def generateStateSection(Module module) {
        var topStateName = ""
        if (module.module !== null) {
            topStateName = module.module.name + "State"
        } else {
            topStateName = "SystemState"
        }
        '''
        /***************************** STATE SPACE ************************************/
            abstract sig «topStateName» extends «STATE_LABEL_SIG» «EMPTY_BRACES»
            «FOR s: module.paragraphs.filter(State)»
                «buildStateHierarchy(s, topStateName)»
            «ENDFOR»
        '''
    }

    def generateEventsSection(Module module) {
        '''
        /***************************** EVENTS SPACE ***********************************/
            «FOR state: module.paragraphs.filter(State)»
                «FOR e: state.stateItems.filter(Event)»
                    «IF e.generatorElement.isGenerated»
                        «generateSig("one", e.generatorElement.getName(options.useFullNames), INTERNAL_EVENT_SIG)»
                    «ELSE»
                        «generateSig("one", e.generatorElement.getName(options.useFullNames), ENVIRONMENT_EVENT_SIG)»
                    «ENDIF»
                «ENDFOR»
            «ENDFOR»
        '''
    }

/***************************** Transitions ************************************/
    def generateTransitionSection(Module module) {
        val List<Transition> transitions = EcoreUtil2.getAllContentsOfType(module,Transition)

        if (transitions.empty) {
            return ''''''
        }
        val List<ExitDecl> exitDeclarations = EcoreUtil2.getAllContentsOfType(module,ExitDecl)
        val List<EnterDecl> enterDeclarations = EcoreUtil2.getAllContentsOfType(module,EnterDecl)

        '''
        /*************************** TRANSITIONS SPACE ********************************/
            «FOR t: transitions»
            «generateSig("one", t.generatorElement.getName(options.useFullNames), TRANSITION_SIG)»
            «ENDFOR»

            «FOR t: transitions»
            // Transition «t.generatorElement.getName(true)»
            «t.generatePreconditionsPredicate»

            «t.generatePosconditionsPredicate(enterDeclarations, exitDeclarations)»

            «t.generateTransitionPredicate»
            «IF hasOrthogonality»

                «t.generateTestStablePredicate»
            «ENDIF»
            «IF hasStateHierarchy || hasOrthogonality»

                «t.generateSemanticsPredicate»
            «ENDIF»
            «ENDFOR»
        '''
    }

    def generatePreconditionsPredicate(Transition t) {
        var triggerEvent =  t.generatorElement.triggerEvent
        var options = new ExpressionOptions()
        '''
        pred pre_«t.signatureName»[s:«SNAPSHOT_SIG»] {
            «t.generatorElement.sourceState.generateInStatePredicate("s")»
            «IF triggerEvent !== null && hasEvents»
                «IF hasOrthogonality»
                    s.stable = True => {
                        «triggerEvent.signatureName» in (s.events & «ENVIRONMENT_EVENT_SIG»)
                    } else {
                        «triggerEvent.signatureName» in s.events
                    }
                «ELSE»
                    «triggerEvent.signatureName» in (s.events & «ENVIRONMENT_EVENT_SIG»)
                «ENDIF»
            «ENDIF»
            «FOR expr: t.body.conditions»
                «expr.compile(options)»
            «ENDFOR»
        }
        '''
    }

    def generatePosconditionsPredicate(Transition t,
        List<EnterDecl> enterDeclarations,
        List<ExitDecl> exitDeclarations
    ){
        val targetState = t.generatorElement.targetState
        val sourceState = t.generatorElement.sourceState
        val constrainedVars = t.getConstrainedVars(enterDeclarations, exitDeclarations)
        val generatedEvents = t.body.genEvents?.names
        val options = new ExpressionOptions()
        '''
        pred pos_«t.signatureName»[s, s':«SNAPSHOT_SIG»] {
            «IF this.options.generateEnterExitFunctions»
            s'.conf = s.conf«IF sourceState !== targetState» - «sourceState.generateExitFunctionName»[«targetState.signatureName»] + «targetState.generateEnterFunctionName»[«sourceState.signatureName», «targetState.signatureName»]«ENDIF»
            «ELSE»
            s'.conf = s.conf - «t.generatorElement.highestExitedState.signatureName» + {
                «FOR state: t.generatorElement.enteredStates.filter[s | s.stateItems.filter(State).empty] SEPARATOR " + "»
                    «state.signatureName»
                «ENDFOR»
            }
            «ENDIF»
            «IF this.options.varsUnchanged»
            «FOR stateVar : snapshotVars»
                «IF !constrainedVars.contains(stateVar) && stateVar.input == false »
                    «FOR name : stateVar.generatorElement.getNames(this.options.useFullNames)»
                        s'.«name.name» = s.«name.name»
                    «ENDFOR»
                «ENDIF»
            «ENDFOR»
            «ENDIF»
            «FOR expr: t.body.actions»
                «expr.compile(options)»
            «ENDFOR»
            «IF hasOrthogonality»
                testIfNextStable[s, s', «t.generateSetOfGeneratedEvents», «t.signatureName»] => {
                    s'.stable = True
                    «IF hasEvents»
                        s.stable = True => {
                            no ((s'.events & «INTERNAL_EVENT_SIG») «IF generatedEvents !== null && !generatedEvents.empty»- «t.generateSetOfGeneratedEvents»«ENDIF»)
                        } else {
                            no ((s'.events & «INTERNAL_EVENT_SIG») - {«IF generatedEvents !== null && !generatedEvents.empty»«t.generateSetOfGeneratedEvents» +«ENDIF» («INTERNAL_EVENT_SIG» & s.events)})
                        }
                    «ENDIF»
                } else {
                    s'.stable = False
                    «IF hasEvents»
                        s.stable = True => {
                            s'.events & «INTERNAL_EVENT_SIG» = «t.generateSetOfGeneratedEvents»
                            s'.events & «ENVIRONMENT_EVENT_SIG» = s.events & «ENVIRONMENT_EVENT_SIG»
                        } else {
                            s'.events = s.events + «t.generateSetOfGeneratedEvents»
                        }
                    «ENDIF»

                    «FOR stateVar : snapshotVars»
                        «IF !constrainedVars.contains(stateVar) && stateVar.input == true »
                            «FOR name : stateVar.generatorElement.getNames(this.options.useFullNames)»
                                s'.«name.name» = s.«name.name»
                            «ENDFOR»
                        «ENDIF»
                    «ENDFOR»
                }
                «ELSE»
                «IF hasEvents»
                    no ((s'.events & «INTERNAL_EVENT_SIG») «IF generatedEvents !== null && !generatedEvents.empty»- «t.generateSetOfGeneratedEvents»«ENDIF»)
                «ENDIF»
            «ENDIF»
            «IF hasEnterPredicates»
                «t.generateEnterPredicates(enterDeclarations)»
            «ENDIF»
            «IF hasExitPredicates»
                «t.generateExitPredicates(exitDeclarations)»
            «ENDIF»
        }
        '''
    }

    private def generateExitPredicates(Transition t, List<ExitDecl> exitDeclarations) {
        val effectiveDeclarations = getEffectiveExitDeclarations(t, exitDeclarations)
        val sb = new StringBuilder()

        for (exitDecl : effectiveDeclarations) {
            val potentialExitedState = exitDecl.generatorElement.originalParentState
            sb.append(generateInStatePredicate(potentialExitedState, "s"))
            sb.append(" => ")
            sb.append(potentialExitedState.generateExitFunctionName())
            sb.append("[s']\n")
        }
        sb.toString()
    }

    private def getEffectiveExitDeclarations(Transition t, List<ExitDecl> exitDeclarations) {
        val effectiveDeclarations = new ArrayList<ExitDecl>()
        val highestExitedState = t.generatorElement.getHighestExitedState()
        val potentialExitedStates = EcoreUtil2.getAllContentsOfType(highestExitedState, State)

        potentialExitedStates.add(highestExitedState)
        for (exitedState : potentialExitedStates) {
            for (exitDecl : exitDeclarations) {
                if (exitDecl.generatorElement.originalParentState == exitedState) {
                    effectiveDeclarations.add(exitDecl)
                }
            }
        }
        effectiveDeclarations
    }

    private def generateEnterPredicates(Transition t, List<EnterDecl> enterDeclarations) {
        val effectiveDeclarations = getEffectiveEnterDeclarations(t, enterDeclarations)
        val sb = new StringBuilder()

        for (enterDecl : effectiveDeclarations) {
            sb.append(enterDecl.generatorElement.originalParentState.generateEnterFunctionName())
            sb.append("[s']\n")
        }
        sb.toString()
    }

    private def getEffectiveEnterDeclarations(Transition t, List<EnterDecl> enterDeclarations) {
        val enteredStates = t.generatorElement.getEnteredStates
        val effectiveDeclarations = new ArrayList<EnterDecl>()

        for (enteredState : enteredStates) {
            for (enterDecl : enterDeclarations) {
                if (enterDecl.generatorElement.originalParentState == enteredState) {
                    effectiveDeclarations.add(enterDecl)
                }
            }
        }
        effectiveDeclarations
    }

    private def generateSetOfGeneratedEvents(Transition t) {
        val generatedEvents = t.body.genEvents?.names
        '''{«IF generatedEvents !==null»«FOR genEvent : generatedEvents SEPARATOR " + "»«genEvent.signatureName»«ENDFOR»«ELSE»none«ENDIF»}'''
    }

    private def generateTestStablePredicate(Transition t) {
        var conflicts = t.generatorElement.conflicts
        var triggerEvent =  t.generatorElement.triggerEvent
        val options = new ExpressionOptions()
        // The predicate variables are renamed and we use "_s" to refer to the
        // current snapshot and "s" to the next snapshot. This is because the
        // code that generates the conditions of the transitions use the variable name "s",
        // and the purpose of the predicate is to test if the possible next snapshot is stable
        var compiledConditions = '''
                                 «FOR c: t.body.conditions»
                                    «c.compile(options)»
                                 «ENDFOR»
                                 '''
        // We need to test if the next transitions is enabled using the same env values
        compiledConditions = compiledConditions.replace(SNAPSHOT_INPUT_VAR_PLACEHOLDER, "_s")

        '''
        pred enabledAfterStep_«t.signatureName»[_s, s: «SNAPSHOT_SIG», t: «TRANSITION_SIG», genEvents: set «INTERNAL_EVENT_SIG»] {
            // Preconditions
            «t.generatorElement.sourceState.generateInStatePredicate("s")»
            «compiledConditions»
            «IF !conflicts.empty || hasEvents»
            _s.stable = True => {
                «IF !conflicts.empty»
                no t & {
                    «FOR conflict: t.generatorElement.conflicts SEPARATOR " + "»
                        «conflict.signatureName»
                    «ENDFOR»
                }
                «ENDIF»
                «IF triggerEvent !== null && hasEvents»«triggerEvent.signatureName» in {(_s.events & «ENVIRONMENT_EVENT_SIG»)  + genEvents}«ENDIF»
            } else {
                «IF !conflicts.empty»
                no {_s.taken + t} & {
                    «FOR conflict: t.generatorElement.conflicts SEPARATOR " + "»
                        «conflict.signatureName»
                    «ENDFOR»
                }
                «ENDIF»
                «IF triggerEvent !== null && hasEvents»«triggerEvent.signatureName» in {_s.events  + genEvents}«ENDIF»
            }
            «ENDIF»
        }
        '''
    }

    private def getConstrainedVars(Transition t, List<EnterDecl> enterDeclarations, List<ExitDecl> exitDeclarations) {
        var constrainedVars = new HashSet<StateVar>
        // Add vars directly constrained by the transition body
        for (action : t.body.actions) {
            var primeVars = EcoreUtil2.getAllContentsOfType(action, Prime)
            // primeVars from named actions
            var namedActions = EcoreUtil2.getAllContentsOfType(action, ElementRef).filter[
                a | a.ref instanceof Action
            ]

            for (na : namedActions) {
                primeVars.addAll(EcoreUtil2.getAllContentsOfType((na.ref as Action).formula, Prime))
            }
            constrainedVars.addAll(primeVars.map[
                d | d.stateVar.eContainer.eContainer as StateVar
            ])
        }

        // Add variables constrained by enter/exit predicates
        var effectiveDeclarations = new ArrayList<EObject>()
        effectiveDeclarations.addAll(getEffectiveEnterDeclarations(t, enterDeclarations))
        effectiveDeclarations.addAll(getEffectiveExitDeclarations(t, exitDeclarations))
        for (decl : effectiveDeclarations) {
            var vars = EcoreUtil2.getAllContentsOfType(decl, ElementRef)
            constrainedVars.addAll(vars.filter[
                d | d.ref.eContainer.eContainer instanceof StateVar
            ].map[
                d | d.ref.eContainer.eContainer as StateVar
            ])
        }
        constrainedVars
    }

    def generateTransitionPredicate(Transition t) {
        '''
        pred «t.signatureName»[s, s': «SNAPSHOT_SIG»] {
            pre_«t.signatureName»[s]
            pos_«t.signatureName»[s, s']
            «IF (hasOrthogonality || hasStateHierarchy)»
                semantics_«t.signatureName»[s, s']
            «ENDIF»
        }
        '''
    }

    def generateSemanticsPredicate(Transition t) {
        var conflicts = t.generatorElement.conflicts
        var higherProrityTransitions = t.generatorElement.transitionsWithHigherPriority
        '''
        pred semantics_«t.signatureName»[s, s': «SNAPSHOT_SIG»] {
            «IF hasOrthogonality»
                (s.stable = True) => {
                    // SINGLE semantics
                    s'.taken = «t.signatureName»
                } else {
                    // SINGLE semantics
                    s'.taken = s.taken + «t.signatureName»
                    «IF !conflicts.empty»
                        // Bigstep "TAKE_ONE" semantics
                        no s.taken & {
                            «FOR conflict: t.generatorElement.conflicts SEPARATOR " + "»
                                «conflict.signatureName»
                            «ENDFOR»
                        }
                    «ENDIF»
                }
            «ELSE»
                s'.taken = «t.signatureName»
            «ENDIF»
            «IF hasStateHierarchy»
                «IF !higherProrityTransitions.empty»
                    // Priority "SOURCE-PARENT" semantics
                    «FOR hpt : higherProrityTransitions»
                        !pre_«hpt.signatureName»[s]
                    «ENDFOR»
                «ENDIF»
            «ENDIF»
        }
        '''
    }

/************************** Enter Functions ***********************************/

    def generateEnterFunctions(Module m) {
        var states = m.paragraphs.filter(State)

        if (states.empty) {
            return ''''''
        }

        '''
        /***************************** ENTER FUNCTIONS ********************************/
            fun enter_State[src, dest: «STATE_LABEL_SIG»]: set «STATE_LABEL_SIG» {
                default_State
            }

            «FOR state: states»
            «state.generateEnterFunction»
            «ENDFOR»
        '''
    }

    def CharSequence generateEnterFunction(State s) {
        var subStates = s.stateItems.filter(State)
        var parentState = s.generatorElement.parentState

        '''
        fun «s.generateEnterFunctionName»[src, dest: «STATE_LABEL_SIG»]: set «s.signatureName» {
            «IF parentState !== null»
                    src in «parentState.signatureName» => «s.generateInitFunctionName»[dest]
                    else «s.generateInitFunctionName»[dest] + «parentState.generateEnterFunctionName»[src, dest]
            «ELSE»
                    src in «STATE_LABEL_SIG» => «s.generateInitFunctionName»[dest]
                    else «s.generateInitFunctionName»[dest] + enter_State[src, dest]
            «ENDIF»
        }

        «FOR substate: subStates»
            «substate.generateEnterFunction»
        «ENDFOR»
        '''
    }

    def generateEnterFunctionName(State s) {
        '''enter_«s.signatureName»'''
    }

/*****************************Enter Predicates ********************************/
    def generateEnterPredicates(Module m) {
        val states = m.paragraphs.filter(State)

        if (states.empty) {
            return ''''''
        }

        val enterDeclarations = EcoreUtil2.getAllContentsOfType(m, EnterDecl)

        '''
        /****************************** ENTER PREDICATES *******************************/

            «FOR declaration: enterDeclarations»
            «generateEnterPredicate(declaration)»
            «ENDFOR»
        '''
    }

    def CharSequence generateEnterPredicate(EnterDecl enterDeclaration) {
        val containerState = enterDeclaration.generatorElement.originalParentState
        val options = new ExpressionOptions()
        '''
        pred «containerState.generateEnterFunctionName»[s: «SNAPSHOT_SIG»] {
            «FOR e: enterDeclaration.body.expressions»
                «e.compile(options)»
            «ENDFOR»
        }
        '''
    }

/*************************** Exit Predicates **********************************/
    def generateExitPredicates(Module m) {
        val states = m.paragraphs.filter(State)

        if (states.empty) {
            return ''''''
        }

        val exitDeclarations = EcoreUtil2.getAllContentsOfType(m, ExitDecl)

        '''
        /****************************** EXIT PREDICATES *******************************/

            «FOR declaration: exitDeclarations»
            «generateExitPredicate(declaration)»
            «ENDFOR»
        '''
    }

    def CharSequence generateExitPredicate(ExitDecl exitDeclaration) {
        val containerState = exitDeclaration.generatorElement.originalParentState
        val options = new ExpressionOptions()
        '''
        pred «containerState.generateExitFunctionName»[s: «SNAPSHOT_SIG»] {
            «FOR e: exitDeclaration.body.expressions»
                «e.compile(options)»
            «ENDFOR»
        }
        '''
    }

/*************************** Init Functions ***********************************/

    def generateInitFunctions(Module m) {
        var states = m.paragraphs.filter(State)

        if (states.empty) {
            return ''''''
        }

        '''
        /**************************** INIT FUNCTIONS **********************************/
            «FOR state: states»
            «state.generateInitFunction»
            «ENDFOR»
        '''
    }

    def CharSequence generateInitFunction(State s) {
        var subStates = s.stateItems.filter(State)
        var concurrentSubStates = subStates.filter[subState | subState.concurrent == true]

        '''
        fun «s.generateInitFunctionName»[dest: «STATE_LABEL_SIG»]: set «s.signatureName» {
            «IF concurrentSubStates.length > 0»
                «FOR region : concurrentSubStates SEPARATOR " +"»
                ((!(dest in «region.signatureName») or (dest = «region.signatureName»)) => «region.generateDefaultFunctionName» else none)
                «ENDFOR»
            «ELSE»
                ((!(dest in «s.signatureName») or (dest = «s.signatureName»)) => «s.generateDefaultFunctionName» else none)
            «ENDIF»
        }

        «FOR subState: subStates»
            «subState.generateInitFunction»
        «ENDFOR»
        '''
    }

    def generateInitFunctionName(State s) {
        '''init_«s.signatureName»'''
    }

/************************** Default Functions ***********************************/

    def generateDefaultFunctions(Module m) {
        var states = m.paragraphs.filter(State)

        if (states.empty) {
            return ''''''
        }

        '''
        /***************************** DEFAULT FUNCTIONS ******************************/
            fun default_State: set «STATE_LABEL_SIG» {
                «FOR state: states SEPARATOR " +"»
                    «state.generateDefaultFunctionName»
                «ENDFOR»
            }

            «FOR state: states»
            «state.generateDefaultFunction»
            «ENDFOR»
        '''
    }

    def CharSequence generateDefaultFunction(State s) {
        var subStates = s.stateItems.filter(State)
        var defaultState = subStates.filter[subState | subState.isDefault == true]
        var concurrentSubStates = subStates.filter[subState | subState.concurrent == true]

        if (subStates.empty) {
            return ''''''
        }

        '''
        «IF !defaultState.empty || !concurrentSubStates.empty»
            fun «s.generateDefaultFunctionName»: set «s.signatureName» {
                «IF !concurrentSubStates.empty»
                    «FOR region: concurrentSubStates SEPARATOR " +"»
                        «region.generateDefaultFunctionName»
                    «ENDFOR»
                «ELSE»
                    «defaultState.get(0).generateDefaultFunctionName»
                «ENDIF»
            }
        «ENDIF»

        «FOR subState: subStates»
            «subState.generateDefaultFunction»
        «ENDFOR»
        '''
    }

    def generateDefaultFunctionName(State s) {
        if (s.stateItems.filter(State).length > 0) {
            '''default_«s.signatureName»'''
        } else {
            '''«s.signatureName»'''
        }

    }

/************************** Exit Functions ***********************************/

    def generateExitFunctions(Module m) {
        var states = m.paragraphs.filter(State)

        if (states.empty) {
            return ''''''
        }

        '''
        /****************************** EXIT FUNCTIONS ********************************/
            fun exit_State[dest: «STATE_LABEL_SIG»]: set «STATE_LABEL_SIG» {
                «STATE_LABEL_SIG»
            }

            «FOR state: states»
            «state.generateExitFunction»
            «ENDFOR»
        '''
    }

    def CharSequence generateExitFunction(State s) {
        var subStates = s.stateItems.filter(State)
        var parentState = s.generatorElement.parentState

        '''
        fun «s.generateExitFunctionName»[dest: «STATE_LABEL_SIG»]: set «STATE_LABEL_SIG» {
            «IF parentState === null»
                dest in «STATE_LABEL_SIG» => «s.signatureName»
                else exit_State[dest]
            «ELSE»
                dest in «parentState.signatureName» => «s.signatureName»
                else «parentState.generateExitFunctionName»[dest]
            «ENDIF»
        }

        «FOR subState: subStates»
            «subState.generateExitFunction»
        «ENDFOR»
        '''
    }

    def generateExitFunctionName(State s) {
        '''exit_«s.signatureName»'''
    }

    def generateSnapshotDefinition(Module module) {
        snapshotVars = new HashSet<StateVar>
        for (s : module.paragraphs.filter(State)) {
            snapshotVars.addAll(s.stateItems.filter(StateVar))
        }
        '''
        // Snapshot definition
            sig «SNAPSHOT_SIG» extends «BASE_SNAPSHOT_SIG» {
                «IF hasOrthogonality»
                    stable: one Bool,
                «ENDIF»
                «IF hasEvents»
                    events: set «EVENT_SIG»,
                «ENDIF»
                «FOR d: snapshotVars SEPARATOR ","»
                    «d.compile»
                «ENDFOR»
            }
        '''
    }

/**************************** Operations Axiom*********************************/

    def generateSignificanceAxioms(Module m) {
        var transitions = EcoreUtil2.getAllContentsOfType(m, Transition)
        '''
        /**************************** SIGNIFICANCE AXIOMS *****************************/
            /** Uncomment the operationsAxiom to find a significant scope. */
            pred significance {
                steps/reachabilityAxiom
                // operationsAxiom
            }
            run significance «generateSignificantScope(m)»
                expect 1

            pred operationsAxiom {
                «FOR t : transitions»
                some s, s': «SNAPSHOT_SIG» | «t.signatureName»[s, s']
                «ENDFOR»
            }
        '''
    }

    def generateModelDefinition(Module m) {
        var transitions = EcoreUtil2.getAllContentsOfType(m, Transition)
        '''
        /***************************** MODEL DEFINITION *******************************/
            pred operation[s, s': «SNAPSHOT_SIG»] {
                «FOR t: transitions SEPARATOR " or"»
                    «t.signatureName»[s, s']
                «ENDFOR»
            }

            pred small_step[s, s': «SNAPSHOT_SIG»] {
                operation[s, s']
            }
            «IF hasOrthogonality»

                pred testIfNextStable[s, s': Snapshot, genEvents: set «INTERNAL_EVENT_SIG», t:«TRANSITION_SIG»] {
                    «FOR t: transitions»
                        !enabledAfterStep_«t.signatureName»[s, s', t, genEvents]
                    «ENDFOR»
                }

                pred isEnabled[s:«SNAPSHOT_SIG»] {
                    «FOR t: transitions SEPARATOR "or"»
                        pre_«t.signatureName»[s]
                    «ENDFOR»
                }
            «ENDIF»

            pred equals[s, s': «SNAPSHOT_SIG»] {
                s'.conf = s.conf
                «IF hasEvents»
                    s'.events = s.events
                «ENDIF»
                s'.taken = s.taken
                «generateSnapshotDeclarationsUnchanged»
            }

            fact {
                all s: «SNAPSHOT_SIG» | s in initial iff init[s]
                all s, s': «SNAPSHOT_SIG» | s->s' in nextStep iff small_step[s, s']
                all s, s': «SNAPSHOT_SIG» | equals[s, s'] => s = s'
                «IF hasOrthogonality»
                    all s: «SNAPSHOT_SIG» | (isEnabled[s] && no s': «SNAPSHOT_SIG» | small_step[s, s']) => s.stable = False
                    all s: Snapshot | s.stable = False => some s.nextStep
                «ENDIF»
                «IF options.forcePath»
                    path
                «ELSE»
                    significance
                «ENDIF»
            }
            «IF options.forcePath»

                pred path {
                    all s:«SNAPSHOT_SIG», s': s.next | operation[s, s']
                    init[first]
                }
                run path «generateSignificantScope(m)»
                    expect 1
            «ENDIF»
            «IF options.assumeSingleInput && hasEvents»

            // Single input assumption
            fact {
                all s: «SNAPSHOT_SIG» | lone (s.events & «ENVIRONMENT_EVENT_SIG»)
            }
            «ENDIF»
        '''
    }

    def generateSnapshotDeclarationsUnchanged() {
        '''
        «IF !snapshotVars.empty»
            // Model specific declarations
            «FOR stateVar : snapshotVars»
                «FOR name : stateVar.generatorElement.getNames(options.useFullNames)»
                    s'.«name.name» = s.«name.name»
                «ENDFOR»
            «ENDFOR»
        «ENDIF»
        '''
    }

    def generateInitialConditions(Module m) {
        var initBlocks = EcoreUtil2.getAllContentsOfType(m, InitDecl)
        val defaultStates = new ArrayList<State>
        m.paragraphs.filter(State).forEach[
            s | defaultStates.addAll(
                Helper.getEnteredChildrenStates(s).filter[
                    enteredState | enteredState.isDefault
                ]
            )
        ]
        val topStates = m.paragraphs.filter(State)
        '''
        /****************************** INITIAL CONDITIONS ****************************/
            pred init[s: «SNAPSHOT_SIG»] {
                «IF options.generateEnterExitFunctions»
                    s.conf = default_State
                «ELSE»
                    s.conf = {
                        «IF hasStateHierarchy»
                            «FOR defaultState: defaultStates SEPARATOR " + "»
                                «defaultState.signatureName»
                            «ENDFOR»
                        «ELSE»
                            «FOR state: topStates SEPARATOR " + "»
                                «state.signatureName»
                            «ENDFOR»
                        «ENDIF»
                    }
                «ENDIF»
                no s.taken
                «IF hasOrthogonality»
                    s.stable = True
                «ENDIF»
                «IF hasEvents»
                    no s.events & InternalEvent
                «ENDIF»
                «IF !initBlocks.empty»
                    // Model specific constraints
                    «FOR init: initBlocks»
                        «init.body.compile.trimBlockBraces»
                    «ENDFOR»
                «ENDIF»
            }
        '''
    }

    def trimBlockBraces(CharSequence expr) {
        var trimmedExpr = expr.toString
        // Convert Char to String because xtend does not work well with char values
        if (trimmedExpr.charAt(0).toString == "{") {
            trimmedExpr = trimmedExpr.substring(1)
        }

        if (trimmedExpr.charAt(trimmedExpr.length - 2).toString == "}") {
            trimmedExpr = trimmedExpr.substring(0, trimmedExpr.length - 2)
        }
        trimmedExpr.trim
    }

/******************************** Invariants **********************************/

    def generateInvariants(Module m) {
        var invariants = EcoreUtil2.getAllContentsOfType(m, InvariantDecl)
        '''
            «IF !invariants.empty»
            /****************************** INVARIANTS ************************************/
                «FOR inv : invariants»
                    «inv.generateInvariantFact»
                «ENDFOR»
            «ENDIF»
        '''
    }

    def generateInvariantFact(InvariantDecl inv) {
        var parentState = inv.generatorElement.originalParentState
        '''
            fact «IF inv.name !== null»«inv.signatureName» «ENDIF»{
                all s: «SNAPSHOT_SIG» | «parentState.generateInStatePredicate("s")» => {
                    «inv.body.compile.trimBlockBraces»
                }
            }

        '''
    }

    def generateInStatePredicate(State s, String snapshotPrefix) {
        if (s.stateItems.filter(State).empty) {
           '''«s.signatureName» in «snapshotPrefix».conf'''
        } else {
           '''(some «s.signatureName» & «snapshotPrefix».conf)'''
        }
    }

    def compile(Escape escape) {
        '''
        «escape.content.replace("{escape}", "").replace("{/escape}", "")»
        '''
    }

    /******************************** Reachability Properties **********************************/

    def generateReachabilityProps(Module m) {
        var basicStates = EcoreUtil2.getAllContentsOfType(m, State).map[
            s | s.generatorElement
        ].filter[s | s.isBasicState]
        '''
        «IF !basicStates.empty»
        /************************* REACHABILITY CHECKS ********************************/
            «FOR state : basicStates»
                «generateReachabilityCheck(state, m)»

            «ENDFOR»
        «ENDIF»
        '''
    }

    def generateReachabilityCheck(StateGeneratorElement s, Module m) {
        '''
        /** Check if state «s.signatureName» is reachable */
        pred «s.signatureName»_reachable {
            ctl_mc[ef[{«getSnapshotSetComprehension(BooleanOperation.CONJUNCTION)» «s.node.generateInStatePredicate("s")»}]]
        }
        run «s.node.signatureName»_reachable «generateSignificantScope(m)»
            expect 1
        '''
    }

    def generateSignificantScope(Module m) {
        var eventsScope = EcoreUtil2.getAllContentsOfType(m, Event).size
        val signatures = EcoreUtil2.getAllContentsOfType(m, Signature).reject[
            qualifiers.contains("lone") ||
            qualifiers.contains("one") ||
            qualifiers.contains("abstract")
        ]
        '''
        for «options.snapshotScope» «SNAPSHOT_SIG», «eventsScope» «EVENT_SIG»«IF !signatures.empty»,«ENDIF»
            «FOR s : signatures SEPARATOR ","»
                «FOR name : s.names SEPARATOR ", "»«DEFAULT_SIG_SCOPE» «name.name»«ENDFOR»
            «ENDFOR»
        '''
    }

    /**
     * Prints the mapping between generated names and a descriptive names in the
     *  original DASH model
     */
    def generateNamesMapping() {
        var generatorElements = generatorElements.values.toList
        Collections.sort(generatorElements,new Comparator<GeneratorElement>() {
            override compare(GeneratorElement o1, GeneratorElement o2) {
                return o1.shortName.compareTo(o2.shortName)
            }
        })
        '''
        /******************************** NAMES MAP ***********************************/
            «FOR o : generatorElements»
                // «o.getName(false).alloyName» -> «o.getName(true)»
            «ENDFOR»
        '''
    }

    def dispatch CharSequence compile(CTLUnaryOperation expr, ExpressionOptions options) {
        var includePrefix = includeCtlPrefix(expr.expression)
        var ret = ""
        if ((expr.op == "ex" || expr.op == "ax") && hasOrthogonality) {
            ret = translateCTLNextExpression(expr, options).toString
        } else {
            ret = '''«expr.op»[{«IF includePrefix» «getSnapshotSetComprehension(expr.expression, BooleanOperation.IMPLICATION)»«ENDIF» «expr.expression.compile(options)»}]'''
        }
        ret
    }

    def boolean includeCtlPrefix(Expr e) {
        var ret = true
        switch e {
            Disjunction: {
                ret = !(isCtlExpression(e.left) || isCtlExpression(e.right))
            }
            Conjunction: {
                ret = !(isCtlExpression(e.left) || isCtlExpression(e.right))
            }
            Implication: {
                ret = !(isCtlExpression(e.left) || isCtlExpression(e.right))
            }
            Negation: {
                ret = !isCtlExpression(e.expression)
            }
            ParensExpression: {
                ret = includeCtlPrefix(e.eContents.get(0) as Expr)
            }
        }
        return ret
    }

    def dispatch CharSequence compile(CTLBinaryOperation expr, ExpressionOptions options) {
        var left = '''«expr.left.compile(options)»'''
        if (expr.left instanceof CTLUnaryOperation) {
            left = '''{«getSnapshotSetComprehension(BooleanOperation.CONJUNCTION)» «left»}'''
        }

        var right = '''«expr.right.compile(options)»'''
        if (expr.right instanceof CTLUnaryOperation) {
            right = '''{«getSnapshotSetComprehension(BooleanOperation.CONJUNCTION)» «right»}'''
        }

        return '''«expr.op»[«left», «right» ]'''
    }

    def CharSequence translateCTLNextExpression(CTLUnaryOperation expr, ExpressionOptions options) {
        '''
        ax[«expr.op.replace("x", "u")»[
            {«uw.watform.dash.generator.ExpressionOptions.SNAPSHOT_VAR_NAME»: «SNAPSHOT_SIG» | s.stable = False},
            {«uw.watform.dash.generator.ExpressionOptions.SNAPSHOT_VAR_NAME»: «SNAPSHOT_SIG» | s.stable = True and «expr.expression.compile(options)»}
        ]]'''
    }

    private def getSnapshotSetComprehension(Expr expr, BooleanOperation operator) {
        var containsCTLFormulas = (expr !== null &&
            !(EcoreUtil2.getAllContentsOfType(expr, CTLBinaryOperation).empty &&
              EcoreUtil2.getAllContentsOfType(expr, CTLUnaryOperation).empty
            )
        )
        var ret = ""
        if (!containsCTLFormulas) {
            var op = ""
            switch operator {
                case CONJUNCTION:
                    op = "and"
                case IMPLICATION:
                    op = "=>"
            }
            ret = '''«uw.watform.dash.generator.ExpressionOptions.SNAPSHOT_VAR_NAME»: «SNAPSHOT_SIG» |«IF hasOrthogonality» s.stable = True «op»«ENDIF»'''
        }
        return ret
    }

    private def getSnapshotSetComprehension(BooleanOperation operator) {
        getSnapshotSetComprehension(null, operator)
    }

    override dispatch CharSequence compile(Conjunction expr, ExpressionOptions options) {
        var ret = ''''''
        if (isCtlExpression(expr.left) || isCtlExpression(expr.right)) {
            ret = '''
                and_ctl[
                    «IF !isCtlExpression(expr.left)»{«getSnapshotSetComprehension(expr.left, BooleanOperation.IMPLICATION)» «ENDIF»«expr.left.compile(options)»«IF !isCtlExpression(expr.left)»}«ENDIF»,
                    «IF !isCtlExpression(expr.right)»{«getSnapshotSetComprehension(expr.right, BooleanOperation.IMPLICATION)» «ENDIF»«expr.right.compile(options)»«IF !isCtlExpression(expr.right)»}«ENDIF»
                ]'''
        } else {
            ret = '''«expr.left.compile(options)» and «expr.right.compile(options)»'''
        }
        return ret
    }

    override dispatch CharSequence compile(Disjunction expr, ExpressionOptions options) {
        var ret = ''''''
        if (isCtlExpression(expr.left) || isCtlExpression(expr.right)) {
            ret = '''
                or_ctl[
                    «IF !isCtlExpression(expr.left)»{«getSnapshotSetComprehension(BooleanOperation.IMPLICATION)» «ENDIF»«expr.left.compile(options)»«IF !isCtlExpression(expr.left)»}«ENDIF»,
                    «IF !isCtlExpression(expr.right)»{«getSnapshotSetComprehension(BooleanOperation.IMPLICATION)» «ENDIF»«expr.right.compile(options)»«IF !isCtlExpression(expr.right)»}«ENDIF»
                ]'''
        } else {
            ret = '''«expr.left.compile(options)» or «expr.right.compile(options)»'''
        }
        return ret
    }

    override dispatch CharSequence compile(Implication expr, ExpressionOptions options) {
        var ret = ''''''
        if (isCtlExpression(expr.left) || isCtlExpression(expr.right)) {
            ret = '''
                imp_ctl[
                    «IF !isCtlExpression(expr.left)»{«getSnapshotSetComprehension(BooleanOperation.CONJUNCTION)» «ENDIF»«expr.left.compile(options)»«IF !isCtlExpression(expr.left)»}«ENDIF»,
                    «IF !isCtlExpression(expr.right)»{«getSnapshotSetComprehension(BooleanOperation.CONJUNCTION)» «ENDIF»«expr.right.compile(options)»«IF !isCtlExpression(expr.right)»}«ENDIF»
                ]'''
        } else {
            ret = '''«expr.left.compile(options)» => «expr.right.compile(options)»'''
        }
        return ret
    }

    override dispatch CharSequence compile(Negation expr, ExpressionOptions options) {
        var ret = ''''''
        if (isCtlExpression(expr.expression)) {
            ret = '''
                not_ctl[
                    «IF !isCtlExpression(expr.expression)»{«getSnapshotSetComprehension(BooleanOperation.IMPLICATION)» «ENDIF»«expr.expression.compile(options)»«IF !isCtlExpression(expr.expression)»}«ENDIF»
                ]'''
        } else {
            ret = '''!(«expr.expression.compile(options)»)'''
        }
        return ret
    }

    def boolean isCtlExpression(EObject e) {
        (e instanceof CTLBinaryOperation || e instanceof CTLUnaryOperation) ||
        (e instanceof ParensExpression && isCtlExpression(e.eContents.get(0)))
    }

    override compile(PredDecl p) {
        val options = new ExpressionOptions()
        '''
        pred «IF p.name !== null»«p.name» «ENDIF»«IF p.params !== null && !p.params.empty»«p.params.generateParams»«ENDIF» {
            «FOR e: p.body.expressions»
                «addCTLModuleCall(e.compile(options), e)»
            «ENDFOR»
        }
        '''
    }

    override compile(AssertDecl a) {
        val options = new ExpressionOptions()
        '''
        assert «IF a.name !== null»«a.name» «ENDIF» {
            «FOR e: a.body.expressions»
                «addCTLModuleCall(e.compile(options), e)»
            «ENDFOR»
        }
        '''
    }

    /**
     * Adds the call to the ctl module method if the expression provided
     * contains CTL operators
     *
     */
    def CharSequence addCTLModuleCall(CharSequence compiledExpression, Expr expr){
        var ret = compiledExpression
        var hasCtlOperations = (
            !EcoreUtil2.getAllContentsOfType(expr, Expr).filter[
                c | isCtlExpression(c)
            ].empty ||
            isCtlExpression(expr)
        )
        if (hasCtlOperations) {
            ret = "ctl_mc[" + compiledExpression + "]"
        }
        ret
    }

    def dispatch CharSequence compile(DashConstant expr, ExpressionOptions options) {
        '''«ExpressionOptions.SNAPSHOT_VAR_NAME».«expr.value»'''
    }

    override dispatch CharSequence compile(Prime expr, ExpressionOptions options) {
        // A prime expression always refers to a StateVar element
        var element = expr.stateVar
        var stateVar = element.eContainer.eContainer as StateVar
        var snapshotDeclName = stateVar.generatorElement.getName(this.options.useFullNames, element)
        '''(s'.«snapshotDeclName»)'''
    }

    override dispatch CharSequence compile(ElementRef expr, ExpressionOptions options) {
        var CharSequence result = ''''''
        var ref = expr.ref
        switch ref {
            Element: {
                if (ref.eContainer.eContainer instanceof StateVar) {
                    var stateVar = ref.eContainer.eContainer as StateVar
                    var name = stateVar.generatorElement.getName(this.options.useFullNames, ref)
                    if (stateVar.isInput) {
                        result = '''(«SNAPSHOT_INPUT_VAR_PLACEHOLDER».«name»)'''
                    } else {
                        result = '''(«ExpressionOptions.SNAPSHOT_VAR_NAME».«name»)'''
                    }
                } else {
                    result = compileRef(expr)
                }
            }
            Condition: {
                result = ref.formula.compile(options)
            }
            Action: {
                result = ref.formula.compile(options)
            }
            State: {
                result = ref.generatorElement.getName(this.options.useFullNames)
            }
            Event: {
                result = ref.generatorElement.getName(this.options.useFullNames)
            }
            default:
                result = compileRef(expr)
        }
        result
    }

    def generateSig(String modifier, String name, String parentName) {
        '''«IF modifier !== null»«modifier» «ENDIF»sig «name.alloyName» «IF parentName !== null»extends «parentName.alloyName» «ENDIF»«EMPTY_BRACES»
        '''
    }

    def alloyName(String str) {
        var String alloyName = "missing_name"
        if (str !== null){
            alloyName = str.substring(0, 1).toUpperCase() + str.substring(1);
        }
        alloyName
    }

    def signatureName(EObject o) {
        signatureName(o.generatorElement)
    }

    def signatureName(GeneratorElement o) {
        o.getName(options.useFullNames).alloyName
    }

    def compile(StateVar sv){
        var names = sv.generatorElement.getNames(options.useFullNames)
        compileDeclaration(sv.decl, names)
    }

    enum BooleanOperation {
        CONJUNCTION,
        IMPLICATION
    }

}