package uw.watform.dash.generator

import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.xtext.generator.IFileSystemAccess2
import org.eclipse.xtext.generator.IGeneratorContext
import uw.watform.dash.dash.Module
import uw.watform.dash.dash.Paragraph
import uw.watform.dash.dash.State
import uw.watform.dash.dash.StateItem
import uw.watform.dash.dash.Event
import uw.watform.dash.dash.Action
import uw.watform.dash.dash.TransBlock
import uw.watform.dash.dash.Signature
import uw.watform.dash.dash.Transition
import uw.watform.dash.dash.StateQualNames
import uw.watform.dash.dash.EventQualNames
import org.eclipse.emf.ecore.EObject
import uw.watform.dash.dash.InitDecl
import uw.watform.dash.dash.Condition
import uw.watform.dash.dash.InvariantDecl
import uw.watform.dash.dash.StateVar

class Dash2DashGenerator extends BaseGenerator {

    override  doGenerate(Resource resource, IFileSystemAccess2 fsa, IGeneratorContext context) {
        if (!resource.validate) {
             return
        }
        init(resource)
        val fileName = getGeneratedFileName(resource)
        for (m : resource.contents.filter(Module)){
            fsa.generateFile(
                fileName + ".dsh",
                m.compile)
        }
    }

    def compile(Module module) {
        '''
        «FOR imp : module.imports»
            «imp.compile»
        «ENDFOR»

        «FOR p : module.paragraphs»
            «p.compile»

        «ENDFOR»
        '''
    }

    def compile(State s){
        '''
        «IF s.concurrent»conc «ENDIF»state «s.name» «IF s.stateItems.empty»«EMPTY_BRACES»
        «ELSE»{
            «FOR item : s.stateItems»
                «item.compile»

            «ENDFOR»
        }
        «ENDIF»
        '''
    }

    def compile(Event e){
        '''
        event «e.getDescriptiveName» «e.body.compile»
        '''
    }

    def compile(Condition c){
        '''
        condition «c.getDescriptiveName» «c.body.compile»
        '''
    }

    def compile(Action action) {
        val options = new ExpressionOptions()
        '''
        action «action.getDescriptiveName» {«action.formula.compile(options)»} «EMPTY_BRACES»
        '''
    }

    def compile(TransBlock block){
        '''
        «IF block.trans.empty»«EMPTY_BRACES»«ELSE»
        {
            «FOR exp : block.trans»
                «exp.compile»
            «ENDFOR»
        }
        «ENDIF»
        '''
    }

    def CharSequence compile(StateItem item){
        switch (item) {
            State: item.compile
            Event: item.compile
            Action: item.compile
            Condition: item.compile
            Transition: item.compile
            InitDecl: item.compile
            StateVar: item.compile
            InvariantDecl: item.compile
        }
    }

    def compile(StateVar sv) {
        '''
        «IF sv.input == true»env «ENDIF»«sv.decl.compile»
        '''
    }

    def compile(InitDecl init) {
        '''
        init «init.getDescriptiveName» «init.body.compile»
        '''
    }

    def compile(InvariantDecl inv) {
        '''
        invariant «inv.getDescriptiveName» «inv.body.compile»
        '''
    }

    def compile(Paragraph p) {
        switch (p) {
            Signature: return p.compile
            State: return p.compile
        }
    }

    def compile(Transition t){
        var body = t.body
        val options = new ExpressionOptions()
        '''
        trans «t.getDescriptiveName» {
            from «if (body.source !== null && !body.source.isAll) (body.source as StateQualNames).compile»
            «IF body.trigger !== null»on «body.trigger.compile»«ENDIF»
            «IF !body.conditions.empty»when  «FOR condition : body.conditions SEPARATOR ','»«condition.compile(options)»«ENDFOR»«ENDIF»
            goto «if (body.target !== null && !body.target.isAll) (body.target as StateQualNames).compile»
            «IF !body.actions.empty»do «FOR action: body.actions SEPARATOR ','»«action.compile(options)»«ENDFOR»«ENDIF»
            «IF body.genEvents !== null»send «body.genEvents.compile»«ENDIF»
        }
        '''
    }

    def compile(StateQualNames list){
        '''
        «list.names.map[state | state.getDescriptiveName].join(', ')»
        '''
    }

    def compile(EventQualNames list){
        '''
        «list?.names.map[event | event.getDescriptiveName].join(', ')»
        '''
    }

    private def getDescriptiveName(EObject o) {
        var name = ""
        switch o {
            State:
                name = o.generatorElement?.getName(options.useFullNames, "/")
            Event:
                name = o.generatorElement?.getName(options.useFullNames)
            Action:
                name = o.generatorElement?.getName(options.useFullNames)
            Transition:
                name = o.generatorElement?.getName(options.useFullNames)
            default:
                name = "generated_name"
        }
        return name
    }
}
