package uw.watform.dash.generator.elements

import org.eclipse.emf.ecore.EObject
import org.eclipse.xtend.lib.annotations.Accessors
import uw.watform.dash.dash.State

abstract class GeneratorElement {
    protected EObject node
    val String NAME_PREFIX = "dsh"

    @Accessors String shortName
    @Accessors String longName

    State originalParentState

    new (EObject n) {
        this(n, 0)
    }

    new (EObject n, int generatorCounter) {
        this.node = n
        this.shortName = getShortName(getPrefix(), generatorCounter)
        this.originalParentState = parentState
    }

    protected def getPrefix() {
        NAME_PREFIX
    }

    /**
     * Short names are used to generate the Alloy code
     */
    protected def String getShortName(String prefix, int counter) {
        prefix + "_" + counter
    }

    def String getName(boolean fullName, String delimiter) {
        if (fullName && longName !== null) {
            return longName.replace(".", delimiter)
        } else {
            return shortName
        }
    }

    def String getName(boolean fullName) {
        getName(fullName, "_")
    }

    def getParentState() {
        getParentState(node)
    }

    def getParentState(EObject obj) {
        for (var container = obj.eContainer; container !== null; container = container.eContainer) {
            if (container instanceof State) {
                return container
            }
        }
        return null
    }

    def getOriginalParentState() {
        return originalParentState
    }

    abstract def EObject getNode()
}
