package uw.watform.dash.generator.elements

import com.google.inject.Singleton
import org.eclipse.emf.ecore.EObject
import java.util.Map
import java.util.HashMap
import uw.watform.dash.dash.State
import uw.watform.dash.dash.Transition
import uw.watform.dash.dash.Event
import java.util.Arrays
import uw.watform.dash.dash.Action
import uw.watform.dash.dash.InitDecl
import uw.watform.dash.dash.Condition
import uw.watform.dash.dash.InvariantDecl
import uw.watform.dash.dash.StateVar
import uw.watform.dash.dash.ExitDecl
import uw.watform.dash.dash.EnterDecl
import org.eclipse.xtext.EcoreUtil2
import java.util.ArrayList
import java.util.List

@Singleton
class Helper {

    Map<EObject, GeneratorElement> elements = new HashMap<EObject, GeneratorElement>
    Map<String, Integer> generatorCounters = new HashMap<String, Integer>

    def clear() {
        elements.clear
        generatorCounters.clear
    }

    def dispatch StateGeneratorElement toGeneratorElement(State s) {
        var StateGeneratorElement e = elements.get(s) as StateGeneratorElement
        if (e === null) {
            e = new StateGeneratorElement(s, getElementNumber(s))
            elements.put(s, e)
        }
        e
    }

    def dispatch TransitionGeneratorElement toGeneratorElement(Transition t) {
        var TransitionGeneratorElement e = elements.get(t) as TransitionGeneratorElement
        if (e === null) {
            e = new TransitionGeneratorElement(t, getElementNumber(t))
            elements.put(t, e)
        }
        e
    }

    def dispatch EventGeneratorElement toGeneratorElement(Event ev) {
        var EventGeneratorElement e = elements.get(ev) as EventGeneratorElement
        if (e === null) {
            e = new EventGeneratorElement(ev, getElementNumber(ev))
            elements.put(ev, e)
        }
        e
    }

    def dispatch ActionGeneratorElement toGeneratorElement(Action a) {
        var ActionGeneratorElement e = elements.get(a) as ActionGeneratorElement
        if (e === null) {
            e = new ActionGeneratorElement(a, getElementNumber(a))
            elements.put(a, e)
        }
        e
    }

    def dispatch ConditionGeneratorElement toGeneratorElement(Condition c) {
        var ConditionGeneratorElement e = elements.get(c) as ConditionGeneratorElement
        if (e === null) {
            e = new ConditionGeneratorElement(c, getElementNumber(c))
            elements.put(c, e)
        }
        e
    }

    def dispatch InvariantGeneratorElement toGeneratorElement(InvariantDecl i) {
        var InvariantGeneratorElement e = elements.get(i) as InvariantGeneratorElement
        if (e === null) {
            e = new InvariantGeneratorElement(i, getElementNumber(i))
            elements.put(i, e)
        }
        e
    }

    def dispatch InitGeneratorElement toGeneratorElement(InitDecl i) {
        var InitGeneratorElement e = elements.get(i) as InitGeneratorElement
        if (e === null) {
            e = new InitGeneratorElement(i, getElementNumber(i))
            elements.put(i, e)
        }
        e
    }

    def dispatch StateVarGeneratorElement toGeneratorElement(StateVar sv) {
        var StateVarGeneratorElement e = elements.get(sv) as StateVarGeneratorElement
        if (e === null) {
            e = new StateVarGeneratorElement(sv, getElementNumber(sv))
            elements.put(sv, e)
        }
        e
    }

    def dispatch ExitDeclGeneratorElement toGeneratorElement(ExitDecl ed) {
        var ExitDeclGeneratorElement e = elements.get(ed) as ExitDeclGeneratorElement
        if (e === null) {
            e = new ExitDeclGeneratorElement(ed, getElementNumber(ed))
            elements.put(ed, e)
        }
        e
    }

    def dispatch EnterDeclGeneratorElement toGeneratorElement(EnterDecl ed) {
        var EnterDeclGeneratorElement e = elements.get(ed) as EnterDeclGeneratorElement
        if (e === null) {
            e = new EnterDeclGeneratorElement(ed, getElementNumber(ed))
            elements.put(ed, e)
        }
        e
    }

    def dispatch GeneratorElement toGeneratorElement(EObject o) {
        throw new IllegalArgumentException("Unhandled parameter types: " +
            Arrays.<Object>asList(o).toString());
    }

    def getGeneratorElements() {
        return this.elements
    }

    private def getElementNumber(EObject o) {
        var counter = generatorCounters.get(o.class.name)
        if (counter === null) {
            counter = 0
        }
        counter = counter.intValue
        counter++
        generatorCounters.put(o.class.name, new Integer(counter))
        counter
    }

    static def ArrayList<State> getSiblings(State s) {
        var siblings =  new ArrayList<State>()
        val parentState = EcoreUtil2.getContainerOfType(s.eContainer, State)
        if (parentState !== null) {
            siblings.addAll(
                parentState.stateItems.filter(State).reject[substate | substate == s]
            )
        }
        siblings
    }

    static def List<State> getEnteredChildrenStates(State s) {
        var enteredStates = new ArrayList<State>()
        enteredStates.add(s)
        val childrenStates = s.stateItems.filter(State)
        for (subState : childrenStates) {
            if (subState.isDefault || subState.isConcurrent) {
                enteredStates.addAll(getEnteredChildrenStates(subState))
            }
        }
        enteredStates
    }

    static def boolean contains(State s1, State s2) {
        if (s1 === null || s2 === null) {
            return false
        }
        s1 === s2 || strictlyContains(s1, s2)
    }

    static def boolean strictlyContains(State s1, State s2) {
        var contents = EcoreUtil2.getAllContentsOfType(s1, State)
        contents.contains(s2)
    }
}
