package uw.watform.dash.scoping

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.naming.DefaultDeclarativeQualifiedNameProvider
import org.eclipse.xtext.naming.QualifiedName
import org.eclipse.xtext.util.Strings
import org.eclipse.xtext.util.IResourceScopeCache
import org.eclipse.xtext.util.Tuples

import com.google.inject.Provider;
import com.google.inject.Inject

import uw.watform.dash.dash.State

/**
 * Calculates the fully qualified name of an EObject by following the state
 * hierarchy
 *
 * @author Jose Serna
 */
class DashQualifiedNameProvider extends DefaultDeclarativeQualifiedNameProvider {
    @Inject
    DashQualifiedNameConverter converter
    @Inject
    IResourceScopeCache cache = IResourceScopeCache.NullImpl.INSTANCE;

    /**
     * Returns the qualified name based on state hierarchy.
     * The fully qualified name of a system element is the sequence of states in
     * the hierarchy.
     *
     * @param the object to calculate the qualified name
     * @return the fully qualified name of obj
     */
    override QualifiedName getFullyQualifiedName(EObject obj) {
        return cache.get(Tuples.pair(obj, "fqn"), obj.eResource(), new Provider<QualifiedName>() {
            override QualifiedName get() {
                var temp = obj
                val name = getResolver().apply(temp);
                if (Strings.isEmpty(name))
                    return null;
                val qualifiedNameFromConverter = converter.toQualifiedName(name);
                while (temp.eContainer() !== null) {
                    temp = temp.eContainer();
                    if (temp instanceof State) {
                        val parentsQualifiedName = getFullyQualifiedName(temp);
                        if (parentsQualifiedName !== null)
                            return parentsQualifiedName.append(qualifiedNameFromConverter);
                    }
                }
                return qualifiedNameFromConverter;
            }
        });
    }
}
