/*
 * generated by Xtext 2.10.0
 */
package uw.watform.dash.validation

import org.eclipse.xtext.validation.Check
import uw.watform.dash.dash.Expr
import org.eclipse.emf.ecore.EReference
import uw.watform.dash.dash.Negation
import uw.watform.dash.dash.DashPackage
import com.google.inject.Inject
import uw.watform.dash.dash.Conjunction
import uw.watform.dash.dash.Comparison
import uw.watform.dash.dash.ElementRef
import uw.watform.dash.dash.Condition
import uw.watform.dash.dash.Action
import uw.watform.dash.dash.Disjunction
import uw.watform.dash.dash.Implication
import uw.watform.dash.dash.Cardinality
import uw.watform.dash.dash.UnionOrDiff
import uw.watform.dash.dash.DotJoin
import uw.watform.dash.dash.Transpose
import uw.watform.dash.dash.Intersection
import uw.watform.dash.dash.DomainRestriction
import uw.watform.dash.dash.RangeRestriction
import uw.watform.dash.dash.Override
import uw.watform.dash.dash.Box
import org.eclipse.emf.ecore.EAttribute
import uw.watform.dash.dash.Import
import uw.watform.dash.scoping.DashImportUriResolver
import uw.watform.dash.dash.TransBody
import uw.watform.dash.dash.TransTemplate
import org.eclipse.emf.common.util.EList
import uw.watform.dash.dash.Event
import uw.watform.dash.dash.PredDecl
import uw.watform.dash.dash.Function
import uw.watform.dash.dash.FactDecl
import uw.watform.dash.validation.type.ExpressionType
import uw.watform.dash.validation.type.UnionType
import uw.watform.dash.validation.type.EmptyType
import java.util.List
import java.util.ArrayList
import uw.watform.dash.dash.TransSet
import org.eclipse.emf.ecore.EObject
import uw.watform.dash.dash.Escape
import uw.watform.dash.dash.StateVar
import uw.watform.dash.dash.Declaration
import uw.watform.dash.dash.Prime
import uw.watform.dash.dash.Element
import uw.watform.dash.dash.SigRef
import uw.watform.dash.dash.SignatureRef
import uw.watform.dash.dash.Signature
import uw.watform.dash.dash.CTLBinaryOperation
import org.eclipse.xtext.EcoreUtil2
import uw.watform.dash.dash.AssertDecl
import uw.watform.dash.dash.ExitDecl
import uw.watform.dash.dash.EnterDecl

/**
 * This class contains custom validation rules. 
 *
 * See https://www.eclipse.org/Xtext/documentation/303_runtime_concepts.html#validation
 */
class DashValidator extends AbstractDashValidator {

    protected static val ISSUE_CODE_PREFIX = "uw.watform.dash.";

    public static val TYPE_MISMATCH = ISSUE_CODE_PREFIX + "TypeMismatch"

    public static val UNEXPECTED_OPERATION = ISSUE_CODE_PREFIX + "UnexpectedOperation"

    public static val UNKNOWN_MODULE = ISSUE_CODE_PREFIX + "UnknownModule"

    public static val WELL_FORMEDNESS_CONSTRAINT = ISSUE_CODE_PREFIX + "WellFormednessConstraint"

    // Error messages
    static val EXPECTED_SAME_TYPE = "Expected the same type. \nLeft hand side: %s \nRight hand side: %s"

    static val UNEXPECTED_TYPE = "Expected type %s but was given %s"

    static val INVALID_DIFF_UNION = "Operator %s can only be used between expressions of the same arity or %s type. \nLeft hand side: %s \nRight hand side: %s"

    static val NO_COMMON_JOIN = "There are no common elements to join the expressions. \nLeft hand side: %s \nRight hand side: %s"

    static val ARITY_MISMATCH = "Expected relations with the same arity. \nLeft hand side: %s \nRight hand side: %s"

    static val EXPECTED_UNARY = "Expected unary relation but was given %s"

    static val EMPTY_RESULT = "The result is empty"

    static val DISJOINT_INTERSECTION = "The expressions are disjoint. The intersection is always empty. \nLeft hand side: %s \nRight hand side: %s"

    static val NULL_TYPE = "Null type"

    static val UNRECOGNIZED_MODULE = "Unrecognized module %s"

    static val WRONG_NUMBER_PARAMETERS = "Wrong number of parameters \nexpected %d but was given %d"

    static val UNRECOGNIZED_TEMPLATE = "Couldn't resolve reference to Transition Template '%s'"

    static val ONE_DEFAULT_STATE_REQUIRED = "There must be one default child state defined"

    static val ALL_SUBSTATES_CONCURRENT = "Either all children states at the same level of the hierarchy are concurrent or none at all"

    static val ESCAPED_CONTENT = "Escaped content is not validated"

    static val STATE_VAR_EXPECTED = "Only state declarations can be primed"

    static val ENV_EVENT_GENERATED = "Trying to generate environmental event '%s'"

    static val ENV_VARIABLE_NEXT_VALUE = "Next value of environmental variable '%s' cannot be constrained"

    static val UNKNOWN_SIG = "Signature '%s' cannot be found"

    static val UNEXPECTED_TEMPORAL_OPERATION = "Temporal expressions are only allowed as part of pred and assert blocks";

    @Inject extension TypeComputer

    @Inject
    DashImportUriResolver uriResolver

    def private checkExpectedBoolean(Expr exp, EReference reference) {
        checkExpectedBoolean(exp, reference, 0)
    }

    def private checkExpectedBoolean(Expr exp, EReference reference, int index) {
        checkExpectedType(exp, TypeComputer.BOOL_TYPE, reference, index)
    }

    def private checkExpectedType(Expr exp, ExpressionType expectedType, EReference reference, int index) {
        val actualType = getTypeAndCheckNotNull(exp, reference)
        if (actualType !== null && actualType != expectedType)
            error(String.format(UNEXPECTED_TYPE, expectedType.toString(), actualType), reference, index, TYPE_MISMATCH)
    }

    def private ExpressionType getTypeAndCheckNotNull(Expr exp, EReference reference) {
        var type = exp?.typeFor
        // Commented to avoid erros due to computation of types.
        // Uncomment to help in the development of the type checker
        // checkNotNull(type, reference)
        return type;
    }

    @Check
    def alertEscapedContent(Escape e) {
        warning(ESCAPED_CONTENT, DashPackage.Literals.ESCAPE.getEIDAttribute(), WELL_FORMEDNESS_CONSTRAINT)
    }

    @Check
    def checkStateHierarchyConstraints(uw.watform.dash.dash.State s) {
        var childrenStates = getChildrenStates(s)
        if (childrenStates.empty) {
            return
        }

        var concurrentStates = childrenStates.filter[ st | st.concurrent]
        if (concurrentStates.empty) {
            // Check default states
            val defaultStates = childrenStates.filter[ st | st.^default]
            if (defaultStates.empty || defaultStates.length > 1) {
                error(ONE_DEFAULT_STATE_REQUIRED,
                    DashPackage.Literals.STATE__DEFAULT,
                    WELL_FORMEDNESS_CONSTRAINT
                )
            }
        } else {
            if (concurrentStates.length != childrenStates.length) {
                error(ALL_SUBSTATES_CONCURRENT,
                    DashPackage.Literals.STATE__CONCURRENT,
                    WELL_FORMEDNESS_CONSTRAINT
                )
            }
        }
    }

    def private List<uw.watform.dash.dash.State> getChildrenStates(EObject o) {

        val childrenStates = new ArrayList<uw.watform.dash.dash.State>
        var Iterable<TransSet>  itemsList = null

        if (o instanceof Event) {
            itemsList = o.body.trans
        }

        if (o instanceof Condition) {
            itemsList = o.body.trans
        }

        // Base case when o is the root State element
        if (o instanceof uw.watform.dash.dash.State) {
            itemsList = o.stateItems.filter[item |
                item instanceof TransSet
            ].map[item | item as TransSet]
        }

        itemsList?.forEach[ item |
            if (!(item instanceof uw.watform.dash.dash.State)) {
                childrenStates.addAll(getChildrenStates(item))
            } else {
                childrenStates.add(item as uw.watform.dash.dash.State)
            }
        ]
        childrenStates
    }

    @Check def checkType(Negation not) {
        checkExpectedBoolean(not.expression, DashPackage.Literals.NEGATION__EXPRESSION)
    }

    @Check def checkType(Conjunction and) {
        checkExpectedBoolean(and.left, DashPackage.Literals.CONJUNCTION__LEFT)
        checkExpectedBoolean(and.right, DashPackage.Literals.CONJUNCTION__RIGHT)
    }

    @Check def checkType(Disjunction or) {
        checkExpectedBoolean(or.left, DashPackage.Literals.DISJUNCTION__LEFT)
        checkExpectedBoolean(or.right, DashPackage.Literals.DISJUNCTION__RIGHT)
    }

    @Check def checkType(Implication imp) {
        if (imp.getElse === null) {
            checkExpectedBoolean(imp.left, DashPackage.Literals.IMPLICATION__LEFT)
            checkExpectedBoolean(imp.right, DashPackage.Literals.IMPLICATION__RIGHT)
        } else {
            // TODO validate branches of conditional,
            // they should be the same type or none
        }
    }

    @Check def checkType(ElementRef ref) {
        if (ref.ref.equals(Condition) || ref.ref.equals(Action)) {
            checkExpectedBoolean(ref, DashPackage.Literals.ELEMENT_REF__REF)
        }
    }

    @Check
    def checkType(Comparison comparison) {
        val leftType = getTypeAndCheckNotNull(comparison.left, DashPackage.Literals.COMPARISON__LEFT)
        val rightType = getTypeAndCheckNotNull(comparison.right, DashPackage.Literals.COMPARISON__RIGHT)

        if (comparison.op == "=" || comparison.op == "in") {
            if (leftType instanceof UnionType && rightType instanceof UnionType) {
                checkSameArity(leftType, rightType, DashPackage.Literals.COMPARISON.getEIDAttribute())
            } else {
                checkSubset(leftType, rightType, DashPackage.Literals.COMPARISON.getEIDAttribute())
            }
        } else {
            checkExpectedInt(leftType, DashPackage.Literals.COMPARISON__LEFT)
            checkExpectedInt(rightType, DashPackage.Literals.COMPARISON__RIGHT)
        }
    }

    @Check
    def checkType(Cardinality c){
        val type = getTypeAndCheckNotNull(c.expression, DashPackage.Literals.CARDINALITY__EXPRESSION)
        checkExpectedRelation(type, DashPackage.Literals.CARDINALITY__EXPRESSION)
    }

    @Check
    def checkType(UnionOrDiff u){
        var leftType = getTypeAndCheckNotNull(u.left, DashPackage.Literals.UNION_OR_DIFF__LEFT)
        var rightType = getTypeAndCheckNotNull(u.right, DashPackage.Literals.UNION_OR_DIFF__RIGHT)

        // The expected type of the expression is based on the type of the left element
        if (leftType.isIntType) {
            checkExpectedInt(rightType, DashPackage.Literals.UNION_OR_DIFF__RIGHT)
        } else if (leftType.isRelationalType) {
            checkSameArity(leftType, rightType, DashPackage.Literals.UNION_OR_DIFF.getEIDAttribute())
        } else if (leftType != TypeComputer.STATE_TYPE && rightType != TypeComputer.STATE_TYPE){
            error(String.format(INVALID_DIFF_UNION,
                u.op,
                TypeComputer.INT_TYPE,
                leftType, rightType),
                DashPackage.Literals.UNION_OR_DIFF.getEIDAttribute(), TYPE_MISMATCH
            )
        }
    }

    @Check
    def checkType(DotJoin e){
        var leftType = getTypeAndCheckNotNull(e.left, DashPackage.Literals.DOT_JOIN__LEFT)
        var rightType = getTypeAndCheckNotNull(e.right, DashPackage.Literals.DOT_JOIN__RIGHT)
        var isJoinable = checkJoinable(leftType, rightType, DashPackage.Literals.DOT_JOIN.EIDAttribute)
        if (isJoinable && e.typeFor.isEmptyType){
            warning(String.format(EMPTY_RESULT),
                DashPackage.Literals.DOT_JOIN.getEIDAttribute(), TYPE_MISMATCH)
        }
    }

    @Check
    def checkType(Transpose e){
        getTypeAndCheckNotNull(e.expression, DashPackage.Literals.TRANSPOSE__EXPRESSION)
    }

    @Check
    def checkType(Intersection e) {
        var leftType = getTypeAndCheckNotNull(e.left, DashPackage.Literals.INTERSECTION__LEFT)
        var rightType = getTypeAndCheckNotNull(e.right, DashPackage.Literals.INTERSECTION__RIGHT)
        var isRelationLeft  = checkExpectedRelation(leftType, DashPackage.Literals.INTERSECTION__LEFT)
        var isRelationRight = checkExpectedRelation(rightType, DashPackage.Literals.INTERSECTION__RIGHT)

        if (isRelationLeft && isRelationRight) {
            var sameArity = checkSameArity(leftType, rightType, DashPackage.Literals.INTERSECTION.EIDAttribute)
            // Only check intersection if both expressions have same arity
            if (sameArity) {
                var res = (leftType as UnionType ).intersect((rightType as UnionType))
                if (res.isEmpty()) {
                    warning(String.format(DISJOINT_INTERSECTION, leftType, rightType),
                        DashPackage.Literals.INTERSECTION.getEIDAttribute(), TYPE_MISMATCH)
                }
            }
        }
    }

    @Check
    def checkType(DomainRestriction e){
        val typeLeft =  getTypeAndCheckNotNull(e.left, DashPackage.Literals.DOMAIN_RESTRICTION__LEFT)
        if (typeLeft !== null && !typeLeft.isUnary)
            error(String.format(EXPECTED_UNARY, typeLeft),
                DashPackage.Literals.DOMAIN_RESTRICTION__LEFT, TYPE_MISMATCH)
    }

    @Check
    def checkType (RangeRestriction e) {
        val typeRight =  getTypeAndCheckNotNull(e.right, DashPackage.Literals.RANGE_RESTRICTION__RIGHT)
        if (typeRight !== null && !typeRight.isUnary) {
            error(String.format(EXPECTED_UNARY, typeRight),
                DashPackage.Literals.RANGE_RESTRICTION__RIGHT, TYPE_MISMATCH)
        }
    }

    @Check
    def checkType(Override e) {
        var leftType = getTypeAndCheckNotNull(e.left, DashPackage.Literals.OVERRIDE__LEFT)
        var rightType = getTypeAndCheckNotNull(e.right, DashPackage.Literals.OVERRIDE__RIGHT)
        // Perform additional checks
        // Operands must have same arity
        checkSameArity(leftType, rightType, DashPackage.Literals.OVERRIDE.EIDAttribute)
    }

    @Check
    /**
     * Box expressions can refer to either the invocation of a predicate,
     * function or template, or they can be used as alternative to join operator
     *  in a manner similar to arrays
     */
    def checkType(Box e){
        var leftType = getTypeAndCheckNotNull(e.left, DashPackage.Literals.BOX__LEFT)
        var args = e.args
        var container = e.eContainer

        if (container instanceof TransBody || container instanceof TransTemplate) {
            checkTransTemplateParameters(e)
        } else {
            // Check when used as array syntax to access elements
            for (var i=0; i< args.length; i++){
                var arg = getTypeAndCheckNotNull(args.get(i), DashPackage.Literals.BOX__ARGS)
                checkJoinable(arg, leftType, DashPackage.Literals.BOX.EIDAttribute)
                leftType = (arg as UnionType).join(leftType as UnionType)
            }
        }
    }

    def private checkTransTemplateParameters(Box e) {
        var EList<Declaration>  formalParameters = null
        var args = e.args

        if (e.left instanceof ElementRef) {
            // Ceck parameters
            var ref = (e.left as ElementRef).ref

            if (ref instanceof PredDecl) {
                formalParameters = ref.params
            }

            if (ref instanceof Function) {
                formalParameters = ref.params
            }

            if (ref instanceof TransTemplate) {
                formalParameters = ref.params
            }

            if (formalParameters !== null) {
                var qtyFormalArgs = formalParameters.fold(0)[result, param | result + param.names.length]
                if (qtyFormalArgs > args.length) {
                    error(String.format(WRONG_NUMBER_PARAMETERS, qtyFormalArgs, args.length),
                        DashPackage.Literals.BOX__ARGS, args.length - qtyFormalArgs, TYPE_MISMATCH
                    )
                }

                val List<Declaration> expandedParameters = new ArrayList<Declaration>
                formalParameters.forEach[ p | p.names.forEach[ name | expandedParameters.add(p)]]

                for (var i=0; i< args.length; i++){
                    var arg = getTypeAndCheckNotNull(args.get(i), DashPackage.Literals.BOX__ARGS)
                    if (i > qtyFormalArgs - 1) {
                        error(String.format(WRONG_NUMBER_PARAMETERS,
                                qtyFormalArgs,
                                args.length
                            ),
                            DashPackage.Literals.BOX__ARGS, i, TYPE_MISMATCH
                        )
                    } else {
                        var formalType = getTypeAndCheckNotNull(expandedParameters.get(i).type, DashPackage.Literals.BOX__ARGS)
                        if (formalType !== null && !formalType.class.isAssignableFrom(arg.class)) {
                            error(String.format(UNEXPECTED_TYPE,
                                    formalType.toString(),
                                    arg
                                ),
                                DashPackage.Literals.BOX__ARGS, i, TYPE_MISMATCH
                            )
                        }

                        if (!arg.isPrimitiveType || arg.isAbstractPrimitiveType) {
                            error(String.format(UNEXPECTED_TYPE,
                                TypeComputer.BOOL_TYPE + " or " + TypeComputer.EVENT_TYPE +
                                 " or " + TypeComputer.STATE_TYPE,
                                  arg
                                ),
                                DashPackage.Literals.BOX__ARGS,
                                i,
                                 TYPE_MISMATCH
                             )
                        }
                    }
                }
            }
        }
    }

    /**
     * Validates modules imported by users.
     * Validates that the module is one of the standard modules shipped with Alloy,
     * or that there exists a file in the path indicated by the module import
     */
    @Check
    def checkImport(Import modImport) {
        var moduleName = modImport.importURI
        if (!uriResolver.isUtilModule(moduleName) && uriResolver.apply(modImport) === null) {
            error(String.format(UNRECOGNIZED_MODULE, modImport.importURI),
                DashPackage.Literals.IMPORT__IMPORT_URI, UNKNOWN_MODULE
            )
        }
    }

    @Check
    def checkType(TransBody e) {
        if (e.params !== null) {
            var params = e.params as Box
            var left = params.left

            if (!(left instanceof ElementRef)) {
                error(String.format(UNRECOGNIZED_TEMPLATE, left), DashPackage.Literals.TRANS_BODY__PARAMS, TYPE_MISMATCH)
            } else {
                var reference = (left as ElementRef)
                if (!(reference.ref instanceof TransTemplate)){
                    error(String.format(UNRECOGNIZED_TEMPLATE, reference.getName()), DashPackage.Literals.TRANS_BODY__PARAMS, TYPE_MISMATCH)
                }
            }
        }

        // Check expressions
        for (var i = 0; i < e.conditions.length; i++) {
            var type = getTypeAndCheckNotNull(e.conditions.get(i), DashPackage.Literals.TRANS_BODY__CONDITIONS)
            if (type!== null && !type.isBooleanType) {
                error(String.format(UNEXPECTED_TYPE,
                     TypeComputer.ABSTRACT_BOOL_TYPE + " or " + TypeComputer.BOOL_TYPE,
                      type
                    ),
                    DashPackage.Literals.TRANS_BODY__CONDITIONS,
                    i,
                     TYPE_MISMATCH
                 )
            }
        }

        for (var i = 0; i < e.actions.length; i++) {
            var type = getTypeAndCheckNotNull(e.actions.get(i), DashPackage.Literals.TRANS_BODY__ACTIONS)
            if (type !== null && !type.isBooleanType) {
                error(String.format(UNEXPECTED_TYPE,
                     TypeComputer.ABSTRACT_BOOL_TYPE + " or " + TypeComputer.BOOL_TYPE,
                      type
                    ),
                    DashPackage.Literals.TRANS_BODY__ACTIONS,
                    i,
                     TYPE_MISMATCH
                 )
            }
        }
    }

    @Check
    def checkType(TransTemplate t) {
        for (var i = 0; i < t.params.length; i++) {
            var type = getTypeAndCheckNotNull(t.params.get(i).type, DashPackage.Literals.BOX__ARGS)
            if (type !== null && !type.isAbstractPrimitiveType) {
                error(String.format(UNEXPECTED_TYPE,
                     TypeComputer.ABSTRACT_BOOL_TYPE + " or " + TypeComputer.ABSTRACT_EVENT_TYPE +
                     " or " + TypeComputer.ABSTRACT_STATE_TYPE,
                      type
                    ),
                    DashPackage.Literals.PARAMETERS__PARAMS,
                    i,
                     TYPE_MISMATCH
                 )
            }
        }
    }

    @Check
    def checkType(Prime e) {
        if (!(e.stateVar.eContainer.eContainer instanceof StateVar)) {
            error(STATE_VAR_EXPECTED, DashPackage.Literals.PRIME__STATE_VAR, TYPE_MISMATCH)
        }
    }

    @Check
    def checkEnvironmentVariable(Prime e) {
        var declaration = e.stateVar.eContainer.eContainer
        if ((declaration instanceof StateVar)) {
            if (declaration.input) {
               error(String.format(ENV_VARIABLE_NEXT_VALUE, e.stateVar.name),
                   DashPackage.Literals.PRIME__STATE_VAR,
                   WELL_FORMEDNESS_CONSTRAINT
               )
            }
        }
    }

    @Check
    def checkGeneratedEvents(TransBody tbody) {
        var genEvents = tbody.genEvents
        if (genEvents !== null) {
            genEvents.names.forEach[ e, i |
                if (e instanceof Event && (e as Event).input) {
                    warning(String.format(ENV_EVENT_GENERATED, (e as Event).name),
                        DashPackage.Literals.TRANS_BODY__GEN_EVENTS,
                        i,
                        WELL_FORMEDNESS_CONSTRAINT
                    )
                }
            ]
        }
    }

    @Check
    def checkType(SigRef sigRef) {
        if (sigRef instanceof SignatureRef) {
            if (!(sigRef.name.eContainer instanceof Signature)) {
                error(String.format(UNKNOWN_SIG, sigRef.name.name),
                    DashPackage.Literals.SIG_REF.getEIDAttribute(),
                    TYPE_MISMATCH
                )
            }
        }
    }

    @Check
    def checkUnexpectedOperation(CTLBinaryOperation e) {
        val isUnexpectedOperation = EcoreUtil2.getAllContainers(e).filter[
            c | c instanceof AssertDecl || c instanceof PredDecl
        ].empty

        if (isUnexpectedOperation) {
            error(UNEXPECTED_TEMPORAL_OPERATION,
                DashPackage.Literals.CTL_BINARY_OPERATION.getEIDAttribute(),
                UNEXPECTED_OPERATION
            )
        }
    }

    @Check
    def checkUnconstrainedEnvironemntalRef(ElementRef ref) {
        val referenceContainer = ref.ref.eContainer.eContainer
        var isInput = false
        if (referenceContainer instanceof StateVar) {
            isInput = referenceContainer.isInput
        }
        if (
            isInput &&
            (EcoreUtil2.getContainerOfType(ref, ExitDecl) !== null ||
            EcoreUtil2.getContainerOfType(ref, EnterDecl) !== null)
        ){
           error(String.format(ENV_VARIABLE_NEXT_VALUE, (ref.ref as Element).name),
               DashPackage.Literals.REF.EIDAttribute,
               WELL_FORMEDNESS_CONSTRAINT
           )
        }
    }

    def private getName(ElementRef reference) {
        var name = ""
        var ref = reference.ref
        switch (ref) {
            Action, Condition, Event, TransTemplate: name = ref.name
            Function: name = ref.name
            FactDecl: name = ref.name
            PredDecl: name = ref.name
            uw.watform.dash.dash.State: name = ref.name
            Element: name = ref.name
        }
        return name
    }

    def private checkSubset(ExpressionType left, ExpressionType right, EAttribute attribute) {
        if (right !== null && left !== null && (right as UnionType).isSubset((left as UnionType)) == EmptyType.getInstance()) {
            error(String.format(EXPECTED_SAME_TYPE, left, right),
                attribute,
                TYPE_MISMATCH
            )
        }
    }

    def private checkExpectedInt(ExpressionType type, EReference reference) {
        if (type !== null && !type.isIntType) {
            error(String.format(UNEXPECTED_TYPE, TypeComputer.INT_TYPE, type), reference, TYPE_MISMATCH)
        }
    }

    def private boolean checkSameArity(ExpressionType left, ExpressionType right, EAttribute attribute) {
        var mismatch = false;
        if (left.isRelationalType) {
            mismatch = !(left as UnionType).getArities().equals((right as UnionType).getArities())
        } else {
            mismatch = (left.arity != right.arity)
        }

        if (mismatch) {
            error(String.format(ARITY_MISMATCH, left, right), attribute, TYPE_MISMATCH)
        }
        !mismatch
    }

    def private boolean checkJoinable(ExpressionType typeLeft, ExpressionType typeRight, EAttribute attribute) {
        var boolean isJoinable = false
        if (!(typeLeft as UnionType).isJoinable((typeRight as UnionType))) {
            error(String.format(NO_COMMON_JOIN, typeLeft, typeRight),
                attribute, TYPE_MISMATCH)
            isJoinable = false
        } else {
            isJoinable = true
        }
        return isJoinable
    }

    def checkNotNull(ExpressionType type,  EReference reference) {
        if (type === null) {
            error(NULL_TYPE, reference, TYPE_MISMATCH)
        }
    }

    def boolean checkExpectedRelation(ExpressionType type, EReference reference) {
        var isRelation = type.isRelationalType
        if (type !== null && !isRelation) {
            error(String.format(UNEXPECTED_TYPE,TypeComputer.RELATIONAL_TYPE.toLowerCase, type), reference, TYPE_MISMATCH)
        }
        isRelation
    }
}
