package uw.watform.dash.validation;

import uw.watform.dash.validation.type.UnionType
import uw.watform.dash.validation.type.BooleanType
import uw.watform.dash.validation.type.SigType
import uw.watform.dash.validation.type.EmptyType
import uw.watform.dash.validation.type.ExpressionType
import uw.watform.dash.validation.type.AbstractBooleanType
import uw.watform.dash.validation.type.AbstractState
import uw.watform.dash.validation.type.AbstractEvent

import uw.watform.dash.dash.Expr
import uw.watform.dash.dash.Disjunction
import uw.watform.dash.dash.Conjunction
import uw.watform.dash.dash.Negation
import uw.watform.dash.dash.Cardinality
import uw.watform.dash.dash.Implication
import uw.watform.dash.dash.BiImplication
import uw.watform.dash.dash.Action
import uw.watform.dash.dash.Condition
import uw.watform.dash.dash.ElementRef
import uw.watform.dash.dash.Int
import uw.watform.dash.dash.Declaration
import uw.watform.dash.dash.ArrowProduct
import uw.watform.dash.dash.Signature
import uw.watform.dash.dash.Constant
import uw.watform.dash.dash.UnionOrDiff
import uw.watform.dash.dash.DotJoin
import uw.watform.dash.dash.Transpose
import uw.watform.dash.dash.Intersection
import uw.watform.dash.dash.DomainRestriction
import uw.watform.dash.dash.RangeRestriction
import uw.watform.dash.dash.Prime
import uw.watform.dash.dash.Override
import uw.watform.dash.dash.Box
import uw.watform.dash.dash.SigRef
import uw.watform.dash.dash.Comparison
import uw.watform.dash.dash.Event
import uw.watform.dash.dash.State
import uw.watform.dash.dash.StatePrimitiveType
import uw.watform.dash.dash.EventPrimitiveType
import uw.watform.dash.dash.BoolPrimitiveType

import uw.watform.dash.validation.type.PrimitiveType
import uw.watform.dash.dash.StateVar
import uw.watform.dash.dash.SignatureRef
import uw.watform.dash.dash.ConstantRef
import uw.watform.dash.dash.Element
import uw.watform.dash.dash.QuantOrMulti
import uw.watform.dash.dash.CTLBinaryOperation
import uw.watform.dash.dash.CTLUnaryOperation

class TypeComputer {

    static val INT_SIG_KEYWORD = "Int";

    public static val INT_TYPE = new UnionType(new SigType(INT_SIG_KEYWORD))

    static val SEQ_SIG_KEYWORD = "seq/Int"

    public static val SEQ_TYPE = new UnionType(new SigType(SEQ_SIG_KEYWORD))

    static val UNIV_SIG_KEYWORD = "univ"

    public static val UNIV_TYPE = new UnionType(new SigType(UNIV_SIG_KEYWORD))

    public static val BOOL_TYPE = new BooleanType()

    public static val ABSTRACT_BOOL_TYPE = new AbstractBooleanType()

    public static val STATE_TYPE = new uw.watform.dash.validation.type.State()

    public static val ABSTRACT_STATE_TYPE = new AbstractState()

    public static val EVENT_TYPE = new uw.watform.dash.validation.type.Event()

    public static val ABSTRACT_EVENT_TYPE = new AbstractEvent()

    public static val EMPTY_TYPE = new UnionType(EmptyType.getInstance())

    //public static val UNIV_TYPE = new BasicType("univ")

    public static val RELATIONAL_TYPE = "Relation"

    def isBooleanType(ExpressionType type) {
        type instanceof AbstractBooleanType
    }

    def isIntType(ExpressionType type) {
        var result = false
        if (type instanceof UnionType) {
            var t = type.toBasicType();
            result = t.intersect(INT_TYPE.toBasicType()) != EmptyType.getInstance();
        }
        return result
    }

    def isRelationalType(ExpressionType type) {
        type instanceof UnionType
    }

    def isEmptyType(ExpressionType type) {
        (type as UnionType).isEmpty()
    }

    def isPrimitiveType(ExpressionType type) {
        type instanceof PrimitiveType
    }

    def isAbstractPrimitiveType(ExpressionType type) {
        type instanceof PrimitiveType && (type as PrimitiveType).isAbstract
    }

    def dispatch ExpressionType typeFor(Expr e) {
        switch (e) {
            Disjunction: BOOL_TYPE
            Conjunction: BOOL_TYPE
            Negation: BOOL_TYPE
            Implication: BOOL_TYPE
            BiImplication: BOOL_TYPE
            Comparison: BOOL_TYPE
            Condition: BOOL_TYPE
            CTLBinaryOperation: BOOL_TYPE
            CTLUnaryOperation: BOOL_TYPE
            // INT types
            Cardinality: INT_TYPE
            Int : INT_TYPE
        }
    }

    def  dispatch ExpressionType typeFor(Constant c){
        switch(c.value) {
            case "Int" : INT_TYPE
            //case "univ" : UNIV_TYPE
            case "none": EMPTY_TYPE
        }
    }

    def dispatch ExpressionType typeFor(ElementRef e) {
        val ref = e.ref
        switch (ref) {
            Action: BOOL_TYPE
            Condition: BOOL_TYPE
            Event: EVENT_TYPE
            State: STATE_TYPE
            Element: switch ref.eContainer {
                Signature: {
                    var type = getSignatureType(ref.name, ref.eContainer as Signature)
                    type
                }
                Declaration: ref.eContainer.typeFor
                StateVar: (ref.eContainer as StateVar).decl.typeFor
            }
        }
    }

    def dispatch ExpressionType typeFor(Declaration d) {
        var type = d.type.typeFor
        if (d.eContainer instanceof Signature) {
            var sig = d.eContainer as Signature
            var sigsType = new UnionType();
            for (n: sig.names) {
                sigsType = sigsType.union(getSignatureType(n.name, sig))
            }
            type = sigsType.product(type as UnionType)
        }
        type
    }

    def dispatch ExpressionType typeFor(uw.watform.dash.dash.PrimitiveType t) {
        switch (t.value) {
            StatePrimitiveType : ABSTRACT_STATE_TYPE
            EventPrimitiveType: ABSTRACT_EVENT_TYPE
            BoolPrimitiveType: ABSTRACT_BOOL_TYPE
        }
    }

    def UnionType getSignatureType(String name, Signature e){
        val sigExtension = e?.extension
        var SigType sigType  = null
        var ret = new UnionType();
        if (sigExtension !== null) {
            if (sigExtension.in) {
                for (SigRef ref : sigExtension.names) {
                    switch ref {
                        SignatureRef: ret = ret.union(getSignatureType(ref.name.name, (ref.name.eContainer as Signature)))
                        ConstantRef: ret = switch ref.value {
                            case "univ": ret.union(UNIV_TYPE)
                            case "seq/Int": ret.union(SEQ_TYPE)
                            case "Int": ret.union(INT_TYPE)
                        }
                    }
                }
                return ret;
            } else {
                if (sigExtension.extends) {
                    val parentName = getSigRefName(sigExtension.parent)
                    var parentType = getSignatureType(parentName, ((sigExtension.parent as SignatureRef).name.eContainer as Signature))
                    // Parent must be a basic type
                    if (!parentType.isUnary()) {
                        // Throw exception it must be a top level signature
                    }
                    sigType = new SigType(name, parentType.toBasicType())
                }
            }
        } else {
            sigType = new SigType(name)
        }
        return new UnionType(sigType)
    }

    private def String getSigRefName(SigRef sigRef) {
        var String name = null
        switch sigRef {
            SignatureRef: name = sigRef.name.name
            ConstantRef: name = sigRef.value
        }
        return name
    }

    def dispatch ExpressionType typeFor(UnionOrDiff u){
        val leftType = u.left.typeFor
        val rightType = u.right.typeFor
        // TODO allow states to be used like signatures
        if (leftType == STATE_TYPE || rightType == STATE_TYPE) {
            return null
        }
        if (u.op == "+") {
            return (leftType as UnionType).union((rightType as UnionType))
        }
        u.left.typeFor
    }

    def dispatch UnionType typeFor (ArrowProduct arrow) {
        var leftType = arrow.left.typeFor
        var rightType = arrow.right?.typeFor
        return (leftType as UnionType).product((rightType as UnionType))
    }

    def dispatch UnionType typeFor(DotJoin e){
        var leftType = e.left.typeFor
        var rightType = e.right.typeFor
        return (leftType as UnionType).join((rightType as UnionType))

    }

    def dispatch UnionType typeFor(Transpose t) {
        val type = t.expression.typeFor
        return (type as UnionType).transpose();
    }

    def dispatch UnionType typeFor(Intersection e){
        var typeLeft = e.left.typeFor
        var typeRight = e.right.typeFor
        return (typeLeft as UnionType).intersect((typeRight as UnionType))
    }

    def dispatch UnionType typeFor(DomainRestriction e) {
        val typeLeft = e.left.typeFor
        val typeRight = e.right.typeFor
        return (typeLeft as UnionType).domainRestrict((typeRight as UnionType))
    }

    def dispatch UnionType typeFor(RangeRestriction e) {
        val typeLeft = e.left.typeFor
        val typeRight = e.right.typeFor
        return (typeLeft as UnionType).rangeRestrict((typeRight as UnionType));
    }

    def dispatch ExpressionType typeFor(Prime e){
        ((e.stateVar).eContainer as Declaration).type.typeFor
    }

    def dispatch ExpressionType typeFor(StateVar e){
        e.decl.type.typeFor
    }

    def dispatch ExpressionType typeFor(QuantOrMulti e){
        if (e.eContainer instanceof Declaration) {
            // Multiplicity expression
            e.expression.typeFor
        } else {
            // Quantification expression
            BOOL_TYPE
        }
    }

    def dispatch UnionType typeFor(Override e) {
        val typeLeft = e.left.typeFor
        val typeRight = e.right.typeFor
        return (typeLeft as UnionType).union((typeRight as UnionType));
    }

    def dispatch UnionType typeFor(Box e){
        var args = e.args
        var ExpressionType type = e.left.typeFor
        for (ex : args){
            type = (ex.typeFor as UnionType).join((type as UnionType))
        }
        return (type as UnionType)
    }
}
